unit master;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, DB, ADODB, Mask, DBCtrls, StrUtils;

type
  TfmMaster = class(TForm)
    Panel1: TPanel;
    buConnection: TButton;
    buExit: TButton;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    Panel3: TPanel;
    Memo1: TMemo;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    ADOConnection1: TADOConnection;
    buReport: TButton;
    ADOQuery1: TADOQuery;
    ADOConnection2: TADOConnection;
    buClear: TButton;
    Panel4: TPanel;
    buMetadata: TButton;
    buVersion: TButton;
    buSchema: TButton;
    buData: TButton;
    Panel5: TPanel;
    buSummary: TButton;
    buReldate: TButton;
    buSQLScript: TButton;
    buImport: TButton;
    Panel6: TPanel;
    ProgressBar1: TProgressBar;
    SourceData: TADODataSet;
    MasterData: TADODataSet;
    Columndoc: TADODataSet;
    Panel7: TPanel;
    buExportSubset: TButton;
    buSetCaptionLanguage: TButton;
    procedure buExitClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure buConnectionClick(Sender: TObject);
    procedure buReportClick(Sender: TObject);
    procedure buVersionClick(Sender: TObject);
    procedure buMetadataClick(Sender: TObject);
    procedure buClearClick(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure buSchemaClick(Sender: TObject);
    procedure buDataClick(Sender: TObject);
    procedure buReldateClick(Sender: TObject);
    procedure buSummaryClick(Sender: TObject);
    procedure buSQLScriptClick(Sender: TObject);
    procedure buImportClick(Sender: TObject);
    procedure buExportSubsetClick(Sender: TObject);
    procedure buSetCaptionLanguageClick(Sender: TObject);
  private
    DBFilename: string;
    DBVersion: string;
    ErrorReport: TStringList;
    InitialDirectory: string;
    function BuildHint(hs: string): string;
    procedure CheckEmptyRecords(tn,kf: string);
    procedure CheckExtraRefx(prefix: string);
    function CheckFieldOrder: TStringList;
    procedure CheckForeignKey(dt,mt,dk,mk: string);
    procedure CheckGaps(tn,kf: string);
    function CheckKeyFields: TStringList;
    procedure CheckLookupValue(tn,fn,ln,lfn: string);
    function CheckNullFieldSize: TStringList;
    procedure CheckPrimaryKey(tn,fn: string);
    procedure CheckSRC(tn,tp: string);
    procedure CheckValues(tn,fn,fv: string);
    function CompareColumndoc: TStringList;
    function CompareEntities: TStringList;
    function CompareLinks: TStringList;
    function CompareTabledoc: TStringList;
    procedure CreateExportDatabaseRelationships;
    procedure CreateExportDatabaseTablesAndFields;
    procedure DoTheMerge;
    function EmptyDatabaseOK: boolean;
    procedure EnableButtons(b: boolean);
    function GetCommodities(entity,e_key: string;keyno: longint): string;
    function GetCountry(entity,e_key: string;keyno: longint): string;
    function GetDBVersion: string;
    function GetFieldType(ft: TFieldType): string;
    function GetLocation: string;
    function GetNames(entity,e_key: string;keyno: longint): string;
    function KeyValuesFileOK(var kv: TStringList): boolean;
    procedure MergeDataTables(tn: string);
    procedure MergeLookupTables(tn,keys: string);
    function OKToMerge: boolean;
    procedure PopulateRecordCountTable;
    procedure PopulateSummaryTable(entity,e_key,e_table: string);
    procedure PopulateSummaryTables;
    function VerifyConformity: TStringList;
  public
    LangVersion: string;
  end;

var
  fmMaster: TfmMaster;

{
resourcestring
  cpClear = 'Clear report';
  cpConnect = 'Connect';
  cpData = 'Verify data values';
  cpDisconnect = 'Disconnect';
  cpExit = 'Exit';
  cpSQLScript = 'Generate SQL insert script';
  cpExportSubset = 'Export subset database';
  cpImport = 'Import database';
  cpMaster = 'GlobalDB System -- Database Verification Tool';
  cpMetadata = 'Compare metadata against master';
  cpReldate = 'Set database release date';
  cpReport = 'Save report';
  cpSchema = 'Verify schema conformity';
  cpSummary = 'Build summary tables';
  cpVersion = 'Verify database version';
  fsComparing = 'Comparing %s against master database ...';
  fsDatabaseVersionFound = 'Database version: %s';
  fsEmptyRecord = 'Data table "%s" (%s) is an empty record';
  fsErrorsFound = 'Errors found: %d';
  fsExtraneousRefx = 'Looking for extraneous %s records ...';
  fsForeignKeyProblem = 'Detail table "%s", field "%s", master table "%s", ' +
                        'field "%s": foreign key problem.';
  fsFoundExtraneousRefx = 'Table "%s" (%s,%s,%s) mismatches with REFNO in %s.';
  fsFoundGaps = 'Table "%s" (%s): there are gaps in numbering.';
  fsInvalidValues = 'Table "%s", field "%s": there are invalid values.';
  fsLookupValues = 'Table "%s", field "%s" contains values not found in "%s".';
  fsMismatchColumndoc = 'Mismatch of columndoc.%s for field "%s.%s", master: %s / target: %s';
  fsMismatchEntities = 'Mismatch of entities.%s for entity "%s", master: %s / target: %s';
  fsMismatchFieldPosition = 'Mismatch %s.%s at position %d, expected field "%s", found field "%s".';
  fsMismatchFieldSize = 'Mismatch %s.%s, expected field size "%d", found "%d".';
  fsMismatchFieldType = 'Mismatch %s.%s, expected field type "%s", found "%s".';
  fsMismatchLinks = 'Mismatch of links.%s for relationship "%s", master: %s / target: %s';
  fsMismatchSrctp = 'Table "%s" (%s) mismatches with SRCTP ("%s") in refmain.';
  fsMismatchTabledoc = 'Mismatch of tabledoc.%s for table "%s", master: %s / target: %s';
  fsMissingEntityRank = 'Missing entities.ENTITY "%s" / entities.RANK "%s" from target';
  fsMissingField = 'Field "%s" is missing from table "%s".';
  fsMissingLink = 'Relationship "%s" is missing from database.';
  fsMissingPrimaryKey = 'Table "%s" is missing index "PrimaryKey", found "%s".';
  fsMissingTable = 'Table "%s" is missing from database.';
  fsMissingTablename = 'Missing tabledoc.TABLENAME "%s" from target';
  fsMissingTablenameFieldname = 'Missing columndoc.TABLENAME "%s" / columndoc.FIELDNAME "%s" from target';
  fsNullValues = 'Table "%s", field(s) "%s": there are NULL values.';
  fsProblemFieldOrder = 'Problems with columndoc.FIELDORDER for tables "%s".';
  fsProblemFieldSize = 'Problems with columndoc.FIELDSIZE for tables "%s".';
  fsProblemKeyFields = 'Mismatch tabledoc.KEYFIELDS for table "%s", expected key fields "%s", found "%s".';
  fsRecordsAdded = 'Added %d records to %s.';
  fsRecordsAffected = 'Records affected: %d in %d tables.';
  fsRecordsExported = 'Records exported: %d.';
  fsSetReldateTo = 'Setting RELDATE to "%s" ...';
  fsSetReldateWarning = 'Value of RELDATE will be set to "%s".';
  fsTooFewFields = 'Table "%s" has too few fields.';
  fsValuesNotUnique = 'Table "%s", field(s) "%s":  value(s) not unique.';
  fsVerifyDateTime = 'GlobalDB System -- Database verification (%s)';
  htData = 'Verify data values: verifies data values of the database (primary ' +
           'key values, data values not found in corresponding lookup tables, ' +
           'problems with foreign keys, orphaned reference detail records, empty ' +
           'records, gaps in detail record numbering and extraneous reference ' +
           'detail records).';
  htExportSubset = 'Export subset database: produces a subset database from the currently ' +
                   'connected database using a file of DGNO or DEPNO values.';
  htSQLScript = 'Generate SQL insert script: generates SQL insert statements necessary to ' +
                'populate a server based database and saves the script to a file.';
  htImport = 'Import database: imports entire content of a GlobalDB database into the one ' +
             'that is currently connected.';
  htMetadata = 'Compare metadata against master: compares tables "tabledoc" and "columndoc" ' +
               'against a standard (such as the seed GlobalDB empty database).';
  htReldate = 'Set database release date: sets release date for every record in the database.';
  htSchema = 'Verify schema conformity: verifies primary key fields for each table ' +
             'defined in "tabledoc", that each field defined in table "columndoc" ' +
             'is present in the database and that its characteristics (field type, ' +
             'size and position) correspond to the GlobalDB standard.';
  htSummary = 'Build summary tables: builds or rebuilds the summary tables used by GQuery.';
  htVersion = 'Verify database version: looks for the occurrence of string "3.19", ' +
              '"3.20" or "3.21" in field dbversion.DBREL. This is required by GQuery.';
  msBuildSummaryTables = 'Build or rebuild summary tables?';
  msBuildingSummaryTables = 'Building and populating summary tables ...';
  msConformity = 'Verifying conformity of database schema to metadata ...';
  msCreateExportDatabaseRelationships = 'Creating relationships ...';
  msCreateExportDatabaseTablesAndFields = 'Creating tables and fields ...';
  msDatabaseVersion = 'Looking for database version ...';
  msDatabaseVersionNotFound = 'Database version string is missing from dbversion.DBREL';
  msEmptyDatabase = 'Verifying that target database for export is empty ...';
  msEmptyRecords = 'Looking for empty records ...';
  msExportDatabaseNotEmpty = 'Target database for export is not empty.  Operation cancelled.';
  msExportingSubset = 'Exporting subset of current database ...';
  msExporting = 'Exporting data to SQL script ...';
  msExportSubsetCompactDatabase = 'After this operation, you should compact the database.';
  msExportSubsetCompleted = 'Export operation completed.';
  msExportSubsetDeleteExtraRecords = 'Deleting excess records ...';
  msExportSubsetWarning1 = 'A subset database will be created by deleting from the current' + chr(10) +
                           'database all records whose values of %s are not in the key' + chr(10) +
                           'values file.' + chr(10) + chr(10) +
                           'WARNING: the deleted records cannot be recovered.  If you' + chr(10) +
                           'do not have a backup copy of this database, click Cancel' + chr(10) +
                           'now!  If you wish to proceed, click OK.';
  msExportSubsetWarning2 = 'If you really want to continue, click OK, otherwise, click Cancel!';
  msFieldOrder = 'Verifying columndoc.FIELDORDER for invalid values ...';
  msFieldSize = 'Verifying columndoc.FIELDSIZE for missing values ...';
  msFixFieldOrder = 'Cannot proceed with conformity test until columndoc.FIELDORDER problems are fixed!';
  msFixFieldSize = 'Cannot proceed with conformity test until columndoc.FIELDSIZE problems are fixed!';
  msFixKeyFields = 'Cannot proceed with conformity test until primary key problems are fixed!';
  msImporting = 'Merging source database into current database ...';
  msIntroLine1 = 'This software tool is designed to help guarantee the conformity ' +
                 'of a database to the GlobalDB System schema and that data values are valid.';
  msIntroLine2 = 'The buttons on the left make the following verifications:';
  msKeyFields = 'Verifying primary keys against tabledoc.KEYFIELDS ...';
  msKeyValuesFile = 'Getting key values for export database...';
  msKeyValuesFileError = 'Key values file is missing or not usable!';
  msLookingForGaps = 'Looking for gaps in RANK sequences ...';
  msMergeCompleted = 'Import operation completed.';
  msMODULEX = '==> As soon as possible, rename field "MODULEX" to "MODULE" in table "tabpages".';
  msOpenExportSubsetDatabase = 'Open database to be populated';
  msOpenKeyValuesFile = 'Open key values file';
  msOpenMasterDatabase = 'Open master database';
  msOpenSourceDatabase = 'Open source database';
  msOpenTargetDatabase = 'Open database';
  msOperationCancelled = 'Operation cancelled.';
  msOverlappingKeyValues = 'Cannot proceed as there are overlapping primary key values!';
  msSaveSQLScript = 'Save SQL insert script';
  msSetReldate = 'Setting RELDATE ...';
  msSummaryTablesBuilt = 'The summary tables have been (re)built.';
  msSymprefno = 'Table "symsrc", field "SYMPREFNO": there are NULL entries.';
  msVerifyForeignKeys = 'Verifying foreign keys ...';
  msVerifyLookupValues = 'Verifying lookup values ...';
  msVerifyPrimaryKeys = 'Verifying primary key values ...';
  msVerifyReferenceSource = 'Verifying reference source type ...';
  msVerifyValidValues = 'Verifying valid values ...';
}

implementation

uses resources;

{$R *.dfm}

const
// table names for export order. (svc: service; lkp: lookup; dta: data; sry: summary; xtr: extra)
  svc = 'dbversion,entities,links,tabledoc,tabpages,unitcvsn,lookup,columndoc';
  lkp = 'activity,agetsma,altcat,anmethod,appliesto,category,cfeat,commodity,component,' +
        'compstage,concentrat,contctry,ctryhost,ctryrock,datatype,datum,dbcode,depclan,' +
        'depstat,deptype,dfregime,dgstat,dimcat,dimunits,dshape,EPSG_Canada,estsize,farunits,' +
        'formatos,formatsw,genlith,grdunits,hdepset,hform,hostrock,howloc,hseries,hstruc,' +
        'journal,lithcomp,mdepset,medium,metgrade,method,metsig,mineral,minstyl,mseries,' +
        'nametype,newsname,ntsmaps,objdated,objloc,product,proglevel,protolth,provstate,' +
        'realm,rescat,response,rform,rigneous,samptype,settingof,sigtype,sourcerock,srctp,' +
        'stratlevel,tecqual,tectobj,tonunits,tstruc,wgtunits,whatdated,who,yearsum';
  dta = 'refmain,authors,filsrc,mapsrc,newsrc,papsrc,refx,repsrc,svolsrc,symsrc,volsrc,dggeol,' +
        'dgagets,dgaltsig,dgcfeat,dgcom,dgcomment,dgctry,dgctryrx,dgdeptyp,dgdigitized,' +
        'dggeosig,dghostnm,dghostrx,dgignrx,dgmetsig,dgminstl,dgmnrlgy,dgname,dgnts,' +
        'dgprogress,dgradage,dgref,dgsample,dgshape,dgstatus,dgtecset,dgtstruc,dgxlink,' +
        'dgclith,dgcproto,dghform,dghlith,dghproto,dghstrat,dghstruc,dgrefx,dgrform,' +
        'dgrlith,dgsdim,dgsmpgrade,dgsmpwgt,dpgeol,dpagets,dpaltsig,dpcfeat,dpcom,dpcomment,' +
        'dpctry,dpctryrx,dpdeptyp,dpdigitized,dpgeosig,dphostrx,dpignrx,dpmetsig,dpminstl,' +
        'dpmnrlgy,dpname,dpnts,dpprogress,dpradage,dpref,dpsample,dpshape,dpstatus,dptecset,' +
        'dptstruc,dpxlink,dpclith,dpcproto,dphform,dphlith,dphproto,dphstrat,dphstruc,dprefx,' +
        'dprform,dprlith,dpsdim,dpsmpgrade,dpsmpwgt,mine,mineoper,mineowner,minerefx,' +
        'prodmain,prodgrad,prodrefx,prodwgt,resmain,rescut,resgrade,resrefx,reswgt,dgaltmin,' +
        'dgcftmin,dgmetmin,dpaltmin,dpcftmin,dpdg,dpmetmin,dpmine,proddg,proddp,prodmine,' +
        'resdg,resdp,resmine,resadded2,resinres';
  sry = 'gq_recordcount,dg_summary,dp_summary';
  xtr = 'compworldau2004,compcanadaau2004';

function TfmMaster.BuildHint(hs: string): string;
var
  i: integer;
begin
  i := 50;
  while i < Length(hs) do
    begin
      if hs[i] = ' '
        then
          begin
            hs[i] := chr(10);
            i := i + 50;
          end
        else Inc(i);
    end;
  Result := hs;
end;

procedure TfmMaster.CheckEmptyRecords(tn,kf: string);
var
  i,j: integer;
  EmptyRecord: boolean;
  KeyFields: TStringList;
  ks: string;
begin
  KeyFields := TStringList.Create;
  try
    KeyFields.CommaText := kf;
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select * from ' + tn);
        Open;
        if FieldCount > (KeyFields.Count + 3) then
          begin
            while not Eof do
              begin
                i := KeyFields.Count;
                EmptyRecord := True;
                while (i < (FieldCount - 3)) and EmptyRecord do
                  begin
                    if not Fields[i].IsNull then
                      begin
                        if Fields[i].DataType = ftBoolean
                          then if Fields[i].AsBoolean
                                 then EmptyRecord := False
                                 else
                          else EmptyRecord := False;
                      end;
                    inc(i);
                  end;
                if EmptyRecord then
                  begin
                    ks := '';
                    for j := 0 to KeyFields.Count - 1 do
                      begin
                        if j > 0 then
                          ks := ks + ',';
                        ks := ks + Fields[j].AsString;
                      end;
                    ErrorReport.Add(Format(fsEmptyRecord,[tn,ks]));
                  end;
                Next;
              end;
          end;
        Close;
        ProgressBar1.StepIt;
        Self.Update;
      end;
  finally
    KeyFields.Free;
  end;
end;

procedure TfmMaster.CheckExtraRefx(prefix: string);
var
  tn,rn,kn: string;
begin
  if prefix = 'RS'
    then
      begin
        tn := 'resmain';
        rn := 'resrefx';
        kn := 'RESNO';
      end
    else
  if prefix = 'PD'
    then
      begin
        tn := 'prodmain';
        rn := 'prodrefx';
        kn := 'PRODNO';
      end
    else
  if prefix = 'MN'
    then
      begin
        tn := 'mine';
        rn := 'minerefx';
        kn := 'MINENO';
      end
    else ;
  with ADOQuery1 do
    begin
      Close;
      SQL.Clear;
      SQL.Add('select ' + rn + '.' + kn + ',' + rn + '.REFNO,' + rn + '.RANK ');
      SQL.Add('from ' + rn + ' inner join ' + tn + ' on ' + rn + '.' + kn + ' = ' + tn + '.' + kn + ' ');
      SQL.Add('where ' + rn + '.REFNO  <> ' + tn + '.REFNO');
      Open;
      while not Eof do
        begin
          ErrorReport.Add(Format(fsFoundExtraneousRefx,[rn,
                          ADOQuery1.FieldByName(kn).AsString,
                          ADOQuery1.FieldByName('REFNO').AsString,
                          ADOQuery1.FieldByName('RANK').AsString,
                          tn]));
          Next;
        end;
      Close;
    end;
end;

function TfmMaster.CheckFieldOrder: TStringList;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  ErrorReport.Clear;
  try
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select tablename,fieldorder from columndoc ');
        SQL.Add('group by tablename,fieldorder ');
        SQL.Add('having count(*) > 1');
        Open;
        while not Eof do
          begin
            if sl.IndexOf(Fields[0].AsString) < 0 then
              sl.Add(Fields[0].AsString);
            Next;
          end;
        Close;
        SQL.Clear;
        SQL.Add('select tablename from columndoc ');
        SQL.Add('group by tablename ');
        SQL.Add('having max(fieldorder) <> count(*)');
        Open;
        while not Eof do
          begin
            if sl.IndexOf(Fields[0].AsString) < 0 then
              sl.Add(Fields[0].AsString);
            Next;
          end;
        Close;
        SQL.Clear;
        SQL.Add('select distinct tablename from columndoc ');
        SQL.Add('where (fieldorder is NULL) or (fieldorder < 1)');
        Open;
        while not Eof do
          begin
            if sl.IndexOf(Fields[0].AsString) < 0 then
              sl.Add(Fields[0].AsString);
            Next;
          end;
        Close;
      end;
    if sl.Count > 0 then
      ErrorReport.Add(Format(fsProblemFieldOrder,[sl.CommaText]));
    ErrorReport.Add(Format(fsErrorsFound,[sl.Count]));
  finally
    Result := ErrorReport;
    sl.Free;
  end;
end;

procedure TfmMaster.CheckForeignKey(dt,mt,dk,mk: string);
var
  n: integer;
  MasterKeys: TStringList;
  DetailKeys: TStringList;
begin
  MasterKeys := TStringList.Create;
  DetailKeys := TStringList.Create;
  try
    MasterKeys.CommaText := StringReplace(mk,';',',',[rfReplaceAll]);
    DetailKeys.CommaText := StringReplace(dk,';',',',[rfReplaceAll]);
    n := MasterKeys.Count;
    with ADOQuery1 do
      case n of
        1 : begin
              Close;
              SQL.Clear;
              SQL.Add('select ' + DetailKeys[0] + ' from ' + dt);
              SQL.Add('where ' + DetailKeys[0] + ' not in ');
              SQL.Add('(select distinct ' + MasterKeys[0] + ' from ' + mt + ')');
              Open;
              if not Eof then
                ErrorReport.Add(Format(fsForeignKeyProblem,[dt,DetailKeys[0],mt,MasterKeys[0]]));
              Close;
            end;
        2 : begin
              Close;
              SQL.Clear;
              SQL.Add('select ' + DetailKeys[1] + ' from ' + dt);
              SQL.Add('where ' + DetailKeys[1] + ' not in ');
              SQL.Add('(select distinct ' + mt + '.' + MasterKeys[1]);
              SQL.Add('from ' + mt + ' inner join ' + dt);
              SQL.Add('on ' + mt + '.' + MasterKeys[0] + ' = ' + dt + '.' + DetailKeys[0] + ')');
              Open;
              if not Eof then
                ErrorReport.Add(Format(fsForeignKeyProblem,[dt,DetailKeys[1],mt,MasterKeys[1]]));
              Close;
            end;
        3 : begin
              ShowMessage('3 keys');
            end;
        4 : begin
              ShowMessage('4 keys');
            end;
      end;
    ProgressBar1.StepIt;
    Self.Update;
  finally
    MasterKeys.Free;
    DetailKeys.Free;
  end;
end;

procedure TfmMaster.CheckGaps(tn,kf: string);
var
  i: integer;
  KeyFields: TStringList;
  KeyValues: TStringList;
begin
  KeyFields := TStringList.Create;
  KeyValues := TStringList.Create;
  try
    KeyFields.CommaText := kf;
    KeyFields.Delete(KeyFields.Count-1);
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select ' + KeyFields.CommaText + ',count(*) as CNT,max(RANK)');
        SQL.Add(' from ' + tn);
        SQL.Add(' group by ' + KeyFields.CommaText);
        SQL.Add(' having max(RANK) > count(*)');
        Open;
        while not Eof do
          begin
            KeyValues.Clear;
            for i := 0 to KeyFields.Count - 1 do
              KeyValues.Add(IntToStr(Fields[i].AsInteger));
            ErrorReport.Add(Format(fsFoundGaps,[tn,KeyValues.CommaText]));
            Next;
          end;
        Close;
      end;
    ProgressBar1.StepIt;
    Self.Update;
  finally
    KeyFields.Free;
    KeyValues.Free;
  end;
end;

function TfmMaster.CheckKeyFields: TStringList;
var
  i: integer;
  fl: TStringList;
  il: TStringList;
  sl: TStringList;
  tl: TStringList;
  TestTable: TADOTable;
begin
  fl := TStringList.Create;
  il := TStringList.Create;
  sl := TStringList.Create;
  tl := TStringList.Create;
  TestTable := TADOTable.Create(nil);
  ErrorReport.Clear;
  try
    ADOConnection1.GetTableNames(tl);
    TestTable.Connection := ADOConnection1;
    with ADOQuery1 do
      begin
        Close;
        SQL.Text := 'select tablename,keyfields from tabledoc';
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        ProgressBar1.Position := 0;
        First;
        while not Eof do
          begin
            if tl.IndexOf(Fields[0].AsString) >= 0 then
              begin
                TestTable.Close;
                TestTable.TableName := Fields[0].AsString;
                TestTable.Open;
                il.Clear;
                TestTable.GetIndexNames(il);
                if il.IndexOf('PrimaryKey') >= 0
                  then
                    begin
                      TestTable.Close;
                      TestTable.IndexName := 'PrimaryKey';
                      TestTable.Open;
                      fl.Clear;
                      for i := 0 to TestTable.IndexFieldCount - 1 do
                        fl.Add(TestTable.IndexFields[i].DisplayName);
                      if fl.CommaText <> Fields[1].AsString then
                        begin
                          ErrorReport.Add(Format(fsProblemKeyFields,[Fields[0].AsString,
                                          Fields[1].AsString,fl.CommaText]));
                          sl.Add(Fields[0].AsString);
                        end;
                    end
                  else
                    begin
                      ErrorReport.Add(Format(fsMissingPrimaryKey,[Fields[0].AsString,il.CommaText]));
                      sl.Add(Fields[0].AsString);
                    end;
              end;
            Next;
            ProgressBar1.StepIt;
            Self.Update;
          end;
        Close;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[sl.Count]));
  finally
    Result := ErrorReport;
    fl.Free;
    il.Free;
    sl.Free;
    tl.Free;
    TestTable.Close;
    TestTable.Free;
    ProgressBar1.Position := 0;
  end;
end;

procedure TfmMaster.CheckLookupValue(tn,fn,ln,lfn: string);
begin
  with ADOQuery1 do
    begin
      Close;
      SQL.Clear;
      SQL.Add('select ' + fn + ' from ' + tn);
      SQL.Add('where ' + fn + ' not in');
      SQL.Add('(select distinct ' + lfn + ' from ' + ln + ')');
      Open;
      if not Eof then
        ErrorReport.Add(Format(fsLookupValues,[tn,fn,ln]));
      Close;
    end;
  ProgressBar1.StepIt;
  Self.Update;
end;

function TfmMaster.CheckNullFieldSize: TStringList;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  ErrorReport.Clear;
  try
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select distinct tablename from columndoc ');
        SQL.Add('where fieldsize is NULL');
        Open;
        while not Eof do
          begin
            sl.Add(Fields[0].AsString);
            Next;
          end;
        Close;
      end;
    if sl.Count > 0 then
      ErrorReport.Add(Format(fsProblemFieldSize,[sl.CommaText]));
    ErrorReport.Add(Format(fsErrorsFound,[sl.Count]));
  finally
    Result := ErrorReport;
    sl.Free;
  end;
end;

procedure TfmMaster.CheckPrimaryKey(tn,fn: string);
var
  i: integer;
  FieldNames: TStringList;
begin
  FieldNames := TStringList.Create;
  try
    FieldNames.CommaText := fn;
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select count(*) as cnt_nulls');
        SQL.Add('from ' + tn + ' where ');
        SQL.Add('(' + FieldNames.Strings[0] + ' is NULL)');
        if FieldNames.Count > 1 then
          for i := 1 to FieldNames.Count - 1 do
            SQL.Add('or (' + FieldNames.Strings[i] + ' is Null)');
        Open;
        if FieldByName('cnt_nulls').AsInteger > 0 then
          ErrorReport.Add(Format(fsNullValues,[tn,fn]));
        Close;
        SQL.Clear;
        SQL.Add('select ' + fn + ' from ' + tn);
        SQL.Add('group by ' + fn);
        SQL.Add('having count(*) > 1');
        Open;
        if not Eof then
          ErrorReport.Add(Format(fsValuesNotUnique,[tn,fn]));
        Close;
      end;
    ProgressBar1.StepIt;
    Self.Update;
  finally
    FieldNames.Free;
  end;
end;

procedure TfmMaster.CheckSRC(tn,tp: string);
begin
  with ADOQuery1 do
    begin
      Close;
      SQL.Clear;
      SQL.Add('select refmain.REFNO,refmain.SRCTP from ' + tn + ' inner join refmain');
      SQL.Add('on ' + tn + '.REFNO = refmain.REFNO where refmain.SRCTP <> ''' + tp + '''');
      Open;
      while not Eof do
        begin
          ErrorReport.Add(Format(fsMismatchSrctp,[tn,
                          ADOQuery1.FieldByName('REFNO').AsString,
                          ADOQuery1.FieldByName('SRCTP').AsString]));
          Next;
        end;
      Close;
    end;
  ProgressBar1.StepIt;
  Self.Update;
end;

procedure TfmMaster.CheckValues(tn,fn,fv: string);
begin
  with ADOQuery1 do
    begin
      Close;
      SQL.Clear;
      SQL.Add('select count(*) as cnt_invalid from ' + tn);
      SQL.Add('where not ' + fn + ' in (' + fv + ')');
      Open;
      if FieldByName('cnt_invalid').AsInteger > 0 then
        ErrorReport.Add(Format(fsInvalidValues,[tn,fn]));
      Close;
    end;
  ProgressBar1.StepIt;
  Self.Update;
end;

function TfmMaster.CompareColumndoc: TStringList;
var
  i: integer;
  MColumndoc: TADOTable;
  TColumndoc: TADOTable;
begin
  MColumndoc := TADOTable.Create(nil);
  TColumndoc := TADOTable.Create(nil);
  ErrorReport.Clear;
  i := 0;
  try
    MColumndoc.Connection := ADOConnection2;
    MColumndoc.TableName := 'columndoc';
    MColumndoc.Open;
    if MColumndoc.RecordCount > 0 then
      ProgressBar1.Max := MColumndoc.RecordCount - 1;
    ProgressBar1.Position := 0;
    MColumndoc.First;
    TColumndoc.Connection := ADOConnection1;
    TColumndoc.TableName := 'columndoc';
    TColumndoc.Open;
    while not MColumndoc.Eof do
      begin
        if TColumndoc.Locate('TABLENAME;FIELDNAME',
                             VarArrayOf([MColumndoc.FieldByName('TABLENAME').AsString,
                                         MColumndoc.FieldByName('FIELDNAME').AsString]),[])
          then
            begin
              if TColumndoc.FieldByName('FIELDTYPE').AsString <>
                 MColumndoc.FieldByName('FIELDTYPE').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchColumndoc,['FIELDTYPE',
                                  MColumndoc.FieldByName('TABLENAME').AsString,
                                  MColumndoc.FieldByName('FIELDNAME').AsString,
                                  MColumndoc.FieldByName('FIELDTYPE').AsString,
                                  TColumndoc.FieldByName('FIELDTYPE').AsString]));
                end;
              if TColumndoc.FieldByName('FIELDSIZE').AsString <>
                 MColumndoc.FieldByName('FIELDSIZE').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchColumndoc,['FIELDSIZE',
                                  MColumndoc.FieldByName('TABLENAME').AsString,
                                  MColumndoc.FieldByName('FIELDNAME').AsString,
                                  MColumndoc.FieldByName('FIELDSIZE').AsString,
                                  TColumndoc.FieldByName('FIELDSIZE').AsString]));
                end;
              if TColumndoc.FieldByName('FIELDORDER').AsString <>
                 MColumndoc.FieldByName('FIELDORDER').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchColumndoc,['FIELDORDER',
                                  MColumndoc.FieldByName('TABLENAME').AsString,
                                  MColumndoc.FieldByName('FIELDNAME').AsString,
                                  MColumndoc.FieldByName('FIELDORDER').AsString,
                                  TColumndoc.FieldByName('FIELDORDER').AsString]));
                end;
              if TColumndoc.FieldByName('LTABLENAME').AsString <>
                 MColumndoc.FieldByName('LTABLENAME').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchColumndoc,['LTABLENAME',
                                  MColumndoc.FieldByName('TABLENAME').AsString,
                                  MColumndoc.FieldByName('FIELDNAME').AsString,
                                  MColumndoc.FieldByName('LTABLENAME').AsString,
                                  TColumndoc.FieldByName('LTABLENAME').AsString]));
                end;
              if TColumndoc.FieldByName('LFIELDNAME').AsString <>
                 MColumndoc.FieldByName('LFIELDNAME').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchColumndoc,['LFIELDNAME',
                                  MColumndoc.FieldByName('TABLENAME').AsString,
                                  MColumndoc.FieldByName('FIELDNAME').AsString,
                                  MColumndoc.FieldByName('LFIELDNAME').AsString,
                                  TColumndoc.FieldByName('LFIELDNAME').AsString]));
                end;
            end
          else
            begin
              Inc(i);
              ErrorReport.Add(Format(fsMissingTablenameFieldname,
                              [MColumndoc.FieldByName('TABLENAME').AsString,
                              MColumndoc.FieldByName('FIELDNAME').AsString]));
            end;
        MColumndoc.Next;
        ProgressBar1.StepIt;
        Self.Update;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[i]));
    MColumndoc.First;
    TColumndoc.First;
    i := 0;
    while not TColumndoc.Eof do
      begin
        if not MColumndoc.Locate('TABLENAME;FIELDNAME',
                                VarArrayOf([TColumndoc.FieldByName('TABLENAME').AsString,
                                            TColumndoc.FieldByName('FIELDNAME').AsString]),[])
          then
            begin
              ErrorReport.Add(Format(fsFoundExtraColumndoc,
                              [TColumndoc.FieldByName('TABLENAME').AsString,
                               TColumndoc.FieldByName('FIELDNAME').AsString]));
              inc(i);
            end;
        TColumndoc.Next;
      end;
    ErrorReport.Add(Format(fsExtrasFound,[i]));
  finally
    Result := ErrorReport;
    MColumndoc.Close;
    MColumndoc.Free;
    TColumndoc.Close;
    TColumndoc.Free;
    ProgressBar1.Position := 0;
  end;
end;

function TfmMaster.CompareEntities: TStringList;
var
  i: integer;
  MEntities: TADOTable;
  TEntities: TADOTable;
begin
  MEntities := TADOTable.Create(nil);
  TEntities := TADOTable.Create(nil);
  ErrorReport.Clear;
  i := 0;
  try
    MEntities.Connection := ADOConnection2;
    MEntities.TableName := 'entities';
    MEntities.Open;
    if MEntities.RecordCount > 0 then
      ProgressBar1.Max := MEntities.RecordCount - 1;
    ProgressBar1.Position := 0;
    MEntities.First;
    TEntities.Connection := ADOConnection1;
    TEntities.TableName := 'entities';
    TEntities.Open;
    while not MEntities.Eof do
      begin
        if TEntities.Locate('ENTITY;RANK',
                             VarArrayOf([MEntities.FieldByName('ENTITY').AsString,
                                         MEntities.FieldByName('RANK').AsString]),[])
          then
            begin
              if TEntities.FieldByName('TABLENAME').AsString <>
                 MEntities.FieldByName('TABLENAME').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchEntities,['TABLENAME',
                                  MEntities.FieldByName('ENTITY').AsString,
                                  MEntities.FieldByName('RANK').AsString,
                                  MEntities.FieldByName('TABLENAME').AsString,
                                  TEntities.FieldByName('TABLENAME').AsString]));
                end;
              if TEntities.FieldByName('TABLEGROUP').AsString <>
                 MEntities.FieldByName('TABLEGROUP').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchEntities,['TABLEGROUP',
                                  MEntities.FieldByName('ENTITY').AsString,
                                  MEntities.FieldByName('RANK').AsString,
                                  MEntities.FieldByName('TABLEGROUP').AsString,
                                  TEntities.FieldByName('TABLEGROUP').AsString]));
                end;
              if TEntities.FieldByName('EDATA').AsString <>
                 MEntities.FieldByName('EDATA').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchEntities,['EDATA',
                                  MEntities.FieldByName('ENTITY').AsString,
                                  MEntities.FieldByName('RANK').AsString,
                                  MEntities.FieldByName('EDATA').AsString,
                                  TEntities.FieldByName('EDATA').AsString]));
                end;
              if TEntities.FieldByName('ECAPTION').AsString <>
                 MEntities.FieldByName('ECAPTION').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchEntities,['ECAPTION',
                                  MEntities.FieldByName('ENTITY').AsString,
                                  MEntities.FieldByName('RANK').AsString,
                                  MEntities.FieldByName('ECAPTION').AsString,
                                  TEntities.FieldByName('ECAPTION').AsString]));
                end;
            end
          else
            begin
              Inc(i);
              ErrorReport.Add(Format(fsMissingEntityRank,
                              [MEntities.FieldByName('ENTITY').AsString,
                              MEntities.FieldByName('RANK').AsString]));
            end;
        MEntities.Next;
        ProgressBar1.StepIt;
        Self.Update;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[i]));
    MEntities.First;
    TEntities.First;
    i := 0;
    while not TEntities.Eof do
      begin
        if not MEntities.Locate('ENTITY;RANK',
                                VarArrayOf([TEntities.FieldByName('ENTITY').AsString,
                                            TEntities.FieldByName('RANK').AsString]),[])
          then
            begin
              ErrorReport.Add(Format(fsFoundExtraEntities,
                              [TEntities.FieldByName('ENTITY').AsString,
                               TEntities.FieldByName('RANK').AsString]));
              inc(i);
            end;
        TEntities.Next;
      end;
    ErrorReport.Add(Format(fsExtrasFound,[i]));
  finally
    Result := ErrorReport;
    MEntities.Close;
    MEntities.Free;
    TEntities.Close;
    TEntities.Free;
    ProgressBar1.Position := 0;
  end;
end;

function TfmMaster.CompareLinks: TStringList;
var
  i: integer;
  MLinks: TADOTable;
  TLinks: TADOTable;
begin
  MLinks := TADOTable.Create(nil);
  TLinks := TADOTable.Create(nil);
  ErrorReport.Clear;
  i := 0;
  try
    MLinks.Connection := ADOConnection2;
    MLinks.TableName := 'links';
    MLinks.Open;
    if MLinks.RecordCount > 0 then
      ProgressBar1.Max := MLinks.RecordCount - 1;
    ProgressBar1.Position := 0;
    MLinks.First;
    TLinks.Connection := ADOConnection1;
    TLinks.TableName := 'links';
    TLinks.Open;
    while not MLinks.Eof do
      begin
        if TLinks.Locate('RELNAME',MLinks.FieldByName('RELNAME').AsString,[])
          then
            begin
              if TLinks.FieldByName('PTABLE').AsString <>
                 MLinks.FieldByName('PTABLE').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchLinks,['PTABLE',
                                  MLinks.FieldByName('RELNAME').AsString,
                                  MLinks.FieldByName('PTABLE').AsString,
                                  TLinks.FieldByName('PTABLE').AsString]));
                end;
              if TLinks.FieldByName('PKEYS').AsString <>
                 MLinks.FieldByName('PKEYS').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchLinks,['PKEYS',
                                  MLinks.FieldByName('RELNAME').AsString,
                                  MLinks.FieldByName('PKEYS').AsString,
                                  TLinks.FieldByName('PKEYS').AsString]));
                end;
              if TLinks.FieldByName('FTABLE').AsString <>
                 MLinks.FieldByName('FTABLE').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchLinks,['FTABLE',
                                  MLinks.FieldByName('RELNAME').AsString,
                                  MLinks.FieldByName('FTABLE').AsString,
                                  TLinks.FieldByName('FTABLE').AsString]));
                end;
              if TLinks.FieldByName('FKEYS').AsString <>
                 MLinks.FieldByName('FKEYS').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchLinks,['FKEYS',
                                  MLinks.FieldByName('RELNAME').AsString,
                                  MLinks.FieldByName('FKEYS').AsString,
                                  TLinks.FieldByName('FKEYS').AsString]));
                end;
              if TLinks.FieldByName('ATTRIBS').AsString <>
                 MLinks.FieldByName('ATTRIBS').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchLinks,['ATTRIBS',
                                  MLinks.FieldByName('RELNAME').AsString,
                                  MLinks.FieldByName('ATTRIBS').AsString,
                                  TLinks.FieldByName('ATTRIBS').AsString]));
                end;
            end
          else
            begin
              Inc(i);
              ErrorReport.Add(Format(fsMissingLink,[MLinks.FieldByName('RELNAME').AsString]));
            end;
        MLinks.Next;
        ProgressBar1.StepIt;
        Self.Update;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[i]));
    MLinks.First;
    TLinks.First;
    i := 0;
    while not TLinks.Eof do
      begin
        if not MLinks.Locate('RELNAME',TLinks.FieldByName('RELNAME').AsString,[])
          then
            begin
              ErrorReport.Add(Format(fsFoundExtraLinks,
                              [TLinks.FieldByName('RELNAME').AsString]));
              inc(i);
            end;
        TLinks.Next;
      end;
    ErrorReport.Add(Format(fsExtrasFound,[i]));
  finally
    Result := ErrorReport;
    MLinks.Close;
    MLinks.Free;
    TLinks.Close;
    TLinks.Free;
    ProgressBar1.Position := 0;
  end;
end;

function TfmMaster.CompareTabledoc: TStringList;
var
  i: integer;
  MTabledoc: TADOTable;
  TTabledoc: TADOTable;
begin
  MTabledoc := TADOTable.Create(nil);
  TTabledoc := TADOTable.Create(nil);
  ErrorReport.Clear;
  i := 0;
  try
    MTabledoc.Connection := ADOConnection2;
    MTabledoc.TableName := 'tabledoc';
    MTabledoc.Open;
    if MTabledoc.RecordCount > 0 then
      ProgressBar1.Max := MTabledoc.RecordCount - 1;
    ProgressBar1.Position := 0;
    MTabledoc.First;
    TTabledoc.Connection := ADOConnection1;
    TTabledoc.TableName := 'tabledoc';
    TTabledoc.Open;
    while not MTabledoc.Eof do
      begin
        if TTabledoc.Locate('TABLENAME',MTabledoc.FieldByName('TABLENAME').AsString,[])
          then
            begin
              if TTabledoc.FieldByName('TABLETYPE').AsString <>
                 MTabledoc.FieldByName('TABLETYPE').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchTabledoc,['TABLETYPE',
                                  MTabledoc.FieldByName('TABLENAME').AsString,
                                  MTabledoc.FieldByName('TABLETYPE').AsString,
                                  TTabledoc.FieldByName('TABLETYPE').AsString]));
                end;
              if TTabledoc.FieldByName('TABLEGROUP').AsString <>
                 MTabledoc.FieldByName('TABLEGROUP').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchTabledoc,['TABLEGROUP',
                                  MTabledoc.FieldByName('TABLENAME').AsString,
                                  MTabledoc.FieldByName('TABLEGROUP').AsString,
                                  TTabledoc.FieldByName('TABLEGROUP').AsString]));
                end;
              if TTabledoc.FieldByName('TABLEPARENT').AsString <>
                 MTabledoc.FieldByName('TABLEPARENT').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchTabledoc,['TABLEPARENT',
                                  MTabledoc.FieldByName('TABLENAME').AsString,
                                  MTabledoc.FieldByName('TABLEPARENT').AsString,
                                  TTabledoc.FieldByName('TABLEPARENT').AsString]));
                end;
              if TTabledoc.FieldByName('KEYFIELDS').AsString <>
                 MTabledoc.FieldByName('KEYFIELDS').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchTabledoc,['KEYFIELDS',
                                  MTabledoc.FieldByName('TABLENAME').AsString,
                                  MTabledoc.FieldByName('KEYFIELDS').AsString,
                                  TTabledoc.FieldByName('KEYFIELDS').AsString]));
                end;
              if TTabledoc.FieldByName('TABLEORDER').AsString <>
                 MTabledoc.FieldByName('TABLEORDER').AsString then
                begin
                  Inc(i);
                  ErrorReport.Add(Format(fsMismatchTabledoc,['TABLEORDER',
                                  MTabledoc.FieldByName('TABLENAME').AsString,
                                  MTabledoc.FieldByName('TABLEORDER').AsString,
                                  TTabledoc.FieldByName('TABLEORDER').AsString]));
                end;
            end
          else
            begin
              Inc(i);
              ErrorReport.Add(Format(fsMissingTablename,[MTabledoc.FieldByName('TABLENAME').AsString]));
            end;
        MTabledoc.Next;
        ProgressBar1.StepIt;
        Self.Update;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[i]));
    MTabledoc.First;
    TTabledoc.First;
    i := 0;
    while not TTabledoc.Eof do
      begin
        if not MTabledoc.Locate('TABLENAME',TTabledoc.FieldByName('TABLENAME').AsString,[])
          then
            begin
              ErrorReport.Add(Format(fsFoundExtraTabledoc,
                              [TTabledoc.FieldByName('TABLENAME').AsString]));
              inc(i);
            end;
        TTabledoc.Next;
      end;
    ErrorReport.Add(Format(fsExtrasFound,[i]));
  finally
    Result := ErrorReport;
    MTabledoc.Close;
    MTabledoc.Free;
    TTabledoc.Close;
    TTabledoc.Free;
    ProgressBar1.Position := 0;
  end;
end;

procedure TfmMaster.CreateExportDatabaseRelationships;
var
  qr: TADOQuery;
  cmd: TADOCommand;
  sqlcmd: string;
  rn,ptable,pkeys,ftable,fkeys,attribs: string;
begin
  Memo1.Lines.Add(msCreateExportDatabaseRelationships);
  Self.Update;
  qr := TADOQuery.Create(nil);
  cmd := TADOCommand.Create(nil);
  try
    qr.Connection := ADOConnection1;
    qr.SQL.Text := 'select RELNAME,PTABLE,PKEYS,FTABLE,FKEYS,ATTRIBS from links order by RELNAME';
    cmd.Connection := ADOConnection2;
    qr.Open;
    while not qr.Eof do
      begin
        rn := qr.Fields[0].AsString;
        ptable := qr.Fields[1].AsString;
        pkeys := StringReplace(qr.Fields[2].AsString,';',',',[rfReplaceAll]);
        ftable := qr.Fields[3].AsString;
        fkeys := StringReplace(qr.Fields[4].AsString,';',',',[rfReplaceAll]);
        attribs := qr.Fields[5].AsString;
        sqlcmd := 'alter table ' + ftable +
                  ' add constraint ' + rn +
                  ' foreign key (' + fkeys + ') ' +
                  ' references ' + ptable +
                  ' (' + pkeys + ')';
        if AnsiPos('CascadeUpdate',attribs) > 0 then
          sqlcmd := sqlcmd + ' on update cascade';
        if AnsiPos('CascadeDelete',attribs) > 0 then
          sqlcmd := sqlcmd + ' on delete cascade';
        cmd.CommandText := sqlcmd;
        try
          cmd.Execute;
        except
          on E: EDatabaseError do
            ShowMessage(sqlcmd + chr(10) + chr(10) + E.Message);
        end;
        qr.Next;
      end;
    qr.Close;
  finally
    qr.Close;
    qr.Free;
    cmd.Free;
  end;
end;

procedure TfmMaster.CreateExportDatabaseTablesAndFields;
var
  qt: TADOQuery;
  qf: TADOQuery;
  cmd: TADOCommand;
  kn: TStringList;
  sqlcmd: string;
  tn: string;
  fn,ft,fs: string;
  nn: string;
begin
  Memo1.Lines.Add(msCreateExportDatabaseTablesAndFields);
  Self.Update;
  qt := TADOQuery.Create(nil);
  qf := TADOQuery.Create(nil);
  cmd := TADOCommand.Create(nil);
  kn := TStringList.Create;
  try
    qt.Connection := ADOConnection1;
    qt.SQL.Text := 'select tablename,keyfields from tabledoc order by tablename';
    qf.Connection := ADOConnection1;
    qf.SQL.Text := 'select tablename,fieldname,fieldtype,fieldsize ' +
                   'from columndoc where tablename = :tablename ' +
                   'order by fieldorder';
    qf.ParamCheck := True;
    cmd.Connection := ADOConnection2;
    cmd.CommandType := cmdText;
    qt.Open;
    while not qt.Eof do
      begin
        tn := qt.Fields[0].AsString;
        kn.CommaText := qt.Fields[1].AsString;
        sqlcmd := 'create table ' + tn + ' (';
        qf.Parameters[0].Value := tn;
        qf.Open;
        while not qf.Eof do
          begin
            fn := qf.Fields[1].AsString;
            if fn = 'MODULE' then
              begin
                fn := 'MODULEX';
                Memo1.Lines.Add(msMODULEX);
                Self.Update;
              end;
            ft := qf.Fields[2].AsString;
            fs := qf.Fields[3].AsString;
            if kn.IndexOf(fn) >= 0
              then nn := ' not null,'
              else nn := ',';
            if ft = 'Number (Single)' then
              sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' single' + nn
            else
            if ft = 'Number (Double)' then
              sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' double' + nn
            else
            if ft = 'Number (Integer)' then
              sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' short' + nn
            else
            if ft = 'Number (Long)' then
              sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' integer' + nn
            else
            if ft = 'Text' then
              if StrToInt(fs) <= 255
                then sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' varchar(' + fs + ')' + nn
                else sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' memo' + nn
            else
            if ft = 'Yes/No' then
              sqlcmd := sqlcmd + chr(10) + '  ' + fn + ' yesno' + nn;
            qf.Next;
          end;
        qf.Close;
        sqlcmd := sqlcmd + chr(10) + '  primary key (' + kn.CommaText + '));';
        cmd.CommandText := sqlcmd;
        try
          cmd.Execute;
        except
          ShowMessage(sqlcmd);
        end;
        qt.Next;
      end;
    qt.Close;
  finally
    qt.Close;
    qt.Free;
    qf.Close;
    qf.Free;
    cmd.Free;
    kn.Free;
  end;
end;

procedure TfmMaster.DoTheMerge;
var
  i: integer;
  TableList: TStringList;
  qry: TADOQuery;
  tmpCursor: TCursor;
begin
  TableList := TStringList.Create;
  qry := TADOQuery.Create(nil);
  tmpCursor := Screen.Cursor;
  try
    Screen.Cursor := crSQLWait;
    qry.Connection := ADOConnection1;
    qry.SQL.Text := 'select tablename,keyfields from tabledoc where tabletype = ''Lookup''';
    qry.Open;
    ProgressBar1.Max := qry.RecordCount - 1;
    ProgressBar1.Position := 0;
    qry.First;
    while not qry.Eof do
      begin
        MergeLookupTables(qry.Fields[0].AsString,qry.Fields[1].AsString);
        ProgressBar1.StepIt;
        qry.Next;
      end;
    qry.Close;

    TableList.CommaText := dta;
    ProgressBar1.Max := TableList.Count - 1;
    ProgressBar1.Position := 0;
    qry.SQL.Text := 'select tablename from tabledoc where tabletype in (''Data'',''Junction'')';
    qry.Open;
    for i := 0 to TableList.Count - 1 do
      begin
        if qry.Locate('tablename',TableList.Strings[i],[loCaseInsensitive]) then
          begin
            MergeDataTables(TableList.Strings[i]);
            ProgressBar1.StepIt;
          end;
      end;
    qry.Close;
  finally
    qry.Close;
    qry.Free;
    TableList.Free;
    Screen.Cursor := tmpCursor;
  end;
end;

function TfmMaster.EmptyDatabaseOK: boolean;
var
  tn: TStringList;
begin
  Result := False;
  Memo1.Lines.Add(msEmptyDatabase);
  Self.Update;
  tn := TStringList.Create;
  try
    OpenDialog1.InitialDir := InitialDirectory;
    OpenDialog1.Title := msOpenExportSubsetDatabase;
    OpenDialog1.FileName := '';
    if OpenDialog1.Execute
      then
        begin
          ADOConnection2.ConnectionString := 'FILE NAME=' + OpenDialog1.FileName;
          ADOConnection2.Open;
          ADOConnection2.GetTableNames(tn);
          ADOConnection2.Close;
          if tn.Count > 0
            then
              begin
                Memo1.Lines.Add(msExportDatabaseNotEmpty);
                Memo1.Lines.Add('');
                Exit;
              end
            else Result := True;
        end
      else
        begin
          Memo1.Lines.Add(msOperationCancelled);
          Memo1.Lines.Add('');
          Exit;
        end;
  finally
    tn.Free;
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.EnableButtons(b: boolean);
begin
  buMetadata.Enabled := b;
  buVersion.Enabled := b;
  buSchema.Enabled := b;
  buData.Enabled := b;
  buSummary.Enabled := b;
  buReldate.Enabled := b;
  buSQLScript.Enabled := b;
  buImport.Enabled := b;
  buExportSubset.Enabled := b;
  buSetCaptionLanguage.Enabled := b;
end;

function TfmMaster.GetCommodities(entity,e_key: string;keyno: longint): string;
var
  NameTable : string;
  NameField : string;
  qry: TADOQuery;
begin
  Result := '';
  qry := TADOQuery.Create(nil);
  try
    qry.Connection := ADOConnection1;
    NameTable := AnsiLowerCase(entity) + 'com';
    NameField := 'COMMODITY';
    with qry do
      begin
        Close;
        SQL.Text := Format('select RANK,%s from %s where %s = %d order by RANK',
                           [NameField,NameTable,e_key,keyno]);
        Open;
        if Eof then
          begin
            Close;
            Exit;
          end;
        if not FieldByName(NameField).IsNull then
          Result := FieldByName(NameField).AsString;
        Next;
        while not Eof do
          begin
            if not FieldByName(NameField).IsNull then
              Result := Result + '; ' + FieldByName(NameField).AsString;
            Next;
          end;
        Close;
      end;
  finally
    qry.Free;
  end;
end;

function TfmMaster.GetCountry(entity,e_key: string;keyno: longint): string;
var
  ctry: string;
//  nts: string;
  dtable: string;
  qry: TADOQuery;
begin
  Result := '';
  qry := TADOQuery.Create(nil);
  try
    qry.Connection := ADOConnection1;
    dtable := AnsiLowerCase(entity) + 'ctry';
    with qry do
      begin
        Close;
        SQL.Text := Format('select RANK,COUNTRY,PROVSTATE from %s where %s = %d order by RANK',
                           [dtable,e_key,keyno]);
        Open;
        if not Eof then
          begin
            if not FieldByName('COUNTRY').IsNull then
              ctry := FieldByName('COUNTRY').AsString;
            if not FieldByName('PROVSTATE').IsNull then
              if ctry <> ''
                then ctry := ctry + ' (' + FieldByName('PROVSTATE').AsString + ')'
                else ctry := FieldByName('PROVSTATE').AsString;
            Next;
            while not Eof do
              begin
                if not FieldByName('COUNTRY').IsNull then
                  ctry := ctry + ' / ' + FieldByName('COUNTRY').AsString;
                if not FieldByName('PROVSTATE').IsNull then
                  if ctry <> ''
                    then ctry := ctry + ' (' + FieldByName('PROVSTATE').AsString + ')'
                    else ctry := FieldByName('PROVSTATE').AsString;
                Next;
              end;
          end;
        Close;
   {
        SQL.Text := 'select count(*) from tabledoc where tablename like ''d*nts''';
        Open;
        if Fields[0].AsInteger > 0
          then
            begin
              Close;
              dtable := AnsiLowerCase(entity) + 'nts';
              SQL.Text := Format('select RANK,MAPID from %s where %s = %d order by RANK',
                                 [dtable,e_key,keyno]);
              Open;
              if not Eof then
                begin
                  if not FieldByName('MAPID').IsNull then
                    nts := FieldByName('MAPID').AsString;
                  Next;
                  while not Eof do
                    begin
                      if not FieldByName('MAPID').IsNull then
                        nts := nts + ' / ' + FieldByName('MAPID').AsString;
                      Next;
                    end;
                end;
            end
          else nts := '';
        Close;
}
      end;
    Result := ctry;
{
    if Result = ''
      then Result := nts
      else if nts <> '' then
             Result := Result + ' (' + nts + ')';
}
  finally
    qry.Free;
  end;
end;

function TfmMaster.GetDBVersion: string;
var
  s: string;
begin
  Result := '';
  with ADOQuery1 do
    begin
      Close;
      SQL.Clear;
      SQL.Add('select DBREL from dbversion');
      Open;
      s := Fields[0].AsString;
      Close;
    end;
  if AnsiPos('3.19',s) > 0 then
    Result := '3.19'
  else
  if AnsiPos('3.20',s) > 0 then
    Result := '3.20'
  else
  if AnsiPos('3.21',s) > 0 then
    Result := '3.21'
  else
  if AnsiPos('3.22',s) > 0 then
    Result := '3.22';
end;

function TfmMaster.GetFieldType(ft: TFieldType): string;
begin
  case ft of
    ftFloat: Result := 'Float';
    ftSmallint: Result := 'Short integer';
    ftInteger: Result := 'Long integer';
    ftWideString: Result := 'Text';
    ftBoolean: Result := 'Yes/No';
    ftMemo: Result := 'Text';
  else Result := 'Unknown';
  end;
end;

function TfmMaster.GetLocation: string;
begin
  Result := '';
  with ADOQuery1 do
    begin
      if not FieldByName('LATDEG').IsNull then
        Result := Format('%2.2d',[FieldByName('LATDEG').AsInteger]);
      if not FieldByName('LATMIN').IsNull then
        Result := Result + '.' + Format('%2.2d',[FieldByName('LATMIN').AsInteger]);
      if not FieldByName('LATSEC').IsNull then
        Result := Result + '.' + Format('%2.2d',[FieldByName('LATSEC').AsInteger]);
      if not FieldByName('NS').IsNull then
        Result := Result + ' ' + FieldByName('NS').AsString;
      if not FieldByName('LONGDEG').IsNull then
        Result := Result + ' -- ' + Format('%3.3d',[FieldByName('LONGDEG').AsInteger]);
      if not FieldByName('LONGMIN').IsNull then
        Result := Result + '.' + Format('%2.2d',[FieldByName('LONGMIN').AsInteger]);
      if not FieldByName('LONGSEC').IsNull then
        Result := Result + '.' + Format('%2.2d',[FieldByName('LONGSEC').AsInteger]);
      if not FieldByName('EW').IsNull then
        Result := Result + ' ' + FieldByName('EW').AsString;
    end;
end;

function TfmMaster.GetNames(entity,e_key: string;keyno: longint): string;
var
  NameTable : string;
  NameField : string;
  qry: TADOQuery;
begin
  Result := '';
  qry := TADOQuery.Create(nil);
  try
    qry.Connection := ADOConnection1;
    NameTable := AnsiLowerCase(entity) + 'name';
    if entity = 'DP'
      then NameField := 'DEPNAME'
      else NameField := 'DGNAME';
    with qry do
      begin
        Close;
        SQL.Text := Format('select RANK,%s from %s where %s = %d order by RANK',
                           [NameField,NameTable,e_key,keyno]);
        Open;
        if not Eof then
          begin
            if not FieldByName(NameField).IsNull then
              Result := FieldByName(NameField).AsString;
            Next;
            while not Eof do
              begin
                if not FieldByName(NameField).IsNull then
                  Result := Result + '; ' + FieldByName(NameField).AsString;
                Next;
              end;
          end;
        Close;
      end;
  finally
    qry.Free;
  end;
end;

function TfmMaster.KeyValuesFileOK(var kv: TStringList): boolean;
var
  i: integer;
  ok: boolean;
  filter,defaultext: string;
  kq: TADOQuery;
  kn: TStringList;
  kfile: TStringList;
  kdata: TStringList;
begin
  Result := False;
  kq := TADOQuery.Create(nil);
  kn := TStringList.Create;
  kfile := TStringList.Create;
  kdata := TStringList.Create;
  try
    Memo1.Lines.Add(msKeyValuesFile);
    Self.Update;
    OpenDialog1.InitialDir := InitialDirectory;
    OpenDialog1.Title := msOpenKeyValuesFile;
    filter := OpenDialog1.Filter;
    defaultext := OpenDialog1.DefaultExt;
    OpenDialog1.Filter := 'Text files (*.txt)|*.txt';
    OpenDialog1.DefaultExt := 'txt';
    OpenDialog1.FileName := '';
    if OpenDialog1.Execute
      then
        begin
          kfile.LoadFromFile(OpenDialog1.FileName);
          for i := 0 to kfile.Count - 1 do
            begin
              kdata.CommaText := StringReplace(kfile.Strings[i],chr(9),',',[rfReplaceAll]);
              kv.Add(kdata.Strings[0]);
            end;
          ok := kfile.Count > 1;
          if not ok
            then
              begin
                Memo1.Lines.Add(msKeyValuesFileError);
                Memo1.Lines.Add('');
                Exit;
              end;
          kq.Connection := ADOConnection1;
          kq.SQL.Text := 'select EDATA from entities where RANK = 1';
          kq.Open;
          while not kq.Eof do
            begin
              kn.Add(kq.Fields[0].AsString);
              kq.Next;
            end;
          kq.Close;
          if kn.IndexOf(kv.Strings[0]) < 0 then
            ok := False;
          for i := 1 to kv.Count - 1 do
            try
              StrToInt(kv.Strings[i]);
            except
              on E: EConvertError do
                ok := False;
            end;
          if not ok
            then
              begin
                Memo1.Lines.Add(msKeyValuesFileError);
                Memo1.Lines.Add('');
              end
            else Result := True;
        end
      else
        begin
          Memo1.Lines.Add(msOperationCancelled);
          Memo1.Lines.Add('');
          Exit;
        end;
  finally
    kq.Close;
    kq.Free;
    kn.Free;
    kfile.Free;
    kdata.Free;
    OpenDialog1.Filter := filter;
    OpenDialog1.DefaultExt := defaultext;
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.MergeDataTables(tn: string);
var
  i: integer;
  cnt: integer;
  flds: TStringList;
begin
  flds := TStringList.Create;
  cnt := 0;
  try
    Columndoc.Parameters[0].Value := tn;
    Columndoc.Open;
    while not Columndoc.Eof do
      begin
        flds.Add(Columndoc.FieldByName('FIELDNAME').AsString);
        Columndoc.Next;
      end;
    Columndoc.Close;
    MasterData.CommandText := tn;
    MasterData.Open;
    SourceData.CommandText := tn;
    SourceData.Open;
    while not SourceData.Eof do
      begin
        MasterData.Append;
        for i := 0 to flds.Count - 1 do
          MasterData.FieldByName(flds.Strings[i]).Value := SourceData.FieldByName(flds.Strings[i]).Value;
        MasterData.Post;
        SourceData.Next;
        inc(cnt);
      end;
  finally
    SourceData.Close;
    MasterData.Close;
    flds.Free;
    Memo1.Lines.Add(Format(fsRecordsAdded,[cnt,tn]));
    Self.Update;
  end;
end;

procedure TfmMaster.MergeLookupTables(tn,keys: string);
var
  i: integer;
  cnt: integer;
  flds: TStringList;
begin
  flds := TStringList.Create;
  cnt := 0;
  try
    Columndoc.Parameters[0].Value := tn;
    Columndoc.Open;
    while not Columndoc.Eof do
      begin
        flds.Add(Columndoc.FieldByName('FIELDNAME').AsString);
        Columndoc.Next;
      end;
    Columndoc.Close;
    MasterData.CommandText := tn;
    MasterData.Open;
    SourceData.CommandText := tn;
    SourceData.Open;
    while not SourceData.Eof do
      begin
        if not MasterData.Locate(keys,SourceData.FieldByName(keys).AsString,[loCaseInsensitive]) then
          begin
            MasterData.Append;
            for i := 0 to flds.Count - 1 do
              MasterData.FieldByName(flds.Strings[i]).Value := SourceData.FieldByName(flds.Strings[i]).Value;
            MasterData.Post;
            inc(cnt);
          end;
        SourceData.Next;
      end;
  finally
    SourceData.Close;
    MasterData.Close;
    flds.Free;
    Memo1.Lines.Add(Format(fsRecordsAdded,[cnt,tn]));
    Self.Update;
  end;
end;

function TfmMaster.OKToMerge: boolean;
var
  qryTarget: TADOQuery;
  qrySource: TADOQuery;
  kn: string;
  minTarget: integer;
  minSource: integer;
  maxTarget: integer;
  maxSource: integer;
  cntTarget: integer;
  cntSource: integer;
  ok: boolean;
begin
  qryTarget := TADOQuery.Create(nil);
  qrySource := TADOQuery.Create(nil);
  try
    qryTarget.Connection := ADOConnection1;
    qrySource.Connection := ADOConnection2;
    qryTarget.SQL.Text := 'select "DGNO",count(*),min(DGNO),max(DGNO) from dggeol ' +
                          'union ' +
                          'select "DEPNO",count(*),min(DEPNO),max(DEPNO) from dpgeol ' +
                          'union ' +
                          'select "MINENO",count(*),min(MINENO),max(MINENO) from mine ' +
                          'union ' +
                          'select "PRODNO",count(*),min(PRODNO),max(PRODNO) from prodmain ' +
                          'union ' +
                          'select "RESNO",count(*),min(RESNO),max(RESNO) from resmain ' +
                          'union ' +
                          'select "REFNO",count(*),min(REFNO),max(REFNO) from refmain';
    qrySource.SQL.Text := qryTarget.SQL.Text;
    qryTarget.Open;
    qrySource.Open;
    ok := True;
    while not qryTarget.Eof do
      begin
        kn := qryTarget.Fields[0].AsString;
        cntTarget := qryTarget.Fields[1].AsInteger;
        minTarget := qryTarget.Fields[2].AsInteger;
        maxTarget := qryTarget.Fields[3].AsInteger;
        cntSource := qrySource.Fields[1].AsInteger;
        minSource := qrySource.Fields[2].AsInteger;
        maxSource := qrySource.Fields[3].AsInteger;
        if (cntTarget > 0) and (cntSource > 0) and not
          ((minSource > maxTarget) or (maxSource < minTarget))
          then ok := False;
        qryTarget.Next;
        qrySource.Next;
      end;
    Result := ok;
  finally
    qryTarget.Close;
    qryTarget.Free;
    qrySource.Close;
    qrySource.Free;
  end;
end;

procedure TfmMaster.PopulateRecordCountTable;
var
  k: string;
  tn: string;
  cmd: TADOCommand;
  qry: TADOQuery;
begin
  cmd := TADOCommand.Create(nil);
  qry := TADOQuery.Create(nil);
  try
    cmd.Connection := ADOConnection1;
    cmd.CommandType := cmdText;
    qry.Connection := ADOConnection1;
    cmd.CommandText := 'create table gq_recordcount (' +
                       '  TABLENAME varchar(16) primary key,' +
                       '  RECORDCOUNT integer' +
                       ');';
    cmd.Execute;
    with ADOQuery1 do
      begin
        Close;
        SQL.Text := 'select tablename from tabledoc';
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        ProgressBar1.Position := 0;
        First;
        while not Eof do
          begin
            tn := Fields[0].AsString;
            qry.Close;
            qry.SQL.Text := Format('select count(*) from %s',[tn]);
            qry.Open;
            k := qry.Fields[0].AsString;
            cmd.CommandText := Format('insert into gq_recordcount (tablename,recordcount) ' +
                                      'values (''%s'',%s);',[tn,k]);
            cmd.Execute;
            Next;
            ProgressBar1.StepIt;
            Self.Update;
          end;
        qry.Close;
        Close;
      end;
  finally
    qry.Close;
    qry.Free;
    cmd.Free;
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.PopulateSummaryTable(entity,e_key,e_table: string);
// populate summary table
var
  tmpCursor: TCursor;
  e_country: string;
  dbname: string;
  compby: string;
  reldate: string;
  Summary: TADOTable;
begin
  tmpCursor := Screen.Cursor;
  Summary := TADOTable.Create(nil);
  try
    Screen.Cursor := crSQLWait;
    Summary.Connection := ADOConnection1;
    // get keyno and location from main table
    e_country := AnsiLowerCase(entity) + 'ctry';
    with ADOQuery1 do
      begin
        Close;
        SQL.Text := 'select dbname,compby,reldate from dbversion';
        Open;
        if FieldByName('DBNAME').IsNull
          then dbname := ''
          else dbname := FieldByName('DBNAME').AsString;
        if FieldByName('COMPBY').IsNull
          then compby := ''
          else compby := FieldByName('COMPBY').AsString;
        if FieldByName('RELDATE').IsNull
          then reldate := ''
          else
            begin
              reldate := FieldByName('RELDATE').AsString;
              reldate := Format('%s-%s-%s',[Copy(reldate,1,4),Copy(reldate,5,2),Copy(reldate,7,2)]);
            end;
        Close;
        SQL.Text := Format('select %s.%s,LATDEG,LATMIN,LATSEC,NS,LONGDEG,LONGMIN,LONGSEC,' +
                           'EW,LONGDD,LATDD,DEPCLAN,COUNTRY from %s left outer join %s ' +
                           'on %s.%s = %s.%s where (%s.RANK is null) or (%s.RANK = 1)',
                           [e_table,e_key,e_table,e_country,e_country,e_key,e_table,e_key,e_country,e_country]);
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        ProgressBar1.Position := 0;
        First;
        Summary.TableName := AnsiLowerCase(entity) + '_summary';
        Summary.Open;
        while not Eof do
          begin
            Summary.Append;
            Summary.FieldByName(e_key).AsInteger := FieldByName(e_key).AsInteger;
            Summary.FieldByName('COUNTRY').AsString := GetCountry(entity,e_key,FieldByName(e_key).AsInteger);
            Summary.FieldByName('LOCATION').AsString := GetLocation;
            Summary.FieldByName('ALLNAMES').AsString := GetNames(entity,e_key,FieldByName(e_key).AsInteger);
            Summary.FieldByName('COMMODITIES').AsString := GetCommodities(entity,e_key,FieldByName(e_key).AsInteger);
            Summary.FieldByName('DEPCLAN').AsString := FieldByName('DEPCLAN').AsString;
            Summary.FieldByName('LONGDD').AsFloat := FieldByName('LONGDD').AsFloat;
            Summary.FieldByName('LATDD').AsFloat := FieldByName('LATDD').AsFloat;
            Summary.FieldByName('DBNAME').AsString := dbname;
            Summary.FieldByName('COMPBY').AsString := compby;
            Summary.FieldByName('RELDATE').AsString := reldate;
            Summary.Post;
            Next;
            ProgressBar1.StepIt;
            Self.Update;
          end;
        Summary.Close;
        Close;
      end;
  finally
    Summary.Free;
    Screen.Cursor := tmpCursor;
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.PopulateSummaryTables;
var
  cmd: TADOCommand;
begin
  cmd := TADOCommand.Create(nil);
  try
    cmd.Connection := ADOConnection1;
    cmd.CommandText := 'create table dg_summary(' +
                       '  DGNO integer primary key,' +
                       '  COUNTRY varchar(35),' +
                       '  LOCATION varchar(30),' +
                       '  ALLNAMES varchar(200),' +
                       '  COMMODITIES varchar(100),' +
                       '  DEPCLAN varchar(50),' +
                       '  LONGDD float,' +
                       '  LATDD float,' +
                       '  DBNAME varchar(30),' +
                       '  COMPBY varchar(200),' +
                       '  RELDATE varchar(10)' +
                       ')';
    cmd.Execute;
    PopulateSummaryTable('DG','DGNO','dggeol');

    cmd.CommandText := 'create table dp_summary(' +
                       '  DEPNO integer primary key,' +
                       '  COUNTRY varchar(35),' +
                       '  LOCATION varchar(30),' +
                       '  ALLNAMES varchar(200),' +
                       '  COMMODITIES varchar(100),' +
                       '  DEPCLAN varchar(50),' +
                       '  LONGDD float,' +
                       '  LATDD float,' +
                       '  DBNAME varchar(30),' +
                       '  COMPBY varchar(200),' +
                       '  RELDATE varchar(10)' +
                       ')';
    cmd.Execute;
    PopulateSummaryTable('DP','DEPNO','dpgeol');
  finally
    cmd.Free;
  end;
end;

function TfmMaster.VerifyConformity: TStringList;
var
  i: integer;
  Columndoc: TADOTable;
  Tabledoc: TADOTable;
  dsTabledoc: TDataSource;
  TestTable: TADOTable;
  t_name: string;
  c_name: string;
  c_position: integer;
  c_type: string;
  c_size: integer;
  ft: TFieldType;
  ok: boolean;
  tl: TStringList;
  cl: TStringList;
begin
  Tabledoc := TADOTable.Create(nil);
  dsTabledoc := TDataSource.Create(nil);
  Columndoc := TADOTable.Create(nil);
  TestTable := TADOTable.Create(nil);
  tl := TStringList.Create;
  cl := TStringList.Create;
  ErrorReport.Clear;
  i := 0;
  try
    ADOConnection1.GetTableNames(tl);
    Tabledoc.Connection := ADOConnection1;
    Tabledoc.TableName := 'tabledoc';
    dsTabledoc.DataSet := Tabledoc;
    Columndoc.Connection := ADOConnection1;
    Columndoc.TableName := 'columndoc';
    Columndoc.MasterSource := dsTabledoc;
    Columndoc.MasterFields := 'TABLENAME';
    TestTable.Connection := ADOConnection1;
    Tabledoc.Open;
    if Tabledoc.RecordCount > 0 then
      ProgressBar1.Max := Tabledoc.RecordCount - 1;
    ProgressBar1.Position := 0;
    Tabledoc.First;
    Columndoc.Open;
    while not Tabledoc.Eof do
      begin
        TestTable.Close;
        t_name := Tabledoc.FieldByName('TABLENAME').AsString;
        if tl.IndexOf(t_name) >= 0
          then
            begin
              TestTable.TableName := t_name;
              TestTable.Open;
              TestTable.GetFieldNames(cl);
              if cl.Count >= Columndoc.RecordCount
                then
                  begin
                    Columndoc.First;
                    while not Columndoc.Eof do
                      begin
                        c_name := Columndoc.FieldByName('FIELDNAME').AsString;
                        if cl.IndexOf(c_name) >= 0
                          then
                            begin
                              c_position := Columndoc.FieldByName('FIELDORDER').AsInteger - 1;
                              c_type := Columndoc.FieldByName('FIELDTYPE').AsString;
                              c_size := StrToInt(Columndoc.FieldByName('FIELDSIZE').AsString);
                              if TestTable.Fields[c_position].FieldName <> c_name
                                then
                                  begin
                                    Inc(i);
                                    ErrorReport.Add(Format(fsMismatchFieldPosition,
                                                    [t_name,c_name,c_position,c_name,
                                                    TestTable.Fields[c_position].FieldName]));
                                  end
                                else
                                  begin
                                    ft := TestTable.Fields[c_position].DataType;
                                    if ((c_type = 'Text') and (ft in [ftWideString,ftMemo])) or
                                       ((c_type = 'Number (Long)') and (ft = ftInteger)) or
                                       ((c_type = 'Number (Integer)') and (ft = ftSmallint)) or
                                       ((c_type = 'Number (Single)') and (ft = ftFloat)) or
                                       ((c_type = 'Number (Double)') and (ft = ftFloat)) or
                                       ((c_type = 'Yes/No') and (ft = ftBoolean))
                                      then ok := True
                                      else ok := False;
                                    if not ok
                                      then
                                        begin
                                          Inc(i);
                                          ErrorReport.Add(Format(fsMismatchFieldType,
                                                          [t_name,c_name,c_type,GetFieldType(ft)]));
                                        end
                                      else
                                        if (ft = ftWideString) and
                                           (TestTable.Fields[c_position].DisplayWidth <> c_size) then
                                          begin
                                            Inc(i);
                                            ErrorReport.Add(Format(fsMismatchFieldSize,
                                                            [t_name,c_name,c_size,
                                                            TestTable.Fields[c_position].DisplayWidth]));
                                          end;
                                  end;
                            end
                          else
                            begin
                              Inc(i);
                              ErrorReport.Add(Format(fsMissingField,[c_name,t_name]));
                            end;
                        Columndoc.Next;
                      end;
                  end
                else
                  begin
                    Inc(i);
                    ErrorReport.Add(Format(fsTooFewFields,[t_name]));
                  end;
            end
          else
            begin
              Inc(i);
              ErrorReport.Add(Format(fsMissingTable,[t_name]));
            end;
        Tabledoc.Next;
        ProgressBar1.StepIt;
        Self.Update;
      end;
    ErrorReport.Add(Format(fsErrorsFound,[i]));
  finally
    Result := ErrorReport;
    TestTable.Close;
    TestTable.Free;
    Columndoc.Close;
    Columndoc.Free;
    dsTabledoc.Free;
    Tabledoc.Close;
    Tabledoc.Free;
    tl.Free;
    cl.Free;
    ProgressBar1.Position := 0;
  end;
end;

procedure TfmMaster.buExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMaster.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ADOConnection1.Close;
  ErrorReport.Free;
end;

procedure TfmMaster.FormShow(Sender: TObject);
begin
  GetDir(0,InitialDirectory);
  fmMaster.Caption := cpMaster;
  buMetadata.Caption := cpMetadata;
  buMetadata.Hint := BuildHint(htMetadata);
  buVersion.Caption :=  cpVersion;
  buVersion.Hint := BuildHint(htVersion);
  buSchema.Caption := cpSchema;
  buSchema.Hint := BuildHint(htSchema);
  buData.Caption := cpData;
  buData.Hint := BuildHint(htData);
  buSetCaptionLanguage.Caption := cpSetCaptionLanguage;
  buSetCaptionLanguage.Hint := BuildHint(htSetCaptionLanguage);
  buSummary.Caption := cpSummary;
  buSummary.Hint := BuildHint(htSummary);
  buReldate.Caption := cpReldate;
  buReldate.Hint := BuildHint(htReldate);
  buSQLScript.Caption := cpSQLScript;
  buSQLScript.Hint := BuildHint(htSQLScript);
  buImport.Caption := cpImport;
  buImport.Hint := BuildHint(htImport);
  buExportSubset.Caption := cpExportSubset;
  buExportSubset.Hint := BuildHint(htExportSubset);
  buConnection.Caption := cpConnect;
  buClear.Caption := cpClear;
  buReport.Caption := cpReport;
  buExit.Caption := cpExit;
  EnableButtons(False);
  Memo1.Clear;
  Memo1.Lines.Add(msIntroLine1);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(msIntroLine2);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htMetadata);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htVersion);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htSchema);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htData);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htSetCaptionLanguage);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htSummary);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htReldate);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htSQLScript);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htImport);
  Memo1.Lines.Add('');
  Memo1.Lines.Add(htExportSubset);
  buClear.Enabled := False;
  buReport.Enabled := False;
  ErrorReport := TStringList.Create;
  ProgressBar1.Min := 0;
  ProgressBar1.Step := 1;
  ProgressBar1.Position := 0;
end;

procedure TfmMaster.buConnectionClick(Sender: TObject);
var
  i: integer;
  s: string;
begin
  if buConnection.Caption = cpConnect
    then
      begin
        Memo1.Clear;
        OpenDialog1.InitialDir := InitialDirectory;
        OpenDialog1.Title := msOpenTargetDatabase;
        OpenDialog1.FileName := '';
        if OpenDialog1.Execute
          then
            begin
              buConnection.Caption := cpDisconnect;
              ADOConnection1.ConnectionString := 'FILE NAME=' + OpenDialog1.FileName;
              DBFilename := ExtractFilename(OpenDialog1.FileName);
              Delete(DBFilename,AnsiPos('.',DBFilename),Length(DBFilename));
              ADOConnection1.Open;
              s := ADOConnection1.ConnectionString;
              i := AnsiPos('Data Source=',s);
              Delete(s,1,i-1);
              i := AnsiPos(';',s);
              Delete(s,i,Length(s));
              StatusBar1.Panels[0].Text := s;
              Memo1.Lines.Add(Format(fsVerifyDateTime,[DateTimeToStr(Now)]));
              Memo1.Lines.Add(s);
              Memo1.Lines.Add('');
              buReport.Enabled := True;
              EnableButtons(True);
            end
          else Exit;
      end
    else
      begin
        ADOConnection1.Close;
        StatusBar1.Panels[0].Text := '';
        buConnection.Caption := cpConnect;
        EnableButtons(False);
      end;
end;

procedure TfmMaster.buReportClick(Sender: TObject);
begin
  if SaveDialog1.Execute then
    Memo1.Lines.SaveToFile(SaveDialog1.FileName);
end;

procedure TfmMaster.buVersionClick(Sender: TObject);
begin
  Memo1.Lines.Add(msDatabaseVersion);
  DBVersion := GetDBVersion;
  if DBVersion = ''
    then Memo1.Lines.Add(msDatabaseVersionNotFound)
    else Memo1.Lines.Add(Format(fsDatabaseVersionFound,[DBVersion]));
  Memo1.Lines.Add('');
end;

procedure TfmMaster.buMetadataClick(Sender: TObject);
var
  tmpCursor: TCursor;
begin
  OpenDialog1.Title := msOpenMasterDatabase;
  if OpenDialog1.Execute then
    begin
      tmpCursor := Screen.Cursor;
      Screen.Cursor := crSQLWait;
      ADOConnection2.ConnectionString := 'FILE NAME=' + OpenDialog1.FileName;
      ADOConnection2.Open;
      Memo1.Lines.Add(Format(fsComparing,['entities']));
      Memo1.Lines.AddStrings(CompareEntities);
      Memo1.Lines.Add('');
      Self.Update;
      Memo1.Lines.Add(Format(fsComparing,['tabledoc']));
      Memo1.Lines.AddStrings(CompareTabledoc);
      Memo1.Lines.Add('');
      Self.Update;
      Memo1.Lines.Add(Format(fsComparing,['columndoc']));
      Memo1.Lines.AddStrings(CompareColumndoc);
      Memo1.Lines.Add('');
      Self.Update;
      Memo1.Lines.Add(Format(fsComparing,['links']));
      Memo1.Lines.AddStrings(CompareLinks);
      Memo1.Lines.Add('');
      Self.Update;
      ADOConnection2.Close;
      Screen.Cursor := tmpCursor;
    end;
end;

procedure TfmMaster.buClearClick(Sender: TObject);
begin
  Memo1.Clear;
  Self.Memo1Change(nil);
end;

procedure TfmMaster.Memo1Change(Sender: TObject);
begin
  buClear.Enabled := Memo1.Lines.Count > 0;
  buReport.Enabled := Memo1.Lines.Count > 0;
end;

procedure TfmMaster.buSchemaClick(Sender: TObject);
var
  tmpCursor: TCursor;
begin
  tmpCursor := Screen.Cursor;
  Screen.Cursor := crSQLWait;
  Memo1.Lines.Add(msKeyFields);
  Memo1.Lines.AddStrings(CheckKeyFields);
  Memo1.Lines.Add('');
  Self.Update;
  if AnsiPos(': 0',Memo1.Lines[Memo1.Lines.Count - 2]) > 0
    then
      begin
        Memo1.Lines.Add(msFieldSize);
        Memo1.Lines.AddStrings(CheckNullFieldsize);
        Memo1.Lines.Add('');
        Self.Update;
        if AnsiPos(': 0',Memo1.Lines[Memo1.Lines.Count - 2]) > 0
          then
            begin
              Memo1.Lines.Add(msFieldOrder);
              Memo1.Lines.AddStrings(CheckFieldOrder);
              Memo1.Lines.Add('');
              Self.Update;
              if AnsiPos(': 0',Memo1.Lines[Memo1.Lines.Count - 2]) > 0
                then
                  begin
                    Memo1.Lines.Add(msConformity);
                    Memo1.Lines.AddStrings(VerifyConformity);
                  end
                else Memo1.Lines.Add(msFixFieldOrder);
            end
          else Memo1.Lines.Add(msFixFieldSize);
      end
    else Memo1.Lines.Add(msFixKeyFields);
  Memo1.Lines.Add('');
  Screen.Cursor := tmpCursor;
  Self.Update;
end;

procedure TfmMaster.buDataClick(Sender: TObject);
var
  tmpCursor: TCursor;
  Metadata: TADOQuery;
begin
  Metadata := TADOQuery.Create(nil);
  try
    Metadata.Connection := ADOConnection1;
    tmpCursor := Screen.Cursor;
    Screen.Cursor := crSQLWait;

    ErrorReport.Clear;
    with Metadata do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select TABLENAME,KEYFIELDS from tabledoc');
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        First;
        ProgressBar1.Position := 0;
        Memo1.Lines.Add(msVerifyPrimaryKeys);
        Self.Update;
        while not Eof do
          begin
            CheckPrimaryKey(FieldByName('TABLENAME').AsString,FieldByName('KEYFIELDS').AsString);
            Next;
          end;
        Close;
      end;
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    with Metadata do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select TABLENAME,FIELDNAME,LTABLENAME,LFIELDNAME ');
        SQL.Add('from columndoc where not LTABLENAME is NULL');
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        First;
        ProgressBar1.Position := 0;
        Memo1.Lines.Add(msVerifyLookupValues);
        Self.Update;
        while not Eof do
          begin
            CheckLookupValue(FieldByName('TABLENAME').AsString,FieldByName('FIELDNAME').AsString,
                             FieldByName('LTABLENAME').AsString,FieldByName('LFIELDNAME').AsString);
            Next;
          end;
        Close;
      end;
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    if GetDBVersion = '3.22' then
      begin
        ErrorReport.Clear;
        with Metadata do
          begin
            Close;
            SQL.Clear;
            SQL.Add('select TABLENAME,FIELDNAME from columndoc ');
            SQL.Add('where FIELDTYPE = ''Yes/No''');
            Open;
            ProgressBar1.Max := RecordCount - 1;
            ProgressBar1.Position := 0;
            Memo1.Lines.Add(msVerifyValidValues);
            Self.Update;
            while not Eof do
              begin
                CheckValues(Fields[0].AsString,Fields[1].AsString,'''T'',''F''');
                Next;
              end;
            Close;
          end;
        Memo1.Lines.AddStrings(ErrorReport);
        Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
        Memo1.Lines.Add('');
        Self.Update;
      end;

    ErrorReport.Clear;
    ProgressBar1.Max := 3;
    ProgressBar1.Position := 0;
    Memo1.Lines.Add(msVerifyValidValues);
    Self.Update;
    CheckValues('DGGEOL','EW','''E'',''W''');
    CheckValues('DGGEOL','NS','''N'',''S''');
    CheckValues('DPGEOL','EW','''E'',''W''');
    CheckValues('DPGEOL','NS','''N'',''S''');
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    with Metadata do
      begin
        SQL.Clear;
        SQL.Add('select FTABLE,PTABLE,FKEYS,PKEYS from links');
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount;
        First;
        ProgressBar1.Position := 0;
        Memo1.Lines.Add(msVerifyForeignKeys);
        Self.Update;
        while not Eof do
          begin
            CheckForeignKey(FieldByName('FTABLE').AsString,FieldByName('PTABLE').AsString,
                            FieldByName('FKEYS').AsString,FieldByName('PKEYS').AsString);
            Next;
          end;
        Close;
      end;
    with ADOQuery1 do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select * from symsrc where symprefno is NULL');
        Open;
        if not Eof then
          ErrorReport.Add(msSymprefno);
        Close;
      end;
    ProgressBar1.StepIt;
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    ProgressBar1.Max := 7;
    ProgressBar1.Position := 0;
    ProgressBar1.Show;
    Memo1.Lines.Add(msVerifyReferenceSource);
    Self.Update;
    CheckSRC('filsrc','F');
    CheckSRC('mapsrc','M');
    CheckSRC('newsrc','NP');
    CheckSRC('papsrc','P');
    CheckSRC('repsrc','R');
    CheckSRC('svolsrc','SV');
    CheckSRC('symsrc','SA');
    CheckSRC('volsrc','V');
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    with Metadata do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select TABLENAME,KEYFIELDS from tabledoc ');
//        SQL.Add('where TABLETYPE <> ''Lookup''');
        SQL.Add('where not TABLETYPE in (''Lookup'',''Junction'')');
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        First;
        ProgressBar1.Position := 0;
        Memo1.Lines.Add(msEmptyRecords);
        Self.Update;
        while not Eof do
          begin
            CheckEmptyRecords(FieldByName('TABLENAME').AsString,
                              FieldByName('KEYFIELDS').AsString);
            Next;
          end;
        Close;
      end;
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    with Metadata do
      begin
        Close;
        SQL.Clear;
        SQL.Add('select TABLENAME,KEYFIELDS from tabledoc ');
        SQL.Add('where (TABLETYPE = ''Data'') and ' +
                '      (TABLEPARENT <> ''[none]'') and ' +
                '      (KEYFIELDS like ''%,RANK'')');
        Open;
        if RecordCount > 0 then
          ProgressBar1.Max := RecordCount - 1;
        First;
        ProgressBar1.Position := 0;
        Memo1.Lines.Add(msLookingForGaps);
        Self.Update;
        while not Eof do
          begin
            CheckGaps(FieldByName('TABLENAME').AsString,
                      FieldByName('KEYFIELDS').AsString);
            Next;
          end;
        Close;
      end;
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

    ErrorReport.Clear;
    Memo1.Lines.Add(Format(fsExtraneousRefx,['prodrefx']));
    Self.Update;
    CheckExtraRefx('PD');
    Memo1.Lines.Add(Format(fsExtraneousRefx,['resrefx']));
    Self.Update;
    CheckExtraRefx('RS');
    Memo1.Lines.Add(Format(fsExtraneousRefx,['minerefx']));
    Self.Update;
    CheckExtraRefx('MN');
    Memo1.Lines.AddStrings(ErrorReport);
    Memo1.Lines.Add(Format(fsErrorsFound,[ErrorReport.Count]));
    Memo1.Lines.Add('');
    Self.Update;

  finally
    Metadata.Close;
    Metadata.Free;
  end;
  ProgressBar1.Position := 0;
  Self.Update;
  Screen.Cursor := tmpCursor;
end;

procedure TfmMaster.buReldateClick(Sender: TObject);
var
  fs: TFormatSettings;
  reldate: string;
  t_name: string;
  i,r: integer;
  eq: TADOQuery;
  tmpCursor: TCursor;
begin
  eq := TADOQuery.Create(nil);
  i := 0;
  r := 0;
  tmpCursor := Screen.Cursor;
  try
    Memo1.Lines.Add(msSetReldate);
    Self.Update;
    fs.ShortDateFormat := 'yyyymmdd';
    fs.LongTimeFormat := 'hhnn';
    reldate := DateToStr(Now,fs) + TimeToStr(Now,fs);
    if MessageDlg(Format(fsSetReldateWarning,[reldate]),mtWarning,[mbOK,mbCancel],0) = mrOK
      then
        begin
          Memo1.Lines.Add(Format(fsSetReldateTo,[reldate]));
          Self.Update;
          Screen.Cursor := crSQLWait;
          eq.Connection := ADOConnection1;
          with ADOQuery1 do
            begin
              Close;
              SQL.Text := 'select tablename from columndoc ' +
                          'where (fieldname = ''RELDATE'') and (tablename not in ' +
                          '(''gq_recordcount'',''dg_summary'',''dp_summary''))';
              Open;
              if RecordCount > 0 then
                ProgressBar1.Max := RecordCount - 1;
              ProgressBar1.Position := 0;
              First;
              while not Eof do
                begin
                  t_name := Fields[0].AsString;
                  eq.SQL.Text := 'update ' + t_name + ' set RELDATE=' + reldate;
                  i := i + eq.ExecSQL;
                  Inc(r);
                  Next;
                  ProgressBar1.StepIt;
                  Self.Update;
                end;
              Close;
            end;
        end
      else Memo1.Lines.Add(msOperationCancelled);
  finally
    eq.Free;
    ProgressBar1.Position := 0;
    Memo1.Lines.Add(Format(fsRecordsAffected,[i,r]));
    Memo1.Lines.Add('');
    Self.Update;
    Screen.Cursor := tmpCursor;
  end;
end;

procedure TfmMaster.buSummaryClick(Sender: TObject);
var
  tmpCursor: TCursor;
  cmd: TADOCommand;
  tl: TStringList;
begin
  Memo1.Lines.Add(msBuildingSummaryTables);
  Self.Update;
  tmpCursor := Screen.Cursor;
  tl := TStringList.Create;
  cmd := TADOCommand.Create(nil);
  try
    Screen.Cursor := crSQLWait;
    Self.Update;
    ADOConnection1.GetTableNames(tl);
    cmd.Connection := ADOConnection1;
    cmd.CommandType := cmdText;
    if tl.IndexOf('gq_recordcount') >= 0 then
      begin
        cmd.CommandText := 'drop table gq_recordcount;';
        cmd.Execute;
      end;
    if tl.IndexOf('dg_summary') >= 0 then
      begin
        cmd.CommandText := 'drop table dg_summary;';
        cmd.Execute;
      end;
    if tl.IndexOf('dp_summary') >= 0 then
      begin
        cmd.CommandText := 'drop table dp_summary;';
        cmd.Execute;
      end;
    PopulateRecordCountTable;
    PopulateSummaryTables;
    Memo1.Lines.Add(msSummaryTablesBuilt);
  finally
    tl.Free;
    cmd.Free;
    Screen.Cursor := tmpCursor;
    Memo1.Lines.Add('');
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.buSQLScriptClick(Sender: TObject);
var
  cmd: TADOCommand;
  qry1: TADOQuery;
  qry2: TADOQuery;
  tmpCursor: TCursor;
  a: array[0..50] of char;
  s: string;
  h: string;
  i,j,k: integer;
  f: TStringList;  // insert script
  t: TStringList;  // list of tables
  tdb: TStringList;
  e: TStringList;  // lines containing character '&'
  QString: string;
  SQLString: string;
  xtl: string;
  version322: boolean;

  procedure LoadSummaryMetadata;
  begin
    cmd.CommandText := 'insert into tabledoc (tablename) values (''gq_recordcount'')';
    cmd.Execute;
    cmd.CommandText := 'insert into tabledoc (tablename) values (''dg_summary'')';
    cmd.Execute;
    cmd.CommandText := 'insert into tabledoc (tablename) values (''dp_summary'')';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''gq_recordcount'',''TABLENAME'',''Text'',1)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''gq_recordcount'',''RECORDCOUNT'',''Number (Long)'',2)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''DGNO'',''Number (Long)'',1)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''COUNTRY'',''Text'',2)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''LOCATION'',''Text'',3)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''ALLNAMES'',''Text'',4)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''COMMODITIES'',''Text'',5)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''DEPCLAN'',''Text'',6)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''LONGDD'',''Number (Double)'',7)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''LATDD'',''Number (Double)'',8)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''DBNAME'',''Text'',9)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''COMPBY'',''Text'',10)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dg_summary'',''RELDATE'',''Text'',11)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''DEPNO'',''Number (Long)'',1)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''COUNTRY'',''Text'',2)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''LOCATION'',''Text'',3)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''ALLNAMES'',''Text'',4)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''COMMODITIES'',''Text'',5)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''DEPCLAN'',''Text'',6)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''LONGDD'',''Number (Double)'',7)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''LATDD'',''Number (Double)'',8)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''DBNAME'',''Text'',9)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''COMPBY'',''Text'',10)';
    cmd.Execute;
    cmd.CommandText := 'insert into columndoc (tablename,fieldname,fieldtype,fieldorder) ' +
                       'values (''dp_summary'',''RELDATE'',''Text'',11)';
    cmd.Execute;
  end;

  function LoadXTL: string;
  var
    qry: TADOQuery;
    cl: TStringList;
  begin
    Result := svc + ',' + lkp + ',' + dta + ',' + sry + ',' + xtr;
    qry := TADOQuery.Create(nil);
    cl := TStringList.Create;
    try
      ADOConnection1.GetFieldNames('TABLEDOC',cl);
      if cl.IndexOf('EXPORTORDER') >= 0 then
        begin
          qry.Connection := ADOConnection1;
          qry.SQL.Text := 'select TABLENAME from TABLEDOC order by EXPORTORDER';
          qry.Open;
          cl.Clear;
          while not qry.Eof do
            begin
              cl.Add(qry.Fields[0].AsString);
              qry.Next;
            end;
          qry.Close;
          cl.Add('gq_recordcount');
          cl.Add('dg_summary');
          cl.Add('dp_summary');
          Result := cl.CommaText;
        end;
    finally
      qry.Close;
      qry.Free;
      cl.Free;
    end;
  end;

begin
//  xtl := svc + ',' + lkp + ',' + dta + ',' + sry + ',' + xtr;
  version322 := GetDBVersion = '3.22';
  xtl := LoadXTL;
  Memo1.Lines.Add(msExporting);
  Self.Update;
  SaveDialog1.Title := msSaveSQLScript;
  SaveDialog1.Filter := 'SQL script (*.sql)|*.sql';
  SaveDialog1.DefaultExt := 'sql';
  SaveDialog1.FileName := DBFileName;
  if SaveDialog1.Execute
    then DBFileName := SaveDialog1.FileName
    else
      begin
        Memo1.Lines.Add(msOperationCancelled);
        Memo1.Lines.Add('');
        Self.Update;
        Exit;
      end;
  Self.Update;
  cmd := TADOCommand.Create(nil);
  qry1 := TADOQuery.Create(nil);
  qry2 := TADOQuery.Create(nil);
  t := TStringList.Create;
  tdb := TStringList.Create;
  f := TStringList.Create;
  tmpCursor := Screen.Cursor;
  try
    Screen.Cursor := crSQLWait;
    buSummaryClick(Self);
    ADOConnection1.GetTableNames(tdb);
    if tdb.Count > 0 then
      ProgressBar1.Max := tdb.Count - 1;
    ProgressBar1.Position := 0;
    Self.Update;
    t.CommaText := xtl;
    cmd.Connection := ADOConnection1;
    cmd.CommandText := 'delete from tabledoc where tablename ' +
                       'in (''gq_recordcount'',''dg_summary'',''dp_summary'');';
    cmd.Execute;
    LoadSummaryMetadata;
    qry1.Connection := ADOConnection1;
    qry2.Connection := ADOConnection1;
    for i := 0 to t.Count - 1 do
      if tdb.IndexOf(t[i]) >= 0 then
        begin
          qry1.SQL.Text := Format('select fieldname,fieldtype,fieldorder ' +
                                  'from columndoc where tablename = ''%s'' ' +
                                  'order by fieldorder',[t[i]]);
          QString := '';
          SQLString := '';
          k := 0;
          qry1.Open;
          while not qry1.Eof do
            begin
              if SQLString <> '' then
                SQLString := SQLString + ',';
              if qry1.FieldByName('FIELDNAME').AsString = 'MODULE'
                then SQLString := SQLString + '[' + qry1.FieldByName('FIELDNAME').AsString + '] as ENTITY'
                else SQLString := SQLString + qry1.FieldByName('FIELDNAME').AsString;
              a[k] := qry1.FieldByName('FIELDTYPE').AsString[1];
              inc(k);
              qry1.Next;
            end;
          qry1.Close;
          qry2.SQL.Clear;
          if (t[i] = 'tabledoc') or (t[i] = 'columndoc')
            then qry2.SQL.Add('select ' + SQLString + ' from ' + t[i] + ' ' +
                              'where TABLENAME not in ' +
                              '(''gq_recordcount'',''dg_summary'',''dp_summary'')')
            else qry2.SQL.Add('select ' + SQLString + ' from ' + t[i]);
          qry2.Open;
          while not qry2.Eof do
            begin
              s := '';
              for j := 0 to qry2.FieldCount - 1 do
                begin
                  if j > 0 then s := s + ',';
                  case a[j] of
                    'N' : if qry2.Fields[j].IsNull
                            then s := s + 'NULL'
                            else s := s + qry2.Fields[j].AsString;
                    'T' : if qry2.Fields[j].IsNull
                            then s := s + 'NULL'
                            else
                              begin
                                h := Copy(qry2.Fields[j].AsString,1,4000);
                                h := AnsiReplaceStr(h,chr(13),' ');
                                h := AnsiReplaceStr(h,chr(10),'');
                                h := AnsiReplaceStr(h,chr(9),'');
                                h := AnsiReplaceStr(h,'''','''''');
                                s := s + '''' + h + '''';
                              end;
                    'Y' : if version322
                            then
                              begin
                                if qry2.Fields[j].IsNull
                                  then h := 'F'
                                  else h := Copy(qry2.Fields[j].AsString,1,1);
                                s := s + '''' + h + '''';
                              end
                            else
                              if qry2.Fields[j].IsNull or not qry2.Fields[j].Value
                                then s := s + '''F'''
                                else s := s + '''T''';
{
                    'Y' : if qry2.Fields[j].IsNull or not qry2.Fields[j].Value
                            then s := s + '''F'''
                            else s := s + '''T''';
}
                  end;
                end;
              if AnsiPos('ENTITY',SQLString) > 0 then
                SQLString := StringReplace(SQLString,'[MODULE] as ENTITY','MODULE',[rfReplaceAll]);
              f.Add('insert into ' + t[i] + ' (' + SQLString + ') values (' + s + ');');
              qry2.Next;
            end;
          qry2.Close;
          f.Add('commit;');
          ProgressBar1.StepIt;
          Self.Update;
        end;
    f.SaveToFile(DBFileName);
    e := TStringList.Create;
    try
      for i := 0 to f.Count - 1 do
        begin
          if AnsiPos('&',f[i]) > 0 then
            e.Add(f[i]);
        end;
      if e.Count > 0 then
        e.SaveToFile(Copy(DBFileName,1,AnsiPos('.',DBFileName) - 1) + '_&.txt');
    finally
      e.Free;
    end;
  finally
    cmd.CommandText := 'delete from tabledoc where tablename ' +
                       'in (''gq_recordcount'',''dg_summary'',''dp_summary'');';
    cmd.Execute;
    Screen.Cursor := tmpCursor;
    cmd.Free;
    qry1.Free;
    qry2.Free;
    Memo1.Lines.Add(Format(fsRecordsExported,[f.Count - 1]));
    Memo1.Lines.Add('');
    ProgressBar1.Position := 0;
    Self.Update;
    f.Free;
    t.Free;
    tdb.Free;
  end;
end;

procedure TfmMaster.buImportClick(Sender: TObject);
begin
  Memo1.Lines.Add(msImporting);
  Self.Update;
  OpenDialog1.InitialDir := InitialDirectory;
  OpenDialog1.Title := msOpenSourceDatabase;
  if OpenDialog1.Execute
    then
      begin
        ADOConnection2.ConnectionString := 'FILE NAME=' + OpenDialog1.FileName;
        ADOConnection2.Open;
        if OKToMerge
          then DoTheMerge
          else Memo1.Lines.Add(msOverlappingKeyValues);
      end
    else
      begin
        ADOConnection2.Close;
        Memo1.Lines.Add(msOperationCancelled);
        Memo1.Lines.Add('');
        Self.Update;
        Exit;
      end;
  ADOConnection2.Close;
  buSummaryClick(Self);
  Memo1.Lines.Add(msMergeCompleted);
  Memo1.Lines.Add('');
  ProgressBar1.Position := 0;
  Self.Update;
end;

procedure TfmMaster.buExportSubsetClick(Sender: TObject);
var
  kv: TStringList;
  tmp: TCursor;
  mt: TADOTable;
  qc: TADOCommand;
  qr: TADOQuery;
  tn: TStringList;
  refnos: TStringList;
  i: integer;
begin
  kv := TStringList.Create;
  tmp := Screen.Cursor;
  mt := TADOTable.Create(nil);
  qc := TADOCommand.Create(nil);
  qr := TADOQuery.Create(nil);
  tn := TStringList.Create;
  refnos := TStringList.Create;
  try
    if not KeyValuesFileOK(kv) then
      Exit;
    if MessageDlg(Format(msExportSubsetWarning1,[kv.Strings[0]]),mtWarning,[mbOK,mbCancel],0) <> mrOK then
      Exit;
    if MessageDlg(msExportSubsetWarning2,mtWarning,[mbOK,mbCancel],0) <> mrOK then
      Exit;
    Screen.Cursor := crSQLWait;
    Memo1.Lines.Add(msExportSubsetDeleteExtraRecords);
    Self.Update;
    mt.Connection := ADOConnection1;
    qc.Connection := ADOConnection1;
    qr.Connection := ADOConnection1;
    if kv.Strings[0] = 'DGNO'
      then
        begin
          mt.TableName := 'dggeol';
          mt.Open;
          while not mt.Eof do
            if kv.IndexOf(mt.FieldByName('DGNO').AsString) < 0
              then mt.Delete
              else mt.Next;
          mt.Close;
          qc.CommandText := 'delete * from dpgeol where DEPNO not in (select DEPNO from dpdg)';
          qc.Execute;
        end
      else
        begin
          mt.TableName := 'dpgeol';
          mt.Open;
          while not mt.Eof do
            if kv.IndexOf(mt.FieldByName('DEPNO').AsString) < 0
              then mt.Delete
              else mt.Next;
          mt.Close;
          qc.CommandText := 'delete * from dggeol where DGNO not in (select DGNO from dpdg)';
          qc.Execute;
        end;
    qc.CommandText := 'delete * from mine where MINENO not in (select MINENO from dpmine)';
    qc.Execute;
    qc.CommandText := 'delete * from prodmain where ' +
                      '  (PRODNO not in (select PRODNO from proddg)) and ' +
                      '  (PRODNO not in (select PRODNO from  proddp)) and ' +
                      '  (PRODNO not in (select PRODNO from prodmine))';
    qc.Execute;
    qc.CommandText := 'delete * from resmain where ' +
                      '  (RESNO not in (select RESNO from resdg)) and ' +
                      '  (RESNO not in (select RESNO from  resdp)) and ' +
                      '  (RESNO not in (select RESNO from resmine))';
    qc.Execute;
    qr.SQL.Text := 'select t.TABLENAME ' +
                   'from tabledoc t inner join columndoc c ' +
                   'on c.TABLENAME = t.TABLENAME ' +
                   'where (c.FIELDNAME = ''REFNO'') and ' +
                   '      (t.TABLEGROUP <> ''References'')';
    qr.Open;
    while not qr.Eof do
      begin
        tn.Add(qr.Fields[0].AsString);
        qr.Next;
      end;
    qr.Close;
    refnos.Sorted := True;
    refnos.Duplicates := dupIgnore;
    for i := 0 to tn.Count - 1 do
      begin
        qr.SQL.Text := Format('select REFNO from %s where REFNO is not null',[tn.Strings[i]]);
        qr.Open;
        while not qr.Eof do
          begin
            refnos.Add(qr.FieldByName('REFNO').AsString);
            qr.Next;
          end;
        qr.Close;
      end;
    mt.TableName := 'refmain';
    mt.Open;
    while not mt.Eof do
      if refnos.IndexOf(mt.FieldByName('REFNO').AsString) < 0
        then mt.Delete
        else mt.Next;
    mt.Close;
    buSummaryClick(Self);
    Memo1.Lines.Add(msExportSubsetCompactDatabase);
    Self.Update;
  finally
    kv.Free;
    mt.Close;
    mt.Free;
    qc.Free;
    qr.Close;
    qr.Free;
    tn.Free;
    refnos.Free;
    Screen.Cursor := tmp;
    Memo1.Lines.Add(msExportSubsetCompleted);
    Memo1.Lines.Add('');
    ProgressBar1.Position := 0;
    Self.Update;
  end;
end;

procedure TfmMaster.buSetCaptionLanguageClick(Sender: TObject);
var
  q: TADOQuery;
  l: string;
begin
  Memo1.Lines.Add(msSettingCaptions);
  Self.Update;
  if fmMaster.LangVersion = 'English'
    then l := 'FCAPTION'
    else l := 'SCAPTION';
  q := TADOQuery.Create(nil);
  try
    q.Connection := ADOConnection1;
    q.SQL.Text := 'update tabledoc set ecaption = ' + l + ';';
    q.ExecSQL;
    q.SQL.Text := 'update columndoc set ecaption = ' + l + ';';
    q.ExecSQL;
  finally
    q.Free;
  end;
    Memo1.Lines.Add(msSettingCaptionsCompleted);
    Memo1.Lines.Add(' ');
    Self.Update;
end;

end.
