unit bldcolumn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Grids, DBGrids, DBCtrls, prepout, Buttons, gqclasses;

type
  TfmBuildOutputColumn = class(TForm)
    paTop: TPanel;
    StatusBar1: TStatusBar;
    buNext: TButton;
    buCancel: TButton;
    gbColumnStyle: TGroupBox;
    rbSingle: TRadioButton;
    rbRepeated: TRadioButton;
    rbConcatenated: TRadioButton;
    lbFields: TListBox;
    laFields: TLabel;
    laSeparators: TLabel;
    laPreviewOutput: TLabel;
    edSampleOutput: TEdit;
    buAddField: TButton;
    gbHeadingSourceTable: TGroupBox;
    laHeading: TLabel;
    edHeading: TEdit;
    laConcatenationSeparator: TLabel;
    edConcatenationSeparator: TEdit;
    buReset: TButton;
    sbRefreshSampleOutput: TSpeedButton;
    meSeparators: TMemo;
    buRemoveField: TButton;
    lbEnclosers: TListBox;
    cbEnclosers: TComboBox;
    buSelectTableAndFields: TButton;
    sbUp: TSpeedButton;
    sbDown: TSpeedButton;
    laSourceTable: TLabel;
    edTable: TEdit;
    laPrefix: TLabel;
    laSuffix: TLabel;
    edPrefix: TEdit;
    edSuffix: TEdit;
    procedure FormShow(Sender: TObject);
    procedure rbConcatenatedClick(Sender: TObject);
    procedure buResetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sbRefreshSampleOutputClick(Sender: TObject);
    procedure buNextClick(Sender: TObject);
    procedure buRemoveFieldClick(Sender: TObject);
    procedure cbEnclosersChange(Sender: TObject);
    procedure lbFieldsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lbEnclosersMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure buSelectTableAndFieldsClick(Sender: TObject);
    procedure buAddFieldClick(Sender: TObject);
    procedure sbUpClick(Sender: TObject);
    procedure sbDownClick(Sender: TObject);
  private
    FieldNames: TStringList;
    FieldNamesLong: TStringList;
    Keys: TStringList;
    TableName: string;
    TableNameLong: string;
    procedure StartNewColumn;
    function GetFieldName(sf: TSearchField): string;
    function GetTableName(st: TSearchTable): string;
    function OkToMove(var lb: TListBox;var n: integer): boolean;
    function RefreshSampleOutput: boolean;
    function VerifyColumn(var ms: string): boolean;
  public
//    BuildMode: string;
    CaptionStyle: integer;
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmBuildOutputColumn: TfmBuildOutputColumn;

implementation

uses dmquery1, mastform, dbtree, selbysql, resources;

{$R *.DFM}

procedure TfmBuildOutputColumn.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

function TfmBuildOutputColumn.GetFieldName(sf: TSearchField): string;
begin
  if CaptionStyle = 0
    then Result := sf.GetName
    else Result := sf.GetCaption;
end;

function TfmBuildOutputColumn.GetTableName(st: TSearchTable): string;
begin
  if CaptionStyle = 0
    then Result := st.GetName
    else Result := st.GetCaption;
end;

function TfmBuildOutputColumn.OkToMove(var lb: TListBox;var n: integer): boolean;
var
  i : integer;
  c : integer;
begin
  Result := False;
  n := -1;
  if lb.Items.Count < 2 then
    Exit;
  c := 0;
  for i := 0 to lb.Items.Count - 1 do
    if lb.Selected[i] then
      begin
        n := i;
        inc(c);
      end;
  if c <> 1 then
    Exit;
  Result := True;
end;

function TfmBuildOutputColumn.RefreshSampleOutput: boolean;
var
  i: integer;
begin
  Result := False;
  if lbFields.Items.Count = 0 then
    begin
      ShowMessage(bldcolumn_NoFieldsInFieldList);
      Exit;
    end;
  while (lbFields.Items.Count - meSeparators.Lines.Count) > 1 do
    meSeparators.Lines.Append('|');
  edSampleOutput.Text := edPrefix.Text;
  for i := 0 to lbFields.Items.Count - 2 do
    edSampleOutput.Text := edSampleOutput.Text + Copy(lbEnclosers.Items[i],1,1) +
                           '[' + lbFields.Items[i] + ']' +
                           Copy(lbEnclosers.Items[i],3,1) + meSeparators.Lines[i];
  edSampleOutput.Text := edSampleOutput.Text + Copy(lbEnclosers.Items[lbEnclosers.Items.Count - 1],1,1) +
                         '[' + lbFields.Items[lbFields.Items.Count - 1] + ']' +
                         Copy(lbEnclosers.Items[lbEnclosers.Items.Count - 1],3,1) + edSuffix.Text;
  if rbConcatenated.Checked then
    edSampleOutput.Text := edSampleOutput.Text + edConcatenationSeparator.Text +
                           edSampleOutput.Text + ' ...';
  edSampleOutput.Text := StringReplace(edSampleOutput.Text,'_',' ',[rfReplaceAll]);
  Result := True;
end;

procedure TfmBuildOutputColumn.StartNewColumn;
begin
  edHeading.Text := '';
  rbSingle.Enabled := False;
  rbSingle.Checked := False;
  rbRepeated.Enabled := False;
  rbRepeated.Checked := False;
  rbConcatenated.Enabled := False;
  rbConcatenated.Checked := False;
  laConcatenationSeparator.Visible := False;
  edConcatenationSeparator.Visible := False;
  edConcatenationSeparator.Text := '';
  edTable.Clear;
  TableName := '';
  TableNameLong := '';
  lbFields.Items.Clear;
  lbEnclosers.Items.Clear;
  cbEnclosers.ItemIndex := 0;
  FieldNames.Clear;
  FieldNamesLong.Clear;
  Keys.Clear;
  meSeparators.Lines.Clear;
  edPrefix.Text := '';
  edSuffix.Text := '';
  edSampleOutput.Text := '';
end;

function TfmBuildOutputColumn.VerifyColumn(var ms: string): boolean;
begin
  if edHeading.Text = '' then
    ms := bldcolumn_NoColumnHeadingEntered;
  if rbConcatenated.Checked and (edConcatenationSeparator.Text = '') then
    begin
      if ms <> '' then
        ms := ms + #10 + #10;
      ms := bldcolumn_NoConcatenationSeparatorEntered;
    end;
  if (lbFields.Items.Count - meSeparators.Lines.Count) > 1 then
    begin
      if ms <> '' then
        ms := ms + #10 + #10;
      ms := ms + bldcolumn_TooFewSeparators;
    end;
  Result := ms = '';
end;

procedure TfmBuildOutputColumn.FormShow(Sender: TObject);
//var
//  i: integer;
//  sl: TStringList;
begin
//  buSelectTableAndFields.Enabled := BuildMode = 'Build';
  buSelectTableAndFields.Enabled := True;
  StartNewColumn;
{
  if BuildMode = 'Build'
    then StartNewColumn
    else with fmPrepareOutput.TransferColumn do
      begin
        edHeading.Text := GetHeading;
        if GetStyle = 'Single'
          then
            begin
              rbSingle.Enabled := True;
              rbSingle.Checked := True;
              rbRepeated.Enabled := False;
              rbRepeated.Checked := False;
              rbConcatenated.Enabled := False;
              rbConcatenated.Checked := False;
              laConcatenationSeparator.Visible := False;
              edConcatenationSeparator.Visible := False;
              edConcatenationSeparator.Text := '';
            end
          else
            begin
              rbSingle.Enabled := False;
              rbSingle.Checked := False;
              rbRepeated.Enabled := True;
              rbRepeated.Checked := GetStyle = 'Repeated';
              rbConcatenated.Enabled := True;
              rbConcatenated.Checked := GetStyle = 'Concatenated';
              laConcatenationSeparator.Visible := rbConcatenated.Checked;
              edConcatenationSeparator.Visible := rbConcatenated.Checked;
              edConcatenationSeparator.Text := GetConcatenationSeparator;
            end;
        TableName := GetTableName;
        TableNameLong := GetTableNameLong;
        if CaptionStyle = 0
          then edTable.Text := TableName
          else edTable.Text := TableNameLong;
        FieldNames.Clear;
        FieldNamesLong.Clear;
        FieldNames.AddStrings(GetFieldList);
        FieldNamesLong.AddStrings(GetFieldListLong);
        Keys.Clear;
        Keys.AddStrings(GetHiKeys);
        Keys.Add(GetLastKey);
        lbFields.Items.Clear;
        if CaptionStyle = 0
          then lbFields.Items.AddStrings(GetFieldList)
          else lbFields.Items.AddStrings(GetFieldListLong);
        lbEnclosers.Items.Clear;
        lbEnclosers.Items.AddStrings(GetEncloserList);
        meSeparators.Lines.Clear;
        sl := TStringList.Create;
        try
          sl.AddStrings(GetSeparatorList);
          for i := 0 to sl.Count - 1 do
            sl.Strings[i] := StringReplace(sl.Strings[i],' ','_',[rfReplaceAll]);
          meSeparators.Lines.AddStrings(sl);
        finally
          sl.Free;
        end;
        edPrefix.Text := GetPrefix;
        edSuffix.Text := GetSuffix;
        RefreshSampleOutput;
      end;
}
end;

procedure TfmBuildOutputColumn.rbConcatenatedClick(Sender: TObject);
begin
  laConcatenationSeparator.Visible := rbConcatenated.Checked;
  edConcatenationSeparator.Visible := rbConcatenated.Checked;
end;

procedure TfmBuildOutputColumn.buResetClick(Sender: TObject);
begin
  StartNewColumn;
end;

procedure TfmBuildOutputColumn.FormCreate(Sender: TObject);
begin
  Self.Caption := bldcolumn_fmBuildOutputColumn;
  gbHeadingSourceTable.Caption := bldcolumn_gbHeadingSourceTable;
  laHeading.Caption := bldcolumn_laHeading;
  laPrefix.Caption := bldcolumn_laPrefix;
  laSuffix.Caption := bldcolumn_laSuffix;
  gbColumnStyle.Caption := bldcolumn_gbColumnStyle;
  rbSingle.Caption := bldcolumn_rbSingle;
  rbRepeated.Caption := bldcolumn_rbRepeated;
  rbConcatenated.Caption := bldcolumn_rbConcatenated;
  laConcatenationSeparator.Caption := bldcolumn_laConcatenationSeparator;
  laSourceTable.Caption := bldcolumn_laSourceTable;
  buSelectTableAndFields.Caption := bldcolumn_buSelectTableAndFields;
  laFields.Caption := bldcolumn_laFields;
  buAddField.Caption := bldcolumn_buAddField;
  buRemoveField.Caption := bldcolumn_buRemoveField;
  laSeparators.Caption := bldcolumn_laSeparators;
  meSeparators.Hint := bldcolumn_meSeparatorsHint;
  laPreviewOutput.Caption := bldcolumn_laPreviewOutput;
  sbRefreshSampleOutput.Hint := bldcolumn_sbRefreshSampleOutputHint;
  buReset.Caption := bldcolumn_buReset;
  buNext.Caption := bldcolumn_buNext;
  buCancel.Caption := bldcolumn_buCancel;
  FieldNames := TStringList.Create;
  FieldNamesLong := TStringList.Create;
  Keys := TStringList.Create;
end;

procedure TfmBuildOutputColumn.FormDestroy(Sender: TObject);
begin
  FieldNames.Free;
  FieldNamesLong.Free;
  Keys.Free;
end;

procedure TfmBuildOutputColumn.sbRefreshSampleOutputClick(Sender: TObject);
begin
  RefreshSampleOutput;
end;

procedure TfmBuildOutputColumn.buNextClick(Sender: TObject);
var
  el: TStringList;
  i: integer;
  ms: string;
  sl: TStringList;
begin
  if lbFields.Items.Count = 0 then
    begin
      MessageDlg(bldcolumn_NoFieldsSelected,mtError,[mbOK],0);
      Self.ModalResult := mrNone;
      Exit;
    end;
  ms := '';
  if not VerifyColumn(ms) then
    begin
      ms := ms + #10 + #10 + bldcolumn_ClickOKOrRetry;
      if MessageDlg(ms,mtWarning,[mbOK,mbRetry],0) = mrRetry then
        begin
          Self.ModalResult := mrNone;
          Exit;
        end;
    end;
  with fmPrepareOutput.TransferColumn do
    begin
      SetHeading(edHeading.Text);
      SetTableName(Tablename);
      SetTableNameLong(TableNameLong);
      if rbSingle.Checked
        then SetStyle(bldcolumn_Single)
        else
          if rbRepeated.Checked
            then SetStyle(bldcolumn_Repeated)
            else
              begin
                SetStyle(bldcolumn_Concatenated);
                SetConcatenationSeparator(edConcatenationSeparator.Text);
              end;
      el := TStringList.Create;
      try
        el.AddStrings(lbEnclosers.Items);
        SetEncloserList(el);
      finally
        el.Free;
      end;
      SetFieldList(FieldNames);
      SetFieldListLong(FieldNamesLong);
      SetKeys(Keys);
      SetPrefix(edPrefix.Text);
      while (lbFields.Items.Count - meSeparators.Lines.Count) > 1 do
        meSeparators.Lines.Append('|');
      sl := TStringList.Create;
      try
        if meSeparators.Lines.Count > 0 then
          for i := 0 to meSeparators.Lines.Count - 1 do
            sl.Add(meSeparators.Lines[i]);
        SetSeparatorList(sl);
      finally
        sl.Free;
      end;
      SetSuffix(edSuffix.Text);
    end;
end;

procedure TfmBuildOutputColumn.buRemoveFieldClick(Sender: TObject);
var
  i: integer;
begin
  if lbFields.ItemIndex < 0 then
    Exit;
  i := lbFields.ItemIndex;
  lbFields.Items.Delete(i);
  lbEnclosers.Items.Delete(i);
  FieldNames.Delete(i);
  FieldNamesLong.Delete(i);
end;

procedure TfmBuildOutputColumn.cbEnclosersChange(Sender: TObject);
begin
  if lbFields.ItemIndex > -1 then
    lbEnclosers.Items[lbFields.ItemIndex] := cbEnclosers.Text;
end;

procedure TfmBuildOutputColumn.lbFieldsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  lbEnclosers.ItemIndex := lbFields.ItemIndex
end;

procedure TfmBuildOutputColumn.lbEnclosersMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  lbFields.ItemIndex := lbEnclosers.ItemIndex
end;

procedure TfmBuildOutputColumn.buSelectTableAndFieldsClick(Sender: TObject);
var
  i: integer;
begin
  if fmDBTree <> nil then
    if fmDBTree.CallingForm <> 'fmBuildOutputColumn' then
      FreeAndNil(fmDBTree);
  if fmDBTree = nil then
    begin
      fmDBTree := TfmDBTree.Create(nil);
      fmDBTree.CallingForm := 'fmBuildOutputColumn';
    end;
  fmDBTree.FormPosition(fmBuildOutputColumn.Top,fmBuildOutputColumn.Left,
                        fmBuildOutputColumn.Height,fmBuildOutputColumn.Width);
  fmDBTree.CaptionStyle := CaptionStyle;
  fmDBTree.ShowModal;
  if SearchFieldCount = 0 then
    Exit;
  Keys.AddStrings(SearchTable.GetKeys);
  TableName := SearchTable.GetName;
  TableNameLong := SearchTable.GetCaption;
  edTable.Text := GetTableName(SearchTable);
  if SearchTable.GetParent = '[none]'
    then
      begin
        rbSingle.Enabled := True;
        rbSingle.Checked := True;
        rbRepeated.Enabled := False;
        rbRepeated.Checked := False;
        rbConcatenated.Enabled := False;
        rbConcatenated.Checked := False;
        edConcatenationSeparator.Enabled := False;
        edConcatenationSeparator.Text := '';
      end
    else
      begin
        rbSingle.Enabled := False;
        rbSingle.Checked := False;
        rbRepeated.Enabled := True;
        rbRepeated.Checked := True;
        rbConcatenated.Enabled := True;
        edConcatenationSeparator.Enabled := True;
      end;
  FieldNames.Clear;
  lbFields.Items.Clear;
  lbEnclosers.Items.Clear;
  meSeparators.Lines.Clear;
  for i := 1 to SearchFieldCount do
    begin
      FieldNames.Add(SearchFields[i].GetName);
      FieldNamesLong.Add(SearchFields[i].GetCaption);
      lbFields.Items.Add(GetFieldName(SearchFields[i]));
      lbEnclosers.Items.Add(cbEnclosers.Items.Strings[0]);
    end;
end;

procedure TfmBuildOutputColumn.buAddFieldClick(Sender: TObject);
var
  i: integer;
begin
  if (FieldNames.Count = 0) or (fmDBTree = nil) or
     ((fmDBTree <> nil) and (fmDBTree.CallingForm <> 'fmBuildOutputColumn')) then
    begin
      fmBuildOutputColumn.buSelectTableAndFieldsClick(Self);
      Exit;
    end;
  fmDBTree.StartingTableName := TableName;
  fmDBTree.FormPosition(fmBuildOutputColumn.Top,fmBuildOutputColumn.Left,
                        fmBuildOutputColumn.Height,fmBuildOutputColumn.Width);
  fmDBTree.CaptionStyle := CaptionStyle;
  fmDBTree.ShowModal;
  if SearchTable.GetName <> TableName then
    begin
      ShowMessage(bldcolumn_MustSelectTableInTree);
      Exit;
    end;
  if SearchFieldCount = 0 then
    Exit;
  for i := 1 to SearchFieldCount do
    if FieldNames.IndexOf(SearchFields[i].GetName) < 0 then
      begin
        FieldNames.Add(SearchFields[i].GetName);
        FieldNamesLong.Add(SearchFields[i].GetCaption);
        lbFields.Items.Add(GetFieldName(SearchFields[i]));
        lbEnclosers.Items.Add(cbEnclosers.Items.Strings[0]);
      end;
end;

procedure TfmBuildOutputColumn.sbUpClick(Sender: TObject);
var
  n: integer;
begin
  if lbFields.Selected[0] then
    Exit;
  if OkToMove(lbFields,n) then
    begin
      lbFields.Items.Move(n,n - 1);
      FieldNames.Move(n,n -1);
      FieldNamesLong.Move(n,n - 1);
      lbFields.Selected[n - 1] := True;
      lbEnclosers.Items.Move(n,n - 1);
      lbEnclosers.Selected[n - 1] := True;
    end;
end;

procedure TfmBuildOutputColumn.sbDownClick(Sender: TObject);
var
  n: integer;
begin
  if lbFields.Selected[lbFields.Items.Count - 1] then
    Exit;
  if OkToMove(lbFields,n) then
    begin
      lbFields.Items.Move(n,n + 1);
      FieldNames.Move(n,n + 1);
      FieldNamesLong.Move(n,n + 1);
      lbFields.Selected[n + 1] := True;
      lbEnclosers.Items.Move(n,n + 1);
      lbEnclosers.Selected[n + 1] := True;
    end;
end;

end.

