unit cmbcond1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CheckLst, ComCtrls, mastform, DBClient, DB;

type
  TfmCombineConditions = class(TForm)
    paBottom: TPanel;
    buNext: TButton;
    buCancel: TButton;
    paTop: TPanel;
    laStep1: TLabel;
    clbSelectConditions: TCheckListBox;
    rbAll: TRadioButton;
    rbAny: TRadioButton;
    rbNot: TRadioButton;
    rbBoolean: TRadioButton;
    rgBoolean: TRadioGroup;
    laJoinDepth: TLabel;
    procedure clbSelectConditionsClickCheck(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure buNextClick(Sender: TObject);
    procedure rbBooleanClick(Sender: TObject);
    procedure rbAllClick(Sender: TObject);
    procedure rbAnyClick(Sender: TObject);
    procedure rbNotClick(Sender: TObject);
    procedure rgBooleanClick(Sender: TObject);
    procedure laJoinDepthClick(Sender: TObject);
  private
    { Private declarations }
    SelectedConditions: TStringList;
    procedure BuildAllSQL;
    procedure BuildAndNotSQL;
    procedure BuildAnySQL;
    procedure BuildNotAndSQL;
    procedure BuildNotSQL;
    procedure CreateCdsCTable;
    function FindCTableRecord(var SourceTable,TestTable: TClientDataSet;
                              j: integer = 0): boolean;
    procedure JoinCheck;
    procedure PopulateCdsTable(var SourceTable,DestinationTable: TClientDataSet;jd: integer);
    procedure PopulateNotTable(tno: integer);
  public
    { Public declarations }
    CCCaptions: TStringList;
    CCKeys: TStringList;
    CCTables: TStringList;
    ConditionShortText: string;
    ConditionLongText: string;
    JoinDepth: integer;
    SQLString: string;
    procedure FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
  end;

var
  fmCombineConditions: TfmCombineConditions;

implementation

uses selrec, dbtree, joindepth, dmquery1, dmglob1, gqclasses, resources;

{$R *.DFM}

var
  CompletedOK: boolean;

procedure TfmCombineConditions.BuildAllSQL;
var
  i: integer;
  m,n: integer;
begin
  m := StrToInt(Copy(SelectedConditions[0],2,2));
  // cration de la table combine
  CreateCdsCTable;
  PopulateCdsTable(Conditions[m].Table,fmSelectRecords.cdsCTable,JoinDepth);
  ConditionShortText := SelectedConditions[0];
  ConditionLongText := cmbcond1_ConditionNo + IntToStr(m);
  // suppression des enregistrements en trop
  for i := 1 to SelectedConditions.Count - 1 do
    begin
      n := StrToInt(Copy(SelectedConditions[i],2,2));
      fmSelectRecords.cdsCTable.Open;
      Conditions[n].Table.Open;
      while not fmSelectRecords.cdsCTable.Eof do
        if not FindCTableRecord(fmSelectRecords.cdsCTable,Conditions[n].Table,JoinDepth)
          then fmSelectRecords.cdsCTable.Delete
          else fmSelectRecords.cdsCTable.Next;
      Conditions[n].Table.Close;
      fmSelectRecords.cdsCTable.Close;
      SQLString := 'OK';
      ConditionShortText := ConditionShortText + ' AND ' + SelectedConditions[i];
      ConditionLongText := ConditionLongText + ' AND ' + cmbcond1_ConditionNo + IntToStr(n);
    end;
end;

procedure TfmCombineConditions.BuildAndNotSQL;
var
  m,n: integer;
begin
  m := StrToInt(Copy(SelectedConditions[0],2,2));
  // cration de la table combine
  CreateCdsCTable;
  PopulateCdsTable(Conditions[m].Table,fmSelectRecords.cdsCTable,JoinDepth);
  ConditionShortText := SelectedConditions[0];
  ConditionLongText := cmbcond1_ConditionNo + IntToStr(m);
  // suppression des enregistrements en trop
  n := StrToInt(Copy(SelectedConditions[1],2,2));
  fmSelectRecords.cdsCTable.Open;
  Conditions[n].Table.Open;
  while not fmSelectRecords.cdsCTable.Eof do
    if FindCTableRecord(fmSelectRecords.cdsCTable,Conditions[n].Table,JoinDepth)
      then fmSelectRecords.cdsCTable.Delete
      else fmSelectRecords.cdsCTable.Next;
  Conditions[n].Table.Close;
  fmSelectRecords.cdsCTable.Close;
  SQLString := 'OK';
  ConditionShortText := ConditionShortText + ' AND (NOT ' + SelectedConditions[1] + ')';
  ConditionLongText := ConditionLongText + ' AND (NOT ' + cmbcond1_ConditionNo + IntToStr(n) + ')';
end;

procedure TfmCombineConditions.BuildAnySQL;
var
  i,j: integer;
  m,n: integer;
begin
  m := StrToInt(Copy(SelectedConditions[0],2,2));
  // cration de la table combine
  CreateCdsCTable;
  PopulateCdsTable(Conditions[m].Table,fmSelectRecords.cdsCTable,JoinDepth);
  ConditionShortText := SelectedConditions[0];
  ConditionLongText := cmbcond1_ConditionNo + IntToStr(m);
  // ajout des enregistrements manquants
  for i := 1 to SelectedConditions.Count - 1 do
    begin
      n := StrToInt(Copy(SelectedConditions[i],2,2));
      fmSelectRecords.cdsCTable.Open;
      Conditions[n].Table.Open;
      while not Conditions[n].Table.Eof do
        begin
          if not FindCTableRecord(Conditions[n].Table,fmSelectRecords.cdsCTable,JoinDepth) then
            begin
              fmSelectRecords.cdsCTable.Append;
              for j := 0 to JoinDepth - 1 do
                fmSelectRecords.cdsCTable.Fields[j].Value := Conditions[n].Table.Fields[j].Value;
              fmSelectRecords.cdsCTable.Post;
            end;
          Conditions[n].Table.Next;
        end;
      Conditions[n].Table.Close;
      fmSelectRecords.cdsCTable.Close;
      SQLString := 'OK';
      ConditionShortText := ConditionShortText + ' OR ' + SelectedConditions[i];
      ConditionLongText := ConditionLongText + ' OR ' + cmbcond1_ConditionNo + IntToStr(n);
    end;
end;

procedure TfmCombineConditions.BuildNotAndSQL;
var
  m,n: integer;
begin
  m := StrToInt(Copy(SelectedConditions[1],2,2));
  // cration de la table combine
  CreateCdsCTable;
  PopulateCdsTable(Conditions[m].Table,fmSelectRecords.cdsCTable,JoinDepth);
  ConditionShortText := '(NOT ' + SelectedConditions[0];
//  ConditionLongText := '(not Condition No.' + IntToStr(m);
  ConditionLongText := '(NOT ' + cmbcond1_ConditionNo + IntToStr(StrToInt(Copy(SelectedConditions[0],2,2)));
  // suppression des enregistrements en trop
  n := StrToInt(Copy(SelectedConditions[0],2,2));
  fmSelectRecords.cdsCTable.Open;
  Conditions[n].Table.Open;
  while not fmSelectRecords.cdsCTable.Eof do
    if FindCTableRecord(fmSelectRecords.cdsCTable,Conditions[n].Table,JoinDepth)
      then fmSelectRecords.cdsCTable.Delete
      else fmSelectRecords.cdsCTable.Next;
  Conditions[n].Table.Close;
  fmSelectRecords.cdsCTable.Close;
  SQLString := 'OK';
  ConditionShortText := ConditionShortText + ') AND ' + SelectedConditions[1];
//  ConditionLongText := ConditionLongText + ') and Condition No.' + IntToStr(n);
  ConditionLongText := ConditionLongText + ') AND ' + cmbcond1_ConditionNo + IntToStr(StrToInt(Copy(SelectedConditions[1],2,2)));
end;

procedure TfmCombineConditions.BuildNotSQL;
var
  i: integer;
  m: integer;
  curs: TCursor;
begin
  curs := Screen.Cursor;
  Screen.Cursor := crSQLWait;
  m := StrToInt(Copy(SelectedConditions[0],2,2));
  // cration de la table combine
  fmSelectRecords.cdsCTable := TClientDataSet.Create(fmSelectRecords);
  with fmSelectRecords.cdsCTable do
    begin
      for i := 0 to Conditions[m].GetKeys.Count - 1 do
        FieldDefs.Add(Conditions[m].GetKey(i),ftInteger,0,True);
      IndexFieldNames := Conditions[m].GetKeyString;
      IndexFieldNames := StringReplace(IndexFieldNames,',',';',[rfReplaceAll]);
      CreateDataSet;
    end;
  PopulateNotTable(fmDBTree.GetTableNameIndex(Conditions[m].GetTable(Conditions[m].GetTables.Count - 1)));

  // suppression des enregistrements en trop
  fmSelectRecords.cdsCTable.Open;
  with Conditions[m].Table do
    begin
      Open;
      while not Eof do
        begin
          if FindCTableRecord(Conditions[m].Table,fmSelectRecords.cdsCTable) then
            fmSelectRecords.cdsCTable.Delete;
          Next;
        end;
      Close;
    end;
  fmSelectRecords.cdsCTable.Close;

  SQLString := 'OK';
  ConditionShortText := cmbcond_NotSelectedConditions;
  ConditionLongText := Format(cmbcond_NotConditionNoFormat,[m]);
  CCCaptions.Clear;
  CCKeys.Clear;
  CCTables.Clear;
  CCCaptions.AddStrings(Conditions[m].GetCaptions);
  CCKeys.AddStrings(Conditions[m].GetKeys);
  CCTables.AddStrings(Conditions[m].GetTables);
  Screen.Cursor := curs;
end;

procedure TfmCombineConditions.FormPosition(const ParentTop,ParentLeft,
  ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

procedure TfmCombineConditions.CreateCdsCTable;
var
  i: integer;
  ks: string;
begin
  fmSelectRecords.cdsCTable := TClientDataSet.Create(fmSelectRecords);
  with fmSelectRecords.cdsCTable do
    begin
      ks := '';
      for i := 0 to JoinDepth - 1 do
        begin
          FieldDefs.Add(CCKeys[i],ftInteger,0,True);
          if i > 0 then
            ks := ks + ';';
          ks := ks + CCKeys[i];
        end;
      IndexFieldNames := ks;
      CreateDataSet;
    end;
end;

function TfmCombineConditions.FindCTableRecord(var SourceTable,TestTable: TClientDataSet;
                                               j: integer = 0): boolean;
var
  jd: integer;
begin
  Result := False;
  if j = 0
    then jd := SourceTable.IndexFieldCount
    else jd := j;
  with SourceTable do
    case jd of
      1 : if TestTable.FindKey([Fields[0].Value]) then
            Result := True;
      2 : if TestTable.FindKey([Fields[0].Value,
                                Fields[1].Value]) then
            Result := True;
      3 : if TestTable.FindKey([Fields[0].Value,
                                Fields[1].Value,
                                Fields[2].Value]) then
            Result := True;
      4 : if TestTable.FindKey([Fields[0].Value,
                                Fields[1].Value,
                                Fields[2].Value,
                                Fields[3].Value]) then
            Result := True;
      5 : if TestTable.FindKey([Fields[0].Value,
                                Fields[1].Value,
                                Fields[2].Value,
                                Fields[3].Value,
                                Fields[4].Value]) then
           Result := True;
      6 : if TestTable.FindKey([Fields[0].Value,
                                Fields[1].Value,
                                Fields[2].Value,
                                Fields[3].Value,
                                Fields[4].Value,
                                Fields[5].Value]) then
            Result := True;
    end;
end;

procedure TfmCombineConditions.JoinCheck;
var
  i,j: integer;
  done,sc: boolean;
  s: string;
begin
  CCCaptions.Clear;
  CCKeys.Clear;
  CCTables.Clear;
  laJoinDepth.Caption := '';
  laJoinDepth.Visible := False;
  JoinDepth := MaxConditions;
  for i := 0 to SelectedConditions.Count - 1 do // nombre maximum de cls pour la jointure
    begin
      j := Conditions[StrToInt(Copy(SelectedConditions[i],2,2))].GetKeys.Count;
      if JoinDepth > j then
        JoinDepth := j;
    end;
  done := False;
  while not done do // nombre maximum de tables communes pour la jointure
    begin
      s := Conditions[StrToInt(Copy(SelectedConditions[0],2,2))].GetTable(JoinDepth - 1);
      sc := True;
      for i := 1 to SelectedConditions.Count - 1 do
        if Conditions[StrToInt(Copy(SelectedConditions[i],2,2))].GetTable(JoinDepth - 1) <> s then
          sc := False;
      if not sc
        then dec(JoinDepth)
        else done := True;
    end;
  if JoinDepth > 1
    then
      begin
        if fmJoinDepth = nil then
          fmJoinDepth := TfmJoinDepth.Create(Self);
        with fmJoinDepth do
          begin
            FormPosition(fmCombineConditions.Top,fmCombineConditions.Left,
                         fmCombineConditions.Height,fmCombineConditions.Width);
            lbJoinTables.Clear;
            for i := 0 to JoinDepth - 1 do
              begin
                j := StrToInt(Copy(SelectedConditions[0],2,2));
                lbJoinTables.Items.Add(Conditions[j].GetCaption(i) + cmdcond1_Table +
                                       Conditions[j].GetTable(i) + ')');
                CCCaptions.Add(Conditions[j].GetCaption(i));
                CCTables.Add(Conditions[j].GetTable(i));
              end;
            ShowModal;
            JoinDepth := SelectedDepth;
          end;
      end
    else
      begin
        j := StrToInt(Copy(SelectedConditions[0],2,2));
        CCCaptions.Add(Conditions[j].GetCaption(0));
        CCTables.Add(Conditions[j].GetTable(0));
      end;
  CCKeys.CommaText := fmDBTree.GetTableKeyString(CCTables[CCTables.Count - 1]);
  while CCCaptions.Count > JoinDepth do
    begin
      CCCaptions.Delete(CCCaptions.Count - 1);
      CCKeys.Delete(CCKeys.Count - 1);
      CCTables.Delete(CCTables.Count - 1);
    end;
  laJoinDepth.Caption := cmdcond1_JoinDepth + IntToStr(JoinDepth);
  laJoinDepth.Visible := True;
end;

procedure TfmCombineConditions.PopulateCdsTable(var SourceTable,DestinationTable: TClientDataSet;
                                                jd: integer);
var
  i: integer;
begin
  SourceTable.Open;
  DestinationTable.Open;
  while not SourceTable.Eof do
    begin
      if not FindCTableRecord(SourceTable,DestinationTable,jd) then
        begin
          DestinationTable.Append;
          for i := 0 to jd - 1 do
            DestinationTable.Fields[i].Value := SourceTable.Fields[i].Value;
          DestinationTable.Post;
        end;
      SourceTable.Next;
    end;
  DestinationTable.Close;
  SourceTable.Close;
end;

procedure TfmCombineConditions.PopulateNotTable(tno: integer);
var
  i: integer;
  k: string;
begin
  with dmGlobalC.FindBySQL do  // peuplement de la table combine
    begin
      Close;
      CommandText := 'select ' + fmDBTree.GetSearchTable(tno).GetSQLKeyString + ' ';
      if fmDBTree.GetSearchTable(tno).GetJunction <> ''
        then CommandText := CommandText + 'from ' + fmDBTree.GetSearchTable(tno).GetJunction + ',' +
                       fmDBTree.GetSearchTable(tno).GetName + ' where ' +
                       '(' + fmDBTree.GetSearchTable(tno).GetName + '.' +
                       fmDBTree.GetSearchTable(tno).GetKey(1) + ' = ' +
                       fmDBTree.GetSearchTable(tno).GetJunction + '.' +
                       fmDBTree.GetSearchTable(tno).GetKey(1) + ')'
        else CommandText := CommandText + 'from ' + fmDBTree.GetSearchTable(tno).GetName;
      Open;
      fmSelectRecords.cdsCTable.Open;
      while not EOF do
        begin
          fmSelectRecords.cdsCTable.Append;
          for i := 0 to fmDBTree.GetSearchTable(tno).GetKeyCount - 1 do
            begin
              k := fmDBTree.GetSearchTable(tno).GetKey(i);
              fmSelectRecords.cdsCTable.FieldByName(k).Value := dmGlobalC.FindBySQL.FieldByName(k).Value;
            end;
          fmSelectRecords.cdsCTable.Post;
          Next;
        end;
      fmSelectRecords.cdsCTable.Close;
      Close;
    end;
end;

procedure TfmCombineConditions.clbSelectConditionsClickCheck(
  Sender: TObject);
var
  i: integer;
begin
  for i := 0 to clbSelectConditions.Items.Count - 1 do
    if clbSelectConditions.State[i] = cbChecked
      then
        if SelectedConditions.IndexOf(clbSelectConditions.Items[i]) < 0
          then SelectedConditions.Add(clbSelectConditions.Items[i])
          else // pour assortir le else avec le bon if
      else if SelectedConditions.IndexOf(clbSelectConditions.Items[i]) >= 0 then
             SelectedConditions.Delete(SelectedConditions.IndexOf(clbSelectConditions.Items[i]));
  rbAll.Checked := False;
  rbAny.Checked := False;
  rbNot.Checked := False;
  rbBoolean.Checked := False;
  rgBoolean.Visible := False;
  laJoinDepth.Visible := False;
  laJoinDepth.Caption := '';
  case SelectedConditions.Count of
    0 : begin
          rbAll.Enabled := False;
          rbAny.Enabled := False;
          rbNot.Enabled := False;
          rbBoolean.Enabled := False;
          rgBoolean.Visible := False;
        end;
    1 : begin
          rbAll.Enabled := False;
          rbAny.Enabled := False;
          rbNot.Enabled := True;
          rbBoolean.Enabled := False;
          rgBoolean.Visible := False;
        end;
    2 : begin
          rbAll.Enabled := True;
          rbAny.Enabled := True;
          rbNot.Enabled := False;
          rbBoolean.Enabled := True;
          rgBoolean.Visible := False;
        end;
    else
      begin
        rbAll.Enabled := True;
        rbAny.Enabled := True;
        rbNot.Enabled := False;
        rbBoolean.Enabled := False;
        rgBoolean.Visible := False;
      end;
  end;
end;

procedure TfmCombineConditions.FormDestroy(Sender: TObject);
begin
  SelectedConditions.Free;
  CCCaptions.Free;
  CCKeys.Free;
  CCTables.Free;
  if fmJoinDepth <> nil then
    FreeAndNil(fmJoinDepth);
end;

procedure TfmCombineConditions.FormCreate(Sender: TObject);
begin
  Self.Caption := cmbcond1_fmCombineConditions;
  rbAll.Caption := cmbcond1_rbAll;
  rbAny.Caption := cmbcond1_rbAny;
  rbNot.Caption := cmbcond1_rbNot;
  rbBoolean.Caption := cmbcond1_rbBoolean;
  rgBoolean.Items.CommaText := cmbcond1_rgBoolean;
  buNext.Caption := cmbcond1_buNext;
  buCancel.Caption := cmbcond1_buCancel;
  SelectedConditions := TStringList.Create;
  SelectedConditions.Sorted := True;
  CCCaptions := TStringList.Create;
  CCKeys := TStringList.Create;
  CCTables := TStringList.Create;
end;

procedure TfmCombineConditions.FormShow(Sender: TObject);
var
  i: integer;
begin
  SelectedConditions.Clear;
  CCCaptions.Clear;
  CCKeys.Clear;
  CCTables.Clear;
  clbSelectConditions.Clear;
  for i := 1 to MaxConditions do
    if (Conditions[i] <> nil) and (i <> fmSelectRecords.CurrentQ) then
      clbSelectConditions.Items.Add(Format('Q%2.2d',[i]));

  buNext.Enabled := False;

  rbAll.Checked := False;
  rbAny.Checked := False;
  rbNot.Checked := False;
  rbBoolean.Checked := False;

  rbAll.Enabled := False;
  rbAny.Enabled := False;
  rbNot.Enabled := False;
  rbBoolean.Enabled := False;

  rgBoolean.Visible := False;

  laJoinDepth.Caption := '';
  laJoinDepth.Visible := False;

  SQLString := '';
  CompletedOK := False;
end;

procedure TfmCombineConditions.buCancelClick(Sender: TObject);
begin
  SQLString := '';
  ConditionShortText := '';
  ConditionLongText := '';
  CCCaptions.Clear;
  CCKeys.Clear;
  CCTables.Clear;
  Close;
end;

procedure TfmCombineConditions.buNextClick(Sender: TObject);
begin
  if not (rbAll.Checked or rbAny.Checked or rbNot.Checked or
         (rbBoolean.Checked and (rgBoolean.ItemIndex >= 0))) then
    if MessageDlg(cmbcond1_YouHaveNotSelectedAnOperation,mtWarning,[mbYes,mbNo],0) = mrNo then
      buCancelClick(Self);
  if rbNot.Checked then
    BuildNotSQL;
  if rbAll.Checked then
    BuildAllSQL;
  if rbAny.Checked then
    BuildAnySQL;
  if rbBoolean.Checked then
    case rgBoolean.ItemIndex of
      0 : BuildAllSQL;
      1 : BuildAnySQL;
      2 : BuildAndNotSQL;
      3 : BuildNotAndSQL;
    end;
  Close;
end;

procedure TfmCombineConditions.rbBooleanClick(Sender: TObject);
begin
  JoinCheck;
  rgBoolean.Items.Clear;
  rgBoolean.Items.Add(Format('%s AND %s',
    [SelectedConditions.Strings[0],SelectedConditions.Strings[1]]));
  rgBoolean.Items.Add(Format('%s OR %s',
    [SelectedConditions.Strings[0],SelectedConditions.Strings[1]]));
  rgBoolean.Items.Add(Format('%s AND (NOT %s)',
    [SelectedConditions.Strings[0],SelectedConditions.Strings[1]]));
  rgBoolean.Items.Add(Format('(NOT %s) AND %s',
    [SelectedConditions.Strings[0],SelectedConditions.Strings[1]]));
  rgBoolean.Visible := True;
end;

procedure TfmCombineConditions.rbAllClick(Sender: TObject);
begin
  rgBoolean.Visible := False;
  JoinCheck;
  buNext.Enabled := True;
end;

procedure TfmCombineConditions.rbAnyClick(Sender: TObject);
begin
  rgBoolean.Visible := False;
  JoinCheck;
  buNext.Enabled := True;
end;

procedure TfmCombineConditions.rbNotClick(Sender: TObject);
begin
  laJoinDepth.Visible := False;
  rgBoolean.Visible := False;
  buNext.Enabled := True;
end;

procedure TfmCombineConditions.rgBooleanClick(Sender: TObject);
begin
  buNext.Enabled := True;
end;

procedure TfmCombineConditions.laJoinDepthClick(Sender: TObject);
begin
  JoinCheck;
end;

end.
