unit dbtree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Db, ImgList, ComCtrls, ExtCtrls, Grids, DBGrids, mastform, gqclasses,
  DBClient, Variants;

type
  TfmDBTree = class(TForm)
    TreeView1: TTreeView;
    ImageList1: TImageList;
    paBottom: TPanel;
    buNext: TButton;
    buCancel: TButton;
    DBGrid1: TDBGrid;
    Splitter1: TSplitter;
    buSelectByKeyValues: TButton;
    buSelectMostRecent: TButton;
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormDestroy(Sender: TObject);
    procedure buNextClick(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBGrid1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TreeView1Click(Sender: TObject);
    procedure TreeView1DblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buSelectByKeyValuesClick(Sender: TObject);
    procedure buSelectMostRecentClick(Sender: TObject);
  private
    AgeSearch: boolean;
    FirstTime: boolean;
    LastCaptionStyle: integer;
    SearchTables: array[0..MaxSearchTables - 1] of TSearchTable;
    TableNames: TStringList;
    procedure ClearAllSearchFields;
    procedure LoadSearchTables;
    procedure LoadTreeView(ParentName: string;ParentNode: TTreeNode);
    procedure ResetQuery(TableGroup: string);
    procedure SetSearchFields;
    procedure SetSearchTable(i: integer);
    procedure ShowNumberOfRecords;
    procedure UpdateFieldGrid(tn: string);
  public
    CaptionStyle: integer;
    CallingForm: string;
    StartingTableName: string;
    procedure FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
    function GetSearchTable(i: integer): TSearchTable;
    function GetTableKeyString(tn: string): string;
    function GetTableNameIndex(tn: string): integer;
  end;

var
  fmDBTree: TfmDBTree;
  SearchTable: TSearchTable;
  SearchFields: array[1..MaxSearchFields] of TSearchField;
  SearchFieldCount: integer;

implementation

uses dmglob1, dmquery1, selrec, startinf, resources;

{$R *.DFM}

procedure TfmDBTree.ClearAllSearchFields;
var
  i: integer;
begin
  for i := 1 to MaxSearchFields do
    if SearchFields[i] <> nil then
      FreeAndNil(SearchFields[i]);
end;

function TfmDBTree.GetSearchTable(i: integer): TSearchTable;
begin
  Result := SearchTables[i];
end;

function TfmDBTree.GetTableKeyString(tn: string): string;
var
  i: integer;
  st: TSearchTable;
begin
  i := GetTableNameIndex(tn);
  st := GetSearchTable(i);
  Result := st.GetKeyString;
end;

function TfmDBTree.GetTableNameIndex(tn: string): integer;
begin
  Result := TableNames.IndexOf(tn);
end;

procedure TfmDBTree.LoadSearchTables;
var
  i: integer;
  CurrentJunctionTable: string;
begin
  CurrentJunctionTable := '';
  dmQuery.cdsTabledoc.Filtered := False;
  dmQuery.cdsTabledoc.Filter := '';
  dmQuery.cdsTabledoc.Open;
  for i := 0 to TableNames.Count - 1 do
    with dmQuery.cdsTabledoc do
      begin
        Locate('TABLENAME',TableNames[i],[]);
        SearchTables[i] := TSearchTable.Create;
        SearchTables[i].SetName(FieldByName('TABLENAME').Value);
        SearchTables[i].SetCaption(FieldByName('ECAPTION').Value);
        SearchTables[i].SetIsJunction(False);
        SearchTables[i].SetJunction(CurrentJunctionTable);
        SearchTables[i].SetKeys(FieldByName('KEYFIELDS').Value);
        SearchTables[i].SetParent(FieldByName('TABLEPARENT').Value);
        SearchTables[i].SetRanked(FieldByName('RANKED').Value);
        if (FieldByName('TABLEGROUP').Value <> e_tablegroup) and
           (FieldByName('TABLETYPE').Value <> 'Junction') then
          SearchTables[i].SetKeys(e_key + ',' + SearchTables[i].GetKeyString);
        if (FieldByName('TABLETYPE').Value = 'Junction') and
           (FieldByName('TABLEPARENT').Value = '[none]') then
          begin
            CurrentJunctionTable := SearchTables[i].GetName;
            SearchTables[i].SetParent(e_table);
            SearchTables[i].SetJunction('');
            SearchTables[i].SetIsJunction(True);
            SearchTables[i].FixJunctionAnomaly(e_key);
          end;
        if (FieldByName('TABLEGROUP').Value <> e_tablegroup) and
           (FieldByName('TABLETYPE').Value = 'Data') and
           (FieldByName('TABLEPARENT').Value = '[none]') then
          with dmQuery.cdsEntities do
            begin
              Open;
              Locate('ENTITY;TABLENAME',VarArrayOf([e_tablegroup,dmQuery.cdsTabledoc.FieldByName('TABLENAME').Value]),[]);
              SearchTables[i].SetParent(FieldByName('EDATA').Value);
              Close;
            end;
        fmStartInfo.pbInfo.StepIt;
      end;
  dmQuery.cdsTabledoc.Close;
end;

procedure TfmDBTree.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

procedure TfmDBTree.LoadTreeView(ParentName: string;ParentNode: TTreeNode);
var
  i,j: integer;
  NewNode: TTreeNode;
  TempStore: TStringList;
begin
  TempStore := TStringList.Create;
  with dmQuery.cdsTabledoc do
    try
      while not EOF do
        begin
          if FieldByName('TABLEPARENT').Value = ParentName
          // la table enfant dpend directement de TABLEPARENT
            then
              begin
                TableNames.Add(FieldByName('TABLENAME').Value);
                if CaptionStyle = 0
                  then NewNode := TreeView1.Items.AddChild(ParentNode,FieldByName('TABLENAME').Value)
                  else NewNode := TreeView1.Items.AddChild(ParentNode,FieldByName('ECAPTION').Value);
                NewNode.ImageIndex := 1;
                NewNode.SelectedIndex := 0;
              end
            else TempStore.Add(FieldByName('TABLENAME').Value);
            // la table enfant ne dpend pas directement de TABLEPARENT
          Next;
        end;
      if CallingForm = 'fmSelectRecords' then
        begin
          j := 0;
          while TempStore.Count > 0 do // traitement des tables restantes
            begin
              Locate('TABLENAME',TempStore.Strings[j],[]);
              i := TableNames.IndexOf(FieldByName('TABLEPARENT').Value);
              if i > -1
              // la table parent est dj dans l'arbre
                then
                  begin
                    TableNames.Insert(i + FieldByName('TABLEORDER').Value,TempStore.Strings[j]);
                    if CaptionStyle = 0
                      then NewNode := TreeView1.Items.AddChild(TreeView1.Items.Item[i],FieldByName('TABLENAME').Value)
                      else NewNode := TreeView1.Items.AddChild(TreeView1.Items.Item[i],FieldByName('ECAPTION').Value);
                    NewNode.ImageIndex := 1;
                    NewNode.SelectedIndex := 0;
                    TempStore.Delete(j);
                  end
                else
                // la table est  un niveau infrieur dans la hirarchie, on passe  la table suivante
                  begin
                    inc(j);
                    if j = TempStore.Count then
                      j := 0;
                  end;
            end;
        end;
    finally
      Close;
      TempStore.Free;
    end;
end;

procedure TfmDBTree.ResetQuery(TableGroup: string);
begin
  with dmQuery.cdsTabledoc do
    begin
      Close;
      Filter := '(TABLETYPE = ''Data'') and (TABLEGROUP = ''' + TableGroup + ''')';
      Filtered := True;
      IndexFieldNames := 'TABLEORDER';
    end;
end;

procedure TfmDBTree.SetSearchFields;
var
  i: integer;
begin
  if CallingForm = 'fmSelectRecords' then
    if AgeSearch then
      begin
        SearchFieldCount := 1;
        if SearchFields[1] = nil then
          SearchFields[1] := TSearchField.Create;
        SearchFields[1].SetSearchType('A');
        Exit;
      end;
  SearchFieldCount := DBGrid1.SelectedRows.Count;
  if SearchFieldCount > 0 then
    begin
      for i := 1 to SearchFieldCount do
        begin
          if SearchFields[i] = nil then
            SearchFields[i] := TSearchField.Create;
          dmQuery.cdsColumndoc.GotoBookmark(pointer(DBGrid1.SelectedRows.Items[i-1]));
          SearchFields[i].SetName(dmQuery.cdsColumndoc.FieldByName('FIELDNAME').Value);
          with dmQuery.cdsColumndoc.FieldByName('ECAPTION') do
            if IsNull
              then SearchFields[i].SetCaption(SearchFields[i].GetName)
              else SearchFields[i].SetCaption(Value);
          if CallingForm = 'fmSelectRecords' then
            with dmQuery.cdsColumndoc do
              if FieldByName('TSEARCH').Value = 'Null'
                then SearchFields[i].SetSearchType('Z')
                else if Copy(FieldByName('FIELDTYPE').Value,1,6) = 'Number'
                       then SearchFields[i].SetSearchType('N')
                       else if Copy(FieldByName('FIELDTYPE').Value,1,4) = 'Text'
                              then if Copy(FieldByName('TSEARCH').Value,1,3) = 'Age'
                                     then SearchFields[i].SetSearchType('A')
                                     else if Copy(FieldByName('TSEARCH').Value,1,5) = 'Query'
                                            then SearchFields[i].SetSearchType('T')
                                            else SearchFields[i].SetSearchType('S')
                              else SearchFields[i].SetSearchType('B');
        end;
    end;
end;

procedure TfmDBTree.SetSearchTable(i: integer);
begin
  with SearchTable do
    begin
      SetName(SearchTables[i].GetName);
      SetCaption(SearchTables[i].GetCaption);
      SetIsJunction(SearchTables[i].GetIsJunction);
      SetJunction(SearchTables[i].GetJunction);
      SetKeys(SearchTables[i].GetKeyString);
      SetParent(SearchTables[i].GetParent);
      SetRanked(SearchTables[i].GetRanked);
    end;
end;

procedure TfmDBTree.ShowNumberOfRecords;
var
  i: integer;
begin
  with dmQuery.cdsTabledoc do
    begin
      Open;
      for i := 0 to TableNames.Count - 1 do
        begin
          Locate('TABLENAME',TableNames.Strings[i],[]);
          TreeView1.Items.Item[i].Text := TreeView1.Items.Item[i].Text +
                                          ' (' + IntToStr(FieldByName('RECORDCOUNT').Value) +
                                          ' ' + dbtree_Records + ')';
          SearchTables[i].SetNumberOfRecords(FieldByName('RECORDCOUNT').Value);
          if FirstTime then
            fmStartInfo.pbInfo.StepIt;
        end;
      Close;
    end;
end;

procedure TfmDBTree.UpdateFieldGrid(tn: string);
begin
  with dmQuery.cdsColumndoc do
    begin
      Close;
      if CaptionStyle = 0
        then DBGrid1.Columns[0].FieldName := 'FIELDNAME'
        else DBGrid1.Columns[0].FieldName := 'ECAPTION';
      DBGrid1.Columns[0].Title.Caption := 'FIELD';
      Filter := '(TABLENAME = ''' + tn + ''')';
      if CallingForm = 'fmSelectRecords'
        then
          if LowerCase(UserInitials) <> 'test' then
            Filter := Filter + ' and (TSEARCH <> ''None'')'
        else
          if LowerCase(UserInitials) <> 'test' then
            Filter := Filter + ' and (TREPORT <> 0)';
      Filtered := True;
      IndexFieldNames := 'FIELDORDER';
      Open;
    end;
end;

procedure TfmDBTree.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  i,n: integer;
begin
  i := TreeView1.Selected.AbsoluteIndex;
// new stuff
  with dmQuery.cdsTabledoc do
    begin
      Open;
      Locate('TABLENAME',SearchTables[i].GetName,[]);
      n := FieldByName('RECORDCOUNT').AsInteger;
      Close;
      if n = 0 then
        begin
          MessageDlg(dbtree_ThisTableIsEmpty,mtWarning,[mbOK],0);
          Exit;
        end;
    end;
// end of new stuff
  with dmQuery.cdsColumndoc do
    begin
      Close;
      Filtered := False;
      Filter := '(TABLENAME = ''' + SearchTables[i].GetName + ''') and ' +
                     '(TSEARCH = ''Agesearch'')';
      Filtered := True;
      Open;
      AgeSearch := RecordCount > 0;
      Close;
    end;
  UpdateFieldGrid(SearchTables[i].GetName);
{
  ShowMessage('Name: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetName + chr(10) +
              'Caption: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetCaption + chr(10) +
              'Parent: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetParent + chr(10) +
              'Junction: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetJunction + chr(10) +
              'Ranked: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetRankedYesNo + chr(10) +
              'KeyFields: ' + SearchTables[TreeView1.Selected.AbsoluteIndex].GetKeyString + chr(10) +
              'KeyCount: ' + IntToStr(SearchTables[TreeView1.Selected.AbsoluteIndex].GetKeyCount));
}
end;

procedure TfmDBTree.FormDestroy(Sender: TObject);
var
  i: integer;
begin
  if SearchTable <> nil then
    FreeAndNil(SearchTable);
  ClearAllSearchFields;
  if TableNames <> nil then
    begin
      TableNames.Free;
      for i := 0 to MaxSearchTables - 1 do
        FreeAndNil(SearchTables[i]);
    end;
  dmQuery.cdsTabledoc.Close;
  dmQuery.cdsEntities.Close;
end;

procedure TfmDBTree.buNextClick(Sender: TObject);
begin
  SetSearchTable(Tablenames.IndexOf(dmQuery.cdsColumndoc.FieldByName('TABLENAME').Value));
  SetSearchFields;
  Close;
end;

procedure TfmDBTree.buCancelClick(Sender: TObject);
begin
  SearchTable.SetName('');
  SearchTable.SetCaption('');
  SearchFieldCount := 0;
  Close;
end;

procedure TfmDBTree.FormShow(Sender: TObject);
var
  entities: TClientDataSet;
  tabledoc: TClientDataSet;
  NewNode: TTreeNode;
  RootNode: TTreeNode;
  Parent: string;
  tmpCursor: TCursor;
  i: integer;
//  tmpTreeData: TStringList;
begin
  AgeSearch := False;
  buNext.Enabled := False;
  buSelectByKeyValues.Enabled := CallingForm = 'fmSelectRecords';
  buSelectMostRecent.Enabled := CallingForm = 'fmSelectRecords';
  if TreeView1.Items.Count > 0
    then
      begin
        if CaptionStyle <> LastCaptionStyle then
          begin
            if CaptionStyle = 0
              then for i := 0 to TreeView1.Items.Count - 1 do
                     TreeView1.Items.Item[i].Text := SearchTables[i].GetName
              else for i := 0 to TreeView1.Items.Count - 1 do
                     TreeView1.Items.Item[i].Text := SearchTables[i].GetCaption;
            ShowNumberOfRecords;
            LastCaptionStyle := CaptionStyle;
          end;
        if StartingTableName = ''
          then
            begin
              TreeView1.FullCollapse;
              TreeView1.Selected := TreeView1.Items[0];
              UpdateFieldGrid(TableNames[0]);
            end
          else
            begin
              i := GetTableNameIndex(StartingTableName);
              StartingTableName := '';
              TreeView1.Selected := TreeView1.Items[i];
              UpdateFieldGrid(TableNames[i]);
              DBGrid1.SetFocus;
            end;
      end
    else  // premire fois, l'arbre est vide
      begin
        fmStartInfo.FormPosition(fmDBTree.Top,fmDBTree.Left,
                                 fmDBTree.Height,fmDBTree.Width);
        fmStartInfo.Label1.Caption := dbtree_BuildingTreeView;
        fmStartInfo.Show;
        fmStartInfo.Update;
        tmpCursor := Screen.Cursor;
        Screen.Cursor := crSQLWait;
        TreeView1.Items.Clear;
        if TableNames <> nil then
          FreeAndNil(TableNames);
        TableNames := TStringList.Create;
        entities := dmQuery.cdsEntities;
        tabledoc := dmQuery.cdsTabledoc;
        with dmGlobalC do
          begin
            dmQuery.cdsColumndoc.Close;
            entities.Close;
            entities.Filter := 'ENTITY = ''' + e_tablegroup + '''';
            entities.Filtered := True;
            entities.Open;
            TableNames.Add(entities.FieldByName('TABLENAME').Value); // table matresse de l'entit racine
            Parent := entities.FieldByName('TABLENAME').Value;
            ResetQuery(entities.FieldByName('TABLEGROUP').Value);
            tabledoc.Open; // requte-liste des tables de l'entit racine
            if CaptionStyle = 0
              then RootNode := TreeView1.Items.Add(nil,tabledoc.FieldByName('TABLENAME').Value) // TableName de la table matresse
              else RootNode := TreeView1.Items.Add(nil,tabledoc.FieldByName('ECAPTION').Value); // ECaption de la table matresse
            RootNode.ImageIndex := 1;
            RootNode.SelectedIndex := 0;
            tabledoc.Next;
            LoadTreeView(Parent,RootNode); // chargement de l'arbre pour l'entit racine
            if CallingForm = 'fmSelectRecords' then
              begin
                entities.Next;
                while not entities.Eof do // chargement des sous-arbres pour chacune des autres entits
                  begin
                    TableNames.Add(entities.FieldByName('EDATA').Value); // nom de la table associative
                    if CaptionStyle = 0
                      then NewNode := TreeView1.Items.AddChild(RootNode,entities.FieldByName('EDATA').Value) // ECaption de la table associative
                      else NewNode := TreeView1.Items.AddChild(RootNode,entities.FieldByName('ECAPTION').Value); // ECaption de la table associative
                    NewNode.ImageIndex := 1;
                    NewNode.SelectedIndex := 0;
                    Parent := entities.FieldByName('TABLENAME').Value; // table matresse de l'entit courante
                    ResetQuery(entities.FieldByName('TABLEGROUP').Value);
                    tabledoc.Open; // requte-liste des tables de l'entit courante
                    TableNames.Add(tabledoc.FieldByName('TABLENAME').Value);
                    if CaptionStyle = 0
                      then NewNode := TreeView1.Items.AddChild(NewNode,tabledoc.FieldByName('TABLENAME').Value) // table matresse de l'entit courante
                      else NewNode := TreeView1.Items.AddChild(NewNode,tabledoc.FieldByName('ECAPTION').Value); // table matresse de l'entit courante
                    NewNode.ImageIndex := 1;
                    NewNode.SelectedIndex := 0;
                    tabledoc.Next; // requte-liste des tables de l'entit courante
                    LoadTreeView(Parent,NewNode); // chargement du sous-arbre de l'entit courante
                    entities.Next;
                  end;
                fmDBTree.ActiveControl := TreeView1;
              end;
          end;
        fmStartInfo.pbInfo.Min := 0;
        fmStartInfo.pbInfo.Step := 1;
        fmStartInfo.pbInfo.Max := 2 * TableNames.Count;
        fmStartInfo.pbInfo.Position := 0;
        LoadSearchTables;
        ShowNumberOfRecords;
        Screen.Cursor := tmpCursor;
        LastCaptionStyle := CaptionStyle;
        fmStartInfo.Hide;
        fmDBTree.Update;
      end;
  FirstTime := False;
{ ---> Full dump of tree data
  tmpTreeData := TStringList.Create;
  try
    tmpTreeData.Add('Caption' + #9 +
                    'IsJunctionYesNo' + #9 +
                    'KeyCount' + #9 +
                    'KeyString' + #9 +
                    'Name' + #9 +
                    'NumberOfRecords' + #9 +
                    'Parent' + #9 +
                    'RankedYesNo' + #9 +
                    'SQLKeyString');
    for i := 0 to TableNames.Count - 1 do
      with SearchTables[i] do
        begin
          tmpTreeData.Add(GetCaption + #9 +
                          GetIsJunctionYesNo + #9 +
                          IntToStr(GetKeyCount) + #9 +
                          GetKeyString + #9 +
                          GetName + #9 +
                          IntToStr(GetNumberOfRecords) + #9 +
                          GetParent + #9 +
                          GetRankedYesNo + #9 +
                          GetSQLKeyString);
        end;
    tmpTreeData.SaveToFile('dbtree_data.txt');
  finally
    tmpTreeData.Free;
  end;
}
end;

procedure TfmDBTree.FormCreate(Sender: TObject);
begin
  Self.Caption := dbtree_fmDBTree;
  buSelectByKeyValues.Caption := dbtree_buSelectByKeyValues;
  buSelectMostRecent.Caption := dbtree_buSelectMostRecent;
  buNext.Caption := dbtree_buNext;
  buCancel.Caption := dbtree_buCancel;
  StartingTableName := '';
  SearchTable := TSearchTable.Create;
  LastCaptionStyle := -1;
  CallingForm := '';
  FirstTime := True;
end;

procedure TfmDBTree.DBGrid1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  buNext.Enabled := True;
end;

procedure TfmDBTree.TreeView1Click(Sender: TObject);
begin
  buNext.Enabled := False;
end;

procedure TfmDBTree.TreeView1DblClick(Sender: TObject);
begin
  if LowerCase(UserInitials) = 'test' then
    ShowMessage(dbtree_Name + SearchTables[TreeView1.Selected.AbsoluteIndex].GetName + chr(10) +
                dbtree_Caption + SearchTables[TreeView1.Selected.AbsoluteIndex].GetCaption + chr(10) +
                dbtree_Parent + SearchTables[TreeView1.Selected.AbsoluteIndex].GetParent + chr(10) +
                dbtree_Junction + SearchTables[TreeView1.Selected.AbsoluteIndex].GetJunction + chr(10) +
                dbtree_Ranked + SearchTables[TreeView1.Selected.AbsoluteIndex].GetRankedYesNo + chr(10) +
                dbtree_KeyFields + SearchTables[TreeView1.Selected.AbsoluteIndex].GetKeyString + chr(10) +
                dbtree_KeyCount + IntToStr(SearchTables[TreeView1.Selected.AbsoluteIndex].GetKeyCount));
end;

procedure TfmDBTree.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with dmQuery do
    begin
      cdsEntities.Close;
      cdsEntities.Filtered := False;
      cdsEntities.Filter := '';
      cdsTabledoc.Close;
      cdsTabledoc.Filtered := False;
      cdsTabledoc.Filter := '';
      cdsColumndoc.Close;
      cdsColumndoc.Filtered := False;
      cdsColumndoc.Filter := '';
    end;
end;

procedure TfmDBTree.buSelectByKeyValuesClick(Sender: TObject);
begin
  SetSearchTable(0);
  SearchFieldCount := 1;
  if SearchFields[1] = nil then
    SearchFields[1] := TSearchField.Create;
  SearchFields[1].SetName(e_key);
  SearchFields[1].SetCaption(e_key);
  SearchFields[1].SetSearchType('K');
  Close;
end;

procedure TfmDBTree.buSelectMostRecentClick(Sender: TObject);
begin
  SetSearchTable(0);
  SearchFieldCount := 1;
  if SearchFields[1] = nil then
    SearchFields[1] := TSearchField.Create;
  SearchFields[1].SetName(e_key);
  SearchFields[1].SetCaption(e_key);
  SearchFields[1].SetSearchType('R');
  Close;
end;

end.
