unit genout;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, gqclasses, Variants, DBClient;

type
  TfmGenerateOutput = class(TForm)
    paTop: TPanel;
    SaveDialog1: TSaveDialog;
    buGenerate: TButton;
    buSummary: TButton;
    buMainScreen: TButton;
    procedure buMainScreenClick(Sender: TObject);
    procedure buSummaryClick(Sender: TObject);
    procedure buGenerateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    function OKToProceed: boolean;
    procedure GenerateHeaders;
    procedure GenerateDataLines;
    function GetSingle(const oc: TOutputColumn): string;
    function GetMultiple(const oc: TOutputColumn;
      const sep: string): string;
    procedure SetMaxAndTypes;
  public
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmGenerateOutput: TfmGenerateOutput;

implementation

uses dmquery1, dmglob1, mastform, prepout, startinf, resources;

var
  ASCII_file: TextFile;
  ColumnCount: integer;

{$R *.DFM}

procedure TfmGenerateOutput.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

procedure TfmGenerateOutput.GenerateDataLines;
var
  ASCII_line: string;
  i,j: integer;
  tmpCursor : TCursor;
begin
  tmpCursor := Screen.Cursor;
  Screen.Cursor := crSQLWait;
  dmQuery.cdsQSummary.First;
  while not dmQuery.cdsQSummary.EOF do
    begin
      ASCII_line := IntToStr(dmQuery.cdsQSummary.FieldByName(e_key).Value);
      i := 1;
      j := 1;
      while (j <= fmPrepareOutput.OutputColumnCount) do
        begin
          if fmPrepareOutput.OutputColumns[i] <> nil then
            with fmPrepareOutput.OutputColumns[i] do
              begin
                if GetStyle = bldcolumn_Repeated
                  then ASCII_line := ASCII_line + #9 + GetMultiple(fmPrepareOutput.OutputColumns[i],#9)
                  else
                    if GetStyle = bldcolumn_Concatenated
                      then ASCII_line := ASCII_line + #9 +
                                         GetMultiple(fmPrepareOutput.OutputColumns[i],
                                             fmPrepareOutput.OutputColumns[i].GetConcatenationSeparator)
                      else ASCII_line := ASCII_line + #9 + GetSingle(fmPrepareOutput.OutputColumns[i]);
                inc(j);
              end;
          inc(i);
        end;
      ASCII_line := ASCII_line + #9 + dbname + #9 + compby + #9 + reldate;
      Writeln(ASCII_file,ASCII_line);
      fmStartInfo.pbInfo.StepIt;
      dmQuery.cdsQSummary.Next;
    end;
  Screen.Cursor := tmpCursor;
end;

procedure TfmGenerateOutput.GenerateHeaders;
var
  ASCII_line: string;
  i,j,k: integer;
begin
  AssignFile(ASCII_file,SaveDialog1.FileName);
  Rewrite(ASCII_file);
  ASCII_line := e_key;
  ColumnCount := 1;
  i := 1;
  j := 1;
  while (j <= fmPrepareOutput.OutputColumnCount) do
    begin
      if fmPrepareOutput.OutputColumns[i] <> nil then
        with fmPrepareOutput.OutputColumns[i] do
          begin
            if GetHeading = '' then
              SetHeading(Format('C%5d5',[Random(30000)]));
            if GetStyle = bldcolumn_Repeated
              then
                begin
                  for k := 0 to GetMaxRankNo - 1 do
                    begin
                      ASCII_line := ASCII_line + #9 + Format(GetHeading + ' %d',[k+1]);
                      inc(ColumnCount);
                    end;
                end
              else  // 'Concatenated' or 'Single'
                begin
                  ASCII_line := ASCII_line + #9 + GetHeading;
                  inc(ColumnCount);
                end;
            inc(j);
          end;
      inc(i);
    end;
  ASCII_line := ASCII_line + #9 + 'DBNAME' + #9 + 'COMPBY' + #9 + 'RELDATE';
  Writeln(ASCII_file,ASCII_line);
end;

function TfmGenerateOutput.GetMultiple(const oc: TOutputColumn; const sep: string): string;
var
  fs: array of string;
  i: integer;
  separator: string;
  SQLString: string;
  ts: string;
begin
  Result := '';
  separator := StringReplace(sep,'_',' ',[rfReplaceAll]);
  SQLString := 'select ' + oc.GetFieldName(0);
  i := 1;
  while (i < oc.FieldListCount) do
    begin
      SQLString := SQLString + ',' + oc.GetFieldName(i);
      inc(i);
    end;
  SQLString := SQLString + ' from ' + oc.GetTableName;
  SQLString := SQLString + ' where ' + e_key + ' = ' +
               IntToStr(dmQuery.cdsQSummary.FieldByName(e_key).Value);
  SQLString := SQLString + ' order by ' + oc.GetLastKey;
  SetLength(fs,oc.FieldListCount);
  with dmGlobalC.FindStuff do
    begin
      CommandText := SQLString;
      Open;
      while not EOF do
        begin
          ts := '';
          for i := 0 to oc.FieldListCount - 1 do
            begin
              if FieldByName(oc.GetFieldName(i)).IsNull
                then fs[i] := ''
                else
                  case oc.GetFieldType(i) of
                    'I' : fs[i] := IntToStr(FieldByName(oc.GetFieldName(i)).Value);
                    'F' : fs[i] := FloatToStrF(FieldByName(oc.GetFieldName(i)).Value,ffGeneral,7,8);
                    'S' : fs[i] := FieldByName(oc.GetFieldName(i)).Value;
                    'B' : if dmGlobalC.IsTrue(FieldByName(oc.GetFieldName(i)).Value)
                            then fs[i] := 'Yes'
                            else fs[i] := 'No';
                  end;
              if (fs[i] <> '') and (oc.GetEncloserList.Strings[i] <> '') then
                begin
                  fs[i] := Copy(oc.GetEncloserList.Strings[i],1,1) + fs[i];
                  fs[i] := fs[i] + Copy(oc.GetEncloserList.Strings[i],3,1);
                end;
              if i = 0
                then ts := fs[0]
                else
                  if fs[i] <> ''
                    then ts := ts + oc.GetSeparatorList.Strings[i - 1] + fs[i];
            end;
          if ts <> '' then
            ts := oc.GetPrefix + ts + oc.GetSuffix;
          if BOF
            then Result := ts
            else Result := Result + separator + ts;
          Next;
        end;
      if oc.GetStyle = bldcolumn_Repeated then
        if Result = ''
          then Result := StringOfChar(#9,oc.GetMaxRankNo - 1)
          else
            if (RecordCount < oc.GetMaxRankNo) then
              for i := RecordCount + 1 to oc.GetMaxRankNo do
                Result := Result + #9;
      Close;
    end;
end;

function TfmGenerateOutput.GetSingle(const oc: TOutputColumn): string;
var
  fs: array of string;
  i: integer;
  SQLString: string;
begin
  Result := '';
  SQLString := 'select ' + oc.GetFieldName(0);
  i := 1;
  while (i < oc.FieldListCount) do
    begin
      SQLString := SQLString + ',' + oc.GetFieldName(i);
      inc(i);
    end;
  SQLString := SQLString + ' from ' + oc.GetTableName;
  SQLString := SQLString + ' where ' + e_key + ' = ' + IntToStr(dmQuery.cdsQSummary.FieldByName(e_key).Value);
  SetLength(fs,oc.FieldListCount);
  with dmGlobalC.FindStuff do
    begin
      CommandText := SQLString;
      Open;
      for i := 0 to oc.FieldListCount - 1 do
        begin
          if FieldByName(oc.GetFieldName(i)).IsNull
            then fs[i] := ''
            else
              case oc.GetFieldType(i) of
                'I' : fs[i] := IntToStr(FieldByName(oc.GetFieldName(i)).Value);
                'F' : fs[i] := FloatToStrF(FieldByName(oc.GetFieldName(i)).Value,ffGeneral,7,8);
                'S' : fs[i] := FieldByName(oc.GetFieldName(i)).Value;
                'B' : if dmGlobalC.IsTrue(FieldByName(oc.GetFieldName(i)).Value)
                        then fs[i] := 'Yes'
                        else fs[i] := 'No';
              end;
          if (fs[i] <> '') and (oc.GetEncloserList.Strings[i] <> '') then
            begin
              fs[i] := Copy(oc.GetEncloserList.Strings[i],1,1) + fs[i];
              fs[i] := fs[i] + Copy(oc.GetEncloserList.Strings[i],3,1);
            end;
          if i = 0
            then Result := fs[0]
            else
              if fs[i] <> ''
                then Result := Result + oc.GetSeparatorList.Strings[i - 1] + fs[i];
        end;
      Close;
    end;
  if Result <> '' then
    Result := oc.GetPrefix + Result + oc.GetSuffix;
end;

function TfmGenerateOutput.OKToProceed: boolean;
begin
  Result := False;
  if dmQuery.cdsQSummary.RecordCount < 1 then
    begin
      ShowMessage(genout_NoRecordsSelectedForOutput);
      Exit;
    end;
  if fmPrepareOutput.OutputColumnCount < 1 then
    begin
      ShowMessage(genout_NoOutputColumnsDefined);
      Exit;
    end;
  if not SaveDialog1.Execute then
    Exit;
  Result := True;
end;

procedure TfmGenerateOutput.SetMaxAndTypes;
var
  i,j,k: integer;
  ft,t: string;
  MaxRank: TAggregate;
begin
  i := 1;
  j := 1;
  while (j <= fmPrepareOutput.OutputColumnCount) do
    begin
      if fmPrepareOutput.OutputColumns[i] <> nil then
        with fmPrepareOutput.OutputColumns[i] do
          begin
            if GetStyle = bldcolumn_Repeated
              then
                with dmGlobalC.FindStuff do
                  begin
                    CommandText := 'select count(*) as MAXRANK from ' + GetTableName +
                                   ' group by ' + GetHiKeys.CommaText;
                    Open;
                    if Eof
                      then SetMaxRankNo(1)
                      else
                        begin
                          MaxRank := Aggregates.Add;
                          MaxRank.Expression := 'Max(MAXRANK)';
                          MaxRank.Active := True;
                          SetMaxRankNo(MaxRank.Value);
                          MaxRank.Active := False;
                          MaxRank.Destroy;
                        end;
                    Close;
                  end
              else SetMaxRankNo(1);
            ft := '';
            dmQuery.cdsColumndoc.Open;
            for k := 0 to FieldListCount - 1 do
              begin
                dmQuery.cdsColumndoc.Locate('TABLENAME;FIELDNAME',VarArrayOf([GetTablename,GetFieldName(k)]),[]);
                t := dmQuery.cdsColumndoc.FieldByName('FIELDTYPE').Value;
                if (t = 'Number (Long)') or (t = 'Number (Integer)')
                  then ft := ft + 'I'
                  else
                    if (t = 'Number (Single)') or (t = 'Number (Double)')
                      then ft := ft + 'F'
                      else
                        if t = 'Text'
                          then ft := ft + 'S'
                          else ft := ft + 'B';  // yes-no
              end;
            dmQuery.cdsColumndoc.Close;
            SetFieldType(ft);
            inc(j);
          end;
      inc(i);
    end;
end;

procedure TfmGenerateOutput.buMainScreenClick(Sender: TObject);
begin
  Close;
end;

procedure TfmGenerateOutput.buSummaryClick(Sender: TObject);
var
  ASCII_line : string;
begin
  if not SaveDialog1.Execute then
    Exit;
  AssignFile(ASCII_file,SaveDialog1.FileName);
  Rewrite(ASCII_file);
  ASCII_line := e_key + #9 + 'LONGITUDE' + #9 + 'LATITUDE' + #9 + 'COUNTRY' + #9 +
                'LOCATION' + #9 + 'ALLNAMES' + #9 + 'COMMODITIES' + #9 +
                'DEPCLAN' + #9 + 'DBNAME' + #9 + 'COMPBY' + #9 + 'RELDATE';
  Writeln(ASCII_file,ASCII_line);
  with dmQuery.cdsQSummary do
    begin
      First;
      while not EOF do
        begin
          ASCII_line := IntToStr(FieldByName(e_key).Value);
          if FieldByName('LONGDD').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + Format('%10.6f',[FieldByName('LONGDD').AsFloat]);
          if FieldByName('LATDD').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + Format('%9.6f',[FieldByName('LATDD').AsFloat]);
          if FieldByName('COUNTRY').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('COUNTRY').Value;
          if FieldByName('LOCATION').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('LOCATION').Value;
          if FieldByName('ALLNAMES').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('ALLNAMES').Value;
          if FieldByName('COMMODITIES').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('COMMODITIES').Value;
          if FieldByName('DEPCLAN').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('DEPCLAN').Value;
          if FieldByName('DBNAME').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('DBNAME').Value;
          if FieldByName('COMPBY').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('COMPBY').Value;
          if FieldByName('RELDATE').IsNull
            then ASCII_line := ASCII_line + #9
            else ASCII_line := ASCII_line + #9 + FieldByName('RELDATE').Value;
          Writeln(ASCII_file,ASCII_line);
          Next;
        end;
      CloseFile(ASCII_file);
      First;
    end;
  ShowMessage(genout_ASCIIFileSavedAs + SaveDialog1.FileName);
end;

procedure TfmGenerateOutput.buGenerateClick(Sender: TObject);
begin
  if not OKToProceed then
    Exit;
  SetMaxAndTypes;
  GenerateHeaders;
  fmGenerateOutput.Update;
  fmStartInfo.Label1.Caption := genout_GeneratingOutputMatrix;
  fmStartInfo.FormPosition(fmGenerateOutput.Top,fmGenerateOutput.Left,fmGenerateOutput.Height,fmGenerateOutput.Width);
  fmStartInfo.pbInfo.Min := 0;
  fmStartInfo.pbInfo.Step := 1;
  fmStartInfo.pbInfo.Max := dmQuery.cdsQSummary.RecordCount;
  fmStartInfo.Show;
  fmStartInfo.Update;
  GenerateDataLines;
  CloseFile(ASCII_file);
  fmStartInfo.Hide;
  ShowMessage(Format(genout_OutputGenerated,[ColumnCount,dmQuery.cdsQSummary.RecordCount]));
end;

procedure TfmGenerateOutput.FormCreate(Sender: TObject);
begin
  Self.Caption := genout_fmGenerateOutput;
  buGenerate.Caption := genout_buGenerate;
  buSummary.Caption := genout_buSummary;
  buMainScreen.Caption := genout_buMainScreen;
  Randomize;
end;

end.
