unit gqclasses;

interface

uses
  classes, dialogs, sysutils, dbclient, forms;

type
  TCondition = class(TObject)
    private
      ConditionType: char;
      Keys: TStringList;
      Tables: TStringList;
      Captions: TStringList;
      ShortText: string;
      LongText: string;
      Hits: string;
    public
      Table: TClientDataSet;
      constructor Create(const i: integer;c: char;f: TForm);
      destructor Destroy; override;
      function GetCaption(i: integer): string;
      function GetCaptions: TStrings;
      function GetHits: string;
      function GetKey(i: integer): string;
      function GetKeys: TStrings;
      function GetNKeyString(i: integer): string;
      function GetKeyString: string;
      function GetLongText: string;
      function GetShortText: string;
      function GetTable(i: integer): string;
      function GetTables: TStrings;
      function GetType: char;
      procedure DropJunction(i: integer);
      procedure SetCaption(i: integer;s: string);
      procedure SetCaptions(cc: TStrings);
      procedure SetHits(s: string);
      procedure SetKeys(s: string);
      procedure SetLongText(s: string);
      procedure SetShortText(s: string);
      procedure SetTable(i: integer;s: string);
      procedure SetTables(tt: TStrings);
      procedure SetType(c: char);
    end;

  TOutputColumn = class(TObject)
    private
      Heading: string;
      TableName: string;
      TableNameLong: string;
      Style: string; // Single | Repeated | Concatenated
      ConcatenationSeparator: string;
      HiKeys: TStringList;
      Keys: TStringList;
      LastKey: string;
      MaxRankNo: integer;
      FieldList: TStringList;
      EncloserList: TStringList;
      FieldListLong: TStringList;
      FieldTypes: string;
      Prefix: string;
      SeparatorList: TStringList;
      Suffix: string;
      procedure SetHiKeys(const sl: TStringList);
      procedure SetLastKey(const sl: TStringList);
    public
      constructor Create;
      destructor Destroy; override;
      function FieldListCount: integer;
      function GetConcatenationSeparator: string;
      function GetEncloserList: TStringList;
      function GetFieldList: TStringList;
      function GetFieldListLong: TStringList;
      function GetFieldName(const i: integer): string;
      function GetFieldNameLong(const i: integer): string;
      function GetFieldType(const i: integer): char;
      function GetFormatString: string;
      function GetHeading: string;
      function GetHiKeys: TStringList;
      function GetKeys: TStringList;
      function GetLastKey: string;
      function GetMaxRankNo: integer;
      function GetPrefix: string;
      function GetSeparatorList: TStringList;
      function GetStyle: string;
      function GetSuffix: string;
      function GetTableName: string;
      function GetTableNameLong: string;
      procedure SetConcatenationSeparator(const s: string);
      procedure SetEncloserList(const sl: TStringList);
      procedure SetFieldList(const sl: TStringList);
      procedure SetFieldListLong(const sl: TStringList);
      procedure SetFieldType(const s: string);
      procedure SetHeading(const s: string);
      procedure SetKeyFields(tn: string);
      procedure SetKeys(const sl: TStringList);
      procedure SetLongNames;
      procedure SetMaxRankNo(const n: integer);
      procedure SetPrefix(const s: string);
      procedure SetSeparatorList(const sl: TStringList);
      procedure SetStyle(const s: string);
      procedure SetSuffix(const s: string);
      procedure SetTableName(const s: string);
      procedure SetTableNameLong(const s: string);
  end;

  TSearchField = class(TObject)
    private
      Name: string;
      Caption: string;
      SearchType: char;
    public
      constructor Create;
      destructor Destroy; override;
      function GetName: string;
      function GetCaption: string;
      function GetSearchType: char;
      procedure SetName(s: string);
      procedure SetCaption(s: string);
      procedure SetSearchType(c: char);
    end;

  TSearchTable = class(TObject)
    private
      Name: string;
      Caption: string;
      Keys: TStringList;
      Ranked: boolean;
      Parent: string;
      Junction: string;
      IsJunction: boolean;
      NumberOfRecords: integer;
    public
      constructor Create;
      destructor Destroy; override;
      procedure FixJunctionAnomaly(s: string);
      function GetName: string;
      function GetCaption: string;
      function GetIsJunction: boolean;
      function GetIsJunctionYesNo: string;
      function GetJunction: string;
      function GetKey(i: integer): string;
      function GetKeyCount: integer;
      function GetKeys: TStrings;
      function GetSQLKeyString: string;
      function GetKeyString: string;
      function GetNumberOfRecords: integer;
      function GetParent: string;
      function GetRanked: boolean;
      function GetRankedYesNo: string;
      procedure SetCaption(s: string);
      procedure SetIsJunction(b: boolean);
      procedure SetJunction(s: string);
      procedure SetKeys(s: string);
      procedure SetName(s: string);
      procedure SetNumberOfRecords(n: integer);
      procedure SetParent(s: string);
      procedure SetRanked(b: boolean);
    end;

implementation

uses dmquery1;

constructor TCondition.Create(const i: integer;c: char;f: TForm);
begin
  inherited Create;
  ConditionType := c;
  Table := TClientDataSet.Create(f);
  Table.Name := Format('t%2.2d',[i]);
  Captions := TStringList.Create;
  Keys := TStringList.Create;
  Tables := TStringList.Create;
end;

destructor TCondition.Destroy;
begin
  Table.Free;
  if Captions <> nil then
    FreeAndNil(Captions);
  if Keys <> nil then
    FreeAndNil(Keys);
  if Tables <> nil then
    FreeAndNil(Tables);
  inherited Destroy;
end;

procedure TCondition.DropJunction(i: integer);
begin
  Captions.Delete(i);
  Tables.Delete(i);
end;

function TCondition.GetCaption(i: integer): string;
begin
  Result := Captions.Strings[i];
end;

function TCondition.GetCaptions: TStrings;
begin
  Result := Captions;
end;

function TCondition.GetHits: string;
begin
  Result := Hits;
end;

function TCondition.GetKey(i: integer): string;
begin
  Result := Keys.Strings[i];
end;

function TCondition.GetKeys: TStrings;
begin
  Result := Keys;
end;

function TCondition.GetKeyString: string;
begin
  Result := Keys.CommaText;
end;

function TCondition.GetLongText: string;
begin
  Result := LongText;
end;

function TCondition.GetNKeyString(i: integer): string;
var
  j: integer;
  ss: string;
begin
  Result := '';
  ss := GetKey(0);
  for j := 1 to i - 1 do
    ss := ss + ',' + GetKey(j);
  Result := ss;
end;

function TCondition.GetShortText: string;
begin
  Result := ShortText;
end;

function TCondition.GetTable(i: integer): string;
begin
  Result := Tables.Strings[i];
end;

function TCondition.GetTables: TStrings;
begin
  Result := Tables;
end;

function TCondition.GetType: char;
begin
  Result := ConditionType;
end;

procedure TCondition.SetCaption(i: integer;s: string);
begin
  Captions.Insert(i,s);
end;

procedure TCondition.SetCaptions(cc: TStrings);
begin
  Captions.AddStrings(cc);
end;

procedure TCondition.SetHits(s: string);
begin
  Hits := s;
end;

procedure TCondition.SetKeys(s: string);
begin
  Keys.CommaText := s;
end;

procedure TCondition.SetLongText(s: string);
begin
  LongText := s;
end;

procedure TCondition.SetShortText(s: string);
begin
  ShortText := s;
end;

procedure TCondition.SetTable(i: integer;s: string);
begin
  Tables.Insert(i,s);
end;

procedure TCondition.SetTables(tt: TStrings);
begin
  Tables.AddStrings(tt);
end;

procedure TCondition.SetType(c: char);
begin
  ConditionType := c;
end;

constructor TOutputColumn.Create;
begin
  inherited Create;
  EncloserList := TStringList.Create;
  FieldList := TStringList.Create;
  FieldListLong := TStringList.Create;
  HiKeys := TStringList.Create;
  Keys := TStringList.Create;
  SeparatorList := TStringList.Create;
end;

destructor TOutputColumn.Destroy;
begin
  EncloserList.Free;
  FieldList.Free;
  FieldListLong.Free;
  HiKeys.Free;
  Keys.Free;
  SeparatorList.Free;
  inherited Destroy;
end;

function TOutputColumn.FieldListCount: integer;
begin
  Result := FieldList.Count;
end;

function TOutputColumn.GetConcatenationSeparator: string;
begin
  Result := StringReplace(ConcatenationSeparator,' ','_',[rfReplaceAll]);
end;

function TOutputColumn.GetFieldList: TStringList;
begin
  Result := FieldList;
end;

function TOutputColumn.GetEncloserList: TStringList;
begin
  Result := EncloserList;
end;

function TOutputColumn.GetFieldListLong: TStringList;
begin
  Result := FieldListLong;
end;

function TOutputColumn.GetFieldName(const i: integer): string;
begin
  Result := FieldList.Strings[i];
end;

function TOutputColumn.GetFieldNameLong(const i: integer): string;
begin
  Result := FieldListLong.Strings[i];
end;

function TOutputColumn.GetFieldType(const i: integer): char;
begin
  Result := FieldTypes[i+1];
end;

function TOutputColumn.GetFormatString: string;
var
  e: string;
  i: integer;
  s: string;
begin
  s := Prefix;
  for i := 0 to FieldList.Count - 2 do
    begin
      e := EncloserList.Strings[i];
      if e <> '' then
        s := s + Copy(e,1,1);
      s := s + '%s';
      if e <> '' then
        s := s + Copy(e,3,1);
      s := s + SeparatorList.Strings[i];
    end;
  e := EncloserList.Strings[EncloserList.Count - 1];
  if e <> '' then
    s := s + Copy(e,1,1);
  s := s + '%s';
  if e <> '' then
    s := s + Copy(e,3,1);
  s := s + Suffix;
  Result := s;
end;

function TOutputColumn.GetHeading: string;
begin
  Result := Heading;
end;

function TOutputColumn.GetHiKeys: TStringList;
begin
  Result := HiKeys;
end;

function TOutputColumn.GetKeys: TStringList;
begin
  Result := Keys;
end;

function TOutputColumn.GetLastKey: string;
begin
  Result := LastKey;
end;

function TOutputColumn.GetPrefix: string;
begin
  Result := Prefix;
end;

function TOutputColumn.GetMaxRankNo: integer;
begin
  Result := MaxRankNo;
end;

function TOutputColumn.GetSeparatorList: TStringList;
begin
  Result := SeparatorList;
end;

function TOutputColumn.GetStyle: string;
begin
  Result := Style;
end;

function TOutputColumn.GetSuffix: string;
begin
  Result := Suffix;
end;

function TOutputColumn.GetTableName: string;
begin
  Result := TableName;
end;

function TOutputColumn.GetTableNameLong: string;
begin
  Result := TableNameLong;
end;

procedure TOutputColumn.SetConcatenationSeparator(const s: string);
begin
  ConcatenationSeparator := StringReplace(s,'_',' ',[rfReplaceAll]);
end;

procedure TOutputColumn.SetFieldList(const sl: TStringList);
begin
  FieldList.Clear;
  FieldList.AddStrings(sl);
end;

procedure TOutputColumn.SetEncloserList(const sl: TStringList);
begin
  EncloserList.Clear;
  EncloserList.AddStrings(sl);
end;

procedure TOutputColumn.SetFieldListLong(const sl: TStringList);
begin
  FieldListLong.Clear;
  FieldListLong.AddStrings(sl);
end;

procedure TOutputColumn.SetFieldType(const s: string);
begin
  FieldTypes := s;
end;

procedure TOutputColumn.SetHeading(const s: string);
begin
  Heading := s;
end;

procedure TOutputColumn.SetHiKeys(const sl: TStringList);
begin
  HiKeys.Clear;
  HiKeys.AddStrings(sl);
  HiKeys.Delete(HiKeys.Count-1);
end;

procedure TOutputColumn.SetKeyFields(tn: string);
var
  kf: TStringList;
begin
  kf := TStringList.Create;
  try
    with dmQuery do
      begin
        cdsTabledoc.Open;
        cdsTabledoc.Locate('TABLENAME',tn,[]);
        kf.CommaText := cdsTabledoc.FieldByName('KEYFIELDS').AsString;
        cdsTabledoc.Close;
        SetKeys(kf);
      end;
  finally
    kf.Free;
  end;
end;

procedure TOutputColumn.SetKeys(const sl: TStringList);
begin
  Keys.AddStrings(sl);
  SetHiKeys(sl);
  SetLastKey(sl);
end;

procedure TOutputColumn.SetLastKey(const sl: TStringList);
begin
  LastKey := sl[sl.Count-1];
end;

procedure TOutputColumn.SetLongNames;
var
  i: integer;
  nl: TStringList;
begin
  with dmQuery do
    begin
      cdsTabledoc.Open;
      cdsTabledoc.Locate('TABLENAME',GetTableName,[]);
      SetTableNameLong(cdsTabledoc.FieldByName('ECAPTION').Value);
      nl := TStringList.Create;
      try
        cdsColumndoc.Open;
        for i := 0 to FieldListCount - 1 do
          begin
            cdsColumndoc.Locate('FIELDNAME',FieldList.Strings[i],[]);
            nl.Add(cdsColumndoc.FieldByName('ECAPTION').Value);
          end;
        SetFieldListLong(nl);
      finally
        nl.Free;
      end;
      cdsColumndoc.Close;
      cdsTabledoc.Close;
    end;
end;

procedure TOutputColumn.SetMaxRankNo(const n: integer);
begin
  MaxRankNo := n;
end;

procedure TOutputColumn.SetPrefix(const s: string);
begin
  Prefix := StringReplace(s,'_',' ',[rfReplaceAll]);
end;

procedure TOutputColumn.SetSeparatorList(const sl: TStringList);
var
  i: integer;
begin
  for i := 0 to sl.Count - 1 do
    sl.Strings[i] := StringReplace(sl.Strings[i],'_',' ',[rfReplaceAll]);
  SeparatorList.Clear;
  SeparatorList.AddStrings(sl);
end;

procedure TOutputColumn.SetStyle(const s: string);
begin
  Style := s;
end;

procedure TOutputColumn.SetSuffix(const s: string);
begin
  Suffix := StringReplace(s,'_',' ',[rfReplaceAll]);
end;

procedure TOutputColumn.SetTableName(const s: string);
begin
  TableName := s;
  SetKeyFields(TableName);
end;

procedure TOutputColumn.SetTableNameLong(const s: string);
begin
  TableNameLong := s;
end;

constructor TSearchField.Create;
begin
  inherited Create;
end;

destructor TSearchField.Destroy;
begin
  inherited Destroy;
end;

function TSearchField.GetName: string;
begin
  Result := Name;
end;

function TSearchField.GetCaption: string;
begin
  Result := Caption;
end;

function TSearchField.GetSearchType: char;
begin
  Result := SearchType;
end;

procedure TSearchField.SetName(s: string);
begin
  Name := s;
end;

procedure TSearchField.SetCaption(s: string);
begin
  Caption := s;
end;

procedure TSearchField.SetSearchType(c: char);
begin
  SearchType := c;
end;

constructor TSearchTable.Create;
begin
  inherited Create;
  Keys := TStringList.Create;
end;

destructor TSearchTable.Destroy;
begin
  Keys.Free;
  inherited Destroy;
end;

procedure TSearchTable.FixJunctionAnomaly(s: string);
var
  i: integer;
begin
  i := Keys.IndexOf(s);
  if i > 0 then
    Keys.Move(i,0);
end;

function TSearchTable.GetCaption: string;
begin
  if Caption <> ''
    then Result := Caption
    else Result := Name;
end;

function TSearchTable.GetIsJunction: boolean;
begin
  Result := IsJunction;
end;

function TSearchTable.GetIsJunctionYesNo: string;
begin
  if IsJunction
    then Result := 'Yes'
    else Result := 'No';
end;

function TSearchTable.GetJunction: string;
begin
  Result := Junction;
end;

function TSearchTable.GetKey(i: integer): string;
begin
  Result := Keys.Strings[i];
end;

function TSearchTable.GetKeyCount: integer;
begin
  Result := Keys.Count;
end;

function TSearchTable.GetKeys: TStrings;
begin
  Result := Keys;
end;

function TSearchTable.GetSQLKeyString: string;
var
  i: integer;
  s: string;
begin
  s := '';
  if Junction = ''
    then s := Name + '.' + Keys[0]
    else s := Junction + '.' + Keys[0];
  if Keys.Count > 1 then
    for i := 1 to Keys.Count - 1 do
      s := s + ',' + Name + '.' + Keys[i];
  Result := s;
end;

function TSearchTable.GetKeyString: string;
begin
  Result := Keys.CommaText;
end;

function TSearchTable.GetName: string;
begin
  Result := Name;
end;

function TSearchTable.GetNumberOfRecords: integer;
begin
  Result := NumberOfRecords;
end;

function TSearchTable.GetParent: string;
begin
  Result := Parent;
end;

function TSearchTable.GetRanked: boolean;
begin
  Result := Ranked;
end;

function TSearchTable.GetRankedYesNo: string;
begin
  if Ranked
    then Result := 'Yes'
    else Result := 'No';
end;

procedure TSearchTable.SetCaption(s: string);
begin
  Caption := s;
end;

procedure TSearchTable.SetIsJunction(b: boolean);
begin
  IsJunction := b;
end;

procedure TSearchTable.SetJunction(s: string);
begin
  Junction := s;
end;

procedure TSearchTable.SetKeys(s: string);
begin
  Keys.CommaText := StringReplace(s,';',',',[rfReplaceAll]);
end;

procedure TSearchTable.SetName(s: string);
begin
  Name := s;
end;

procedure TSearchTable.SetNumberOfRecords(n: integer);
begin
  NumberOfRecords := n;
end;

procedure TSearchTable.SetParent(s: string);
begin
  Parent := s;
end;

procedure TSearchTable.SetRanked(b: boolean);
begin
  Ranked := b;
end;

end.


