unit mastform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Mask, DBCtrls, Grids, DBGrids, ComCtrls, TConnect, ShellAPI;

type
  TfmMaster = class(TForm)
    paTop: TPanel;
    stbMaster: TStatusBar;
    paControl: TPanel;
    buSelectDB: TButton;
    buSelectRecords: TButton;
    buPrepareTemplate: TButton;
    buGenerateOutput: TButton;
    buResetAll: TButton;
    paNavigator: TPanel;
    dnQSummary: TDBNavigator;
    dbdgQSummary: TDBGrid;
    buExit: TButton;
    laVersion: TLabel;
    buGenerateHTML: TButton;
    buDisplayPage: TButton;
    procedure buSelectDBClick(Sender: TObject);
    procedure buExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure buResetAllClick(Sender: TObject);
    procedure buSelectRecordsClick(Sender: TObject);
    procedure buPrepareTemplateClick(Sender: TObject);
    procedure buGenerateOutputClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buGenerateHTMLClick(Sender: TObject);
    procedure buDisplayPageClick(Sender: TObject);
    procedure buSelectDBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    procedure ActivateControls(YesOrNo : boolean);
    function GetDBVersion: string;
  public
    AliasDriver: string;
    AltKeyDown: boolean;
    AnchorDirectory: string;
    DBVersion: string;
    GQVersion: string;
    UserName: string;
    UserPassword: string;
  end;

resourcestring
  ConstCaption1 = 'GQuery -- 3.7 ';
  ConstCaption2 = ' (3.19..3.22 -- 2011-06-23)';

const
  MaxConditions = 20;
  MaxSearchFields = 15;
  MaxOutputColumns = 99;
  MaxSearchTables = 150;
  MaxTemplates = 20;
  MaxAgeTables = 10;

var
  fmMaster: TfmMaster;
  AltIPAddress: string;
  UserInitials: string;
  dbname: string;
  compby: string;
  reldate: string;
  dburl: string;
  entity: string;
  e_tablegroup: string;
  e_table: string;
  e_key: string;
  WildCard: char;
  TempTables: boolean;

implementation

uses dmglob1, seldb, startinf, selentity, selrec, genout,
  prepout, warning, dbtree, dmquery1, genhtml319, genhtml320, dbversion,
  resources;

{$R *.DFM}

procedure TfmMaster.ActivateControls(YesOrNo : boolean);
begin
  buSelectRecords.Enabled := YesOrNo;
  buPrepareTemplate.Enabled := YesOrNo;
  buGenerateHTML.Enabled := DBVersion <> '';
  buGenerateOutput.Enabled := YesOrNo;
end;

function TfmMaster.GetDBVersion: string;
begin
  Result := '';
  if fmDBVersion = nil then
    fmDBVersion := TfmDBVersion.Create(nil);
  try
    fmDBVersion.FormPosition(Self.Top,Self.Left,Self.Height,Self.Width);
    fmDBVersion.ShowModal;
    Result := fmDBVersion.DatabaseRelease;
  finally
    FreeAndNil(fmDBVersion);
  end;
  if Result = '' then
    begin
      MessageDlg(mastform_UpgradeLevelNotSpecified,mtInformation,[mbOK],0);
      buGenerateHTML.Enabled := False;
    end;
end;

procedure TfmMaster.buSelectDBClick(Sender: TObject);
var
  sd: TfmSelectDB;
  se: TfmSelectEntity;
  s: string;
begin
  if not dmGlobalC.IdentifyUser then
    Exit;
  // select DB
  if sd = nil then
    sd := TfmSelectDB.Create(nil);
  try
    sd.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
    if sd.ShowModal = mrAbort then
      begin
        dmGlobalC.FindBySQL.Close;
        Exit;
      end;
  finally
    FreeAndNil(sd);
  end;

  // select entity
  entity := '';
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select d.dbname,d.dbrel,d.compby,d.reldate,e.tablegroup,e.tablename,' +
                     'e.edata,e.ecaption,g.recordcount ' +
                     'from dbversion d,entities e inner join gq_recordcount g ' +
                     'on e.tablename=g.tablename ' +
                     'where (e.rank = 1) ' +
                     'and (e.tablegroup in (''Deposit Groups'',''Deposits'')) ' +
                     'and (g.recordcount > 0)';
      Open;
      dbname := FieldByName('DBNAME').AsString;
      DBVersion := FieldByName('DBREL').AsString;
      if AnsiPos('3.19',DBVersion) > 0
        then DBVersion := '3.19'
        else
      if AnsiPos('3.20',DBVersion) > 0
        then DBVersion := '3.20'
        else
      if AnsiPos('3.21',DBVersion) > 0
        then DBVersion := '3.21'
        else
      if AnsiPos('3.22',DBVersion) > 0
        then DBVersion := '3.22'
        else DBVersion := GetDBVersion;
      compby := FieldByName('COMPBY').AsString;
      s := FieldByName('RELDATE').AsString;
      reldate := Format('%s-%s-%s',[Copy(s,1,4),Copy(s,5,2),Copy(s,7,2)]);
      // update status bar
      stbMaster.Panels[0].Text := dbname + ' (' + compby + ')';
      if se = nil then
        se := TfmSelectEntity.Create(nil);
      try
        se.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
        se.ShowModal;
      finally
        FreeAndNil(se);
      end;
      Close;
      if dburl <> ''
        then dburl := dburl + '/' + entity;
    end;

  if e_tablegroup = 'Deposits'
    then stbMaster.Panels[1].Text := selentity_Deposits
    else stbMaster.Panels[1].Text := selentity_DepositGroups;
  fmMaster.Update;
  fmStartInfo.Label1.Caption := mastform_BuildingSummaryTables;
  fmStartInfo.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
  fmStartInfo.Show;
  fmStartInfo.Update;
  dmQuery.CPD_SummaryTable(entity);
  stbMaster.Panels[2].Text := mastform_Selected + IntToStr(dmQuery.n_selected) + ' / ' + IntToStr(n_total);
  stbMaster.Panels[3].Text := IntToStr(1) + ' / ' + IntToStr(dmQuery.n_selected);
  fmStartInfo.Hide;
  ActivateControls(True);
  buSelectDB.Enabled := False;
  buResetAll.Enabled := True;
  buDisplayPage.Enabled := dburl <> '';
  buDisplayPage.Visible := buDisplayPage.Enabled;
  with fmMaster.dbdgQSummary.Columns do
    begin
      Items[0].Width := 50;
      Items[1].Width := 100;
      Items[2].Width := 150;
      Items[3].Width := 250;
      Items[4].Width := 250;
    end;
end;

procedure TfmMaster.buExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMaster.FormCreate(Sender: TObject);
begin
  Self.Caption := mastform_fmMaster;
  buExit.Caption := mastform_buExit;
  buSelectDB.Caption := mastform_buSelectDB;
  buSelectDB.Hint := mastform_buSelectDBHint;
  buSelectRecords.Caption := mastform_buSelectRecords;
  buSelectRecords.Hint := mastform_buSelectRecordsHint;
  buPrepareTemplate.Caption := mastform_buPrepareTemplate;
  buPrepareTemplate.Hint := mastform_buPrepareTemplateHint;
  buGenerateOutput.Caption := mastform_buGenerateOutput;
  buGenerateOutput.Hint := mastform_buGenerateOutputHint;
  buGenerateHTML.Caption := mastform_buGenerateHTML;
  buGenerateHTML.Hint := mastform_buGenerateHTMLHint;
  buResetAll.Caption := mastform_buResetAll;
  buResetAll.Hint := mastform_buResetAllHint;
  buDisplayPage.Caption := mastform_buDisplayPage;
  fmWarning := TfmWarning.Create(nil);
  try
    if fmWarning.ShowModal <> mrOK
      then Application.Terminate
      else
        if UserInitials = 'ZZZ'
          then
            begin
              buSelectDB.Enabled := False;
              ActivateControls(False);
              ShowMessage(mastform_NoInitials);
              Application.Terminate;
            end
          else
            begin
              fmStartInfo.Label1.Caption := mastform_BuildingDataModule;
              fmStartInfo.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
              fmStartInfo.Show;
              fmStartInfo.Update;
              dmGlobalC := TdmGlobalC.Create(nil);
              dmQuery := TdmQuery.Create(nil);
              TempTables := False;
              UserName := '';
              UserPassword := '';
            end;
  finally
    FreeAndNil(fmWarning);
  end;
// Center form on screen, fill screen if 800x600
  fmMaster.Top := Trunc ((Screen.Height - 28 - 572) / 2);
  fmMaster.Left := Trunc ((Screen.Width - 800) / 2);
  fmMaster.Height := 572;
  fmMaster.Width := 800;
  AnchorDirectory := ExtractFilePath(ParamStr(0));
end;

procedure TfmMaster.FormShow(Sender: TObject);
begin
  if GQVersion = 'BDE'
    then WildCard := '*'
    else WildCard := '%';
  if GQVersion = 'C_S' then
    dmGlobalC.SocketConnection1.Address := AltIPAddress;
  laVersion.Caption := ConstCaption1 + GQVersion + ConstCaption2;
  AltKeyDown := False;
  buSelectDB.SetFocus;
  fmStartInfo.Hide;
end;

procedure TfmMaster.buResetAllClick(Sender: TObject);
begin
  dmGlobalC.ResetDatabase;
  stbMaster.Panels[0].Text := '';
  stbMaster.Panels[1].Text := '';
  stbMaster.Panels[2].Text := '';
  stbMaster.Panels[3].Text := '';
  DBVersion := '';
  ActivateControls(False);
  with dmQuery do
    begin
      DeleteAgeTables;
      cdsQSummary.DisableControls;
      cdsQSummary.Free;
      cdsSummary.Free;
    end;
  if fmSelectRecords <> nil then
    FreeAndNil(fmSelectRecords);
  if fmPrepareOutput <> nil then
    FreeAndNil(fmPrepareOutput);
  if fmGenerateOutput <> nil then
    FreeAndNil(fmGenerateOutput);
  if fmGenerateHTML319 <> nil then
    FreeAndNil(fmGenerateHTML319);
  if fmGenerateHTML320 <> nil then
    FreeAndNil(fmGenerateHTML320);
  if fmDBTree <> nil then
    FreeAndNil(fmDBTree);
  if dmGlobalC <> nil then
    FreeAndNil(dmGlobalC);
  dmGlobalC := TdmGlobalC.Create(nil);
  dmGlobalC.SocketConnection1.Address := AltIPAddress;
  buSelectDB.Enabled := True;
  buResetAll.Enabled := False;
  buDisplayPage.Enabled := False;
  buDisplayPage.Visible := False;
  buSelectDB.SetFocus;
end;

procedure TfmMaster.buSelectRecordsClick(Sender: TObject);
begin
  if fmSelectRecords = nil then
    begin
      fmSelectRecords := TfmSelectRecords.Create(Self);
      fmSelectRecords.CurrentQ := 1;
    end;
  fmSelectRecords.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
  fmSelectRecords.ShowModal;
  with dmQuery.cdsQSummary do
    begin
      DisableControls;
      if fmSelectRecords.SelectAll
        then dmQuery.PopulateQSummary(dmQuery.cdsSummary)
        else dmQuery.PopulateQSummary(Conditions[fmSelectRecords.CurrentQ].Table);
      dmQuery.n_selected := RecordCount;
      stbMaster.Panels[2].Text := mastform_Selected + IntToStr(dmQuery.n_selected) + ' / ' + IntToStr(n_total);
      stbMaster.Panels[3].Text := IntToStr(1) + ' / ' + IntToStr(dmQuery.n_selected);
      EnableControls;
    end;
end;

procedure TfmMaster.buPrepareTemplateClick(Sender: TObject);
begin
  if fmPrepareOutput = nil then
    fmPrepareOutput := TfmPrepareOutput.Create(Self);
  fmPrepareOutput.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
  fmPrepareOutput.ShowModal;
end;

procedure TfmMaster.buGenerateOutputClick(Sender: TObject);
begin
  if fmPrepareOutput = nil then
    fmPrepareOutput := TfmPrepareOutput.Create(Self);
  if fmGenerateOutput = nil then
    fmGenerateOutput := TfmGenerateOutput.Create(Self);
  fmGenerateOutput.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
  fmGenerateOutput.ShowModal;
end;

procedure TfmMaster.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if fmDBTree <> nil then
    FreeAndNil(fmDBTree);
  if fmGenerateOutput <> nil then
    FreeAndNil(fmGenerateOutput);
  if fmPrepareOutput <> nil then
    FreeAndNil(fmPrepareOutput);
  if fmGenerateHTML319 <> nil then
    FreeAndNil(fmGenerateHTML319);
  if fmGenerateHTML320 <> nil then
    FreeAndNil(fmGenerateHTML320);
  if fmSelectRecords <> nil then
    FreeAndNil(fmSelectRecords);
  if dmGlobalC <> nil then
    FreeAndNil(dmGlobalC);
  if dmQuery <> nil then
    FreeAndNil(dmQuery);
end;

procedure TfmMaster.buGenerateHTMLClick(Sender: TObject);
begin
  if DBVersion = '' then
    Exit;
  if DBVersion = '3.19'
    then
      begin
        if fmGenerateHTML319 = nil then
          fmGenerateHTML319 := TfmGenerateHTML319.Create(nil);
        try
          fmGenerateHTML319.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
          fmGenerateHTML319.ShowModal;
        finally
          FreeAndNil(fmGenerateHTML319);
        end;
      end
    else
      begin
        if fmGenerateHTML320 = nil then
          fmGenerateHTML320 := TfmGenerateHTML320.Create(nil);
        try
          fmGenerateHTML320.FormPosition(fmMaster.Top,fmMaster.Left,fmMaster.Height,fmMaster.Width);
          fmGenerateHTML320.ShowModal;
        finally
          FreeAndNil(fmGenerateHTML320);
        end;
      end;
end;

procedure TfmMaster.buDisplayPageClick(Sender: TObject);
var
  c: string;
begin
  c := '';
  if Copy(dburl,Length(dburl),1) = '/'
    then c := ''
    else c := '/';
  ShellExecute(Handle,'open',PChar(dburl + c + AnsiLowerCase(entity) +
               IntToStr(dmQuery.cdsQSummary.FieldByName(e_key).AsInteger) + '.html'),
               nil,PChar(dburl),0);
end;

procedure TfmMaster.buSelectDBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_Menu then
    AltKeyDown := True;
end;

initialization
  UserInitials := '';
  repeat
    if not InputQuery(mastform_UserIdentificationInfo,mastform_UserIdentificationPrompt,UserInitials)
      then UserInitials := 'ZZZ';
    if UserInitials = ''
      then ShowMessage(mastform_UserIdentificationInvalid);
  until UserInitials <> '';
  AltIPAddress := '132.156.90.231';
  if UserInitials = 'iptest' then
    begin
      AltIPAddress := InputBox(mastform_ServerIdentificationInfo,mastform_ServerIdentificationPrompt,'');
      UserInitials := '';
      repeat
        if not InputQuery(mastform_UserIdentificationInfo,mastform_UserIdentificationPrompt,UserInitials)
          then UserInitials := 'ZZZ';
        if UserInitials = ''
          then ShowMessage(mastform_UserIdentificationInvalid);
      until UserInitials <> '';
    end;
  fmStartInfo := TfmStartInfo.Create(nil);

finalization
  with fmStartInfo do
    begin
      Release;
    end;

end.

