unit prepout;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, Buttons, Grids, mastform, gqclasses;

type
  TfmPrepareOutput = class(TForm)
    paTop: TPanel;
    stbMaster: TStatusBar;
    paControls: TPanel;
    buBuildColumn: TButton;
    buClearAllColumns: TButton;
    buClearColumn: TButton;
    sgOutputColumns: TStringGrid;
    sbUpColumn: TSpeedButton;
    sbDownColumn: TSpeedButton;
    buEditColumn: TButton;
    paExit: TPanel;
    buMainScreen: TButton;
    paFiller: TPanel;
    paTablesColumns: TPanel;
    laTablesColumns: TLabel;
    cbTablesColumns: TComboBox;
    paTemplate: TPanel;
    buLoadTemplate: TButton;
    buSaveTemplate: TButton;
    sdSaveTemplate: TSaveDialog;
    odLoadTemplate: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure buBuildColumnClick(Sender: TObject);
    procedure buClearColumnClick(Sender: TObject);
    procedure buClearAllColumnsClick(Sender: TObject);
//    procedure buEditColumnClick(Sender: TObject);
    procedure sgOutputColumnsClick(Sender: TObject);
    procedure cbTablesColumnsChange(Sender: TObject);
    procedure sbUpColumnClick(Sender: TObject);
    procedure sbDownColumnClick(Sender: TObject);
    procedure buSaveTemplateClick(Sender: TObject);
    procedure buLoadTemplateMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure buLoadTemplateClick(Sender: TObject);
  private
    procedure ClearAllOutputColumns;
    procedure ClearOutputColumn(i: integer);
    procedure CopyColumn(s, d: TOutputColumn);
//    procedure ProcessOutputColumn(BuildMode: string);
    procedure ProcessOutputColumn;
    procedure RefreshGridRow(const n: integer);
    procedure RefreshGrid;
    procedure ClearGridRow(const n: integer);
    procedure LoadTemplate(const fn: string);
  public
    CaptionStyle: integer;
    CurrentC: integer;
    OutputColumnCount: integer;
    OutputColumns: array[1..MaxOutputColumns] of TOutputColumn;
    TransferColumn: TOutputColumn;
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmPrepareOutput: TfmPrepareOutput;

implementation

uses bldcolumn, seltmpl, dbtree, resources;

var
//  OutputColumns: array[1..MaxOutputColumns] of TOutputColumn;
  TemplateSaved: boolean;

{$R *.DFM}

procedure TfmPrepareOutput.ClearAllOutputColumns;
var
  i : integer;
begin
  i := 1;
  while (OutputColumnCount > 0) and (i <= MaxOutputColumns) do
    begin
      ClearOutputColumn(i);
      inc(i);
    end;
  CurrentC := 1;
end;

procedure TfmPrepareOutput.ClearOutputColumn(i: integer);
begin
  if OutputColumns[i] <> nil then
    begin
      FreeAndNil(OutputColumns[i]);
      ClearGridRow(i);
      dec(OutputColumnCount);
    end;
end;

procedure TfmPrepareOutput.ClearGridRow(const n: integer);
var
  i: integer;
begin
  for i := 1 to 5 do
    sgOutputColumns.Cells[i,n] := '';
  TemplateSaved := False;
end;

procedure TfmPrepareOutput.CopyColumn(s,d: TOutputColumn);
begin
  d.SetConcatenationSeparator(s.GetConcatenationSeparator);
  d.SetEncloserList(s.GetEncloserList);
  d.SetFieldList(s.GetFieldList);
  d.SetFieldListLong(s.GetFieldListLong);
  d.SetHeading(s.GetHeading);
  d.SetKeys(s.GetKeys);
  d.SetPrefix(s.GetPrefix);
  d.SetSeparatorList(s.GetSeparatorList);
  d.SetStyle(s.GetStyle);
  d.SetSuffix(s.GetSuffix);
  d.SetTableName(s.GetTableName);
  d.SetTableNameLong(s.GetTableNameLong);
end;

procedure TfmPrepareOutput.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

procedure TfmPrepareOutput.LoadTemplate(const fn: string);
var
  i,j: integer;
  el: TStringList;  // encloser list
  fl: TStringList;  // field list
  tl: TStringList;  // template list
begin
  tl := TStringList.Create;
  try
    tl.LoadFromFile(fn);
    stbMaster.Panels.Items[0].Text := tl.Strings[1];
    stbMaster.Panels.Items[1].Text := tl.Strings[2];
    for i := 0 to 2 do   // template header
      tl.Delete(0);
    ClearAllOutputColumns;
    CurrentC := 1;
    while tl.Count > 0 do
      begin
        OutputColumns[CurrentC] := TOutputColumn.Create;
        if tl.Strings[0] = '[none]'
          then OutputColumns[CurrentC].SetHeading('')
          else OutputColumns[CurrentC].SetHeading(tl.Strings[0]);
        tl.Delete(0);
        OutputColumns[CurrentC].SetTableName(tl.Strings[0]);
        tl.Delete(0);
        OutputColumns[CurrentC].SetStyle(tl.Strings[0]);
        tl.Delete(0);
        if OutputColumns[CurrentC].GetStyle = bldcolumn_Concatenated then
          begin
            OutputColumns[CurrentC].SetConcatenationSeparator(tl.Strings[0]);
            tl.Delete(0);
          end;
        i := StrToInt(tl.Strings[0]);
        tl.Delete(0);
        el := TStringList.Create;
        fl := TStringList.Create;
        try
          for j := 1 to i do
            begin
              fl.Add(tl.Strings[0]);
              tl.Delete(0);
              el.Add(tl.Strings[0]);
              tl.Delete(0);
            end;
          OutputColumns[CurrentC].SetFieldList(fl);
          OutputColumns[CurrentC].SetEncloserList(el);
        finally
          el.Free;
          fl.Free;
        end;
        if tl.Strings[0] = '[none]'
          then OutputColumns[CurrentC].SetPrefix('')
          else OutputColumns[CurrentC].SetPrefix(tl.Strings[0]);
        tl.Delete(0);
        fl := TStringList.Create;
        try
          for j := 1 to i - 1 do
            begin
              fl.Add(tl.Strings[0]);
              tl.Delete(0);
            end;
          OutputColumns[CurrentC].SetSeparatorList(fl);
        finally
          fl.Free;
        end;
        if tl.Strings[0] = '[none]'
          then OutputColumns[CurrentC].SetSuffix('')
          else OutputColumns[CurrentC].SetSuffix(tl.Strings[0]);
        tl.Delete(0);
        OutputColumns[CurrentC].SetLongNames;
        inc(OutputColumnCount);
        inc(CurrentC);
      end;
  finally
    tl.Free;
  end;
end;

//procedure TfmPrepareOutput.ProcessOutputColumn(BuildMode: string);
procedure TfmPrepareOutput.ProcessOutputColumn;
begin
  if fmBuildOutputColumn = nil then
    fmBuildOutputColumn := TfmBuildOutputColumn.Create(Self);
  fmBuildOutputColumn.CaptionStyle := CaptionStyle;
//  fmBuildOutputColumn.BuildMode := BuildMode;
  TransferColumn := TOutputColumn.Create;
  try
//    if BuildMode = 'Edit' then
//      CopyColumn(OutputColumns[CurrentC],TransferColumn);
    fmBuildOutputColumn.FormPosition(fmPrepareOutput.Top,fmPrepareOutput.Left,fmPrepareOutput.Height,fmPrepareOutput.Width);
    if fmBuildOutputColumn.ShowModal = mrCancel then
      Exit;
//    if BuildMode = 'Build' then
//      begin
        OutputColumns[CurrentC] := TOutputColumn.Create;
        inc(OutputColumnCount);
//      end;
    CopyColumn(TransferColumn,OutputColumns[CurrentC]);
  finally
    TransferColumn.Free;
  end;
  RefreshGridRow(CurrentC);
end;

procedure TfmPrepareOutput.RefreshGrid;
var
  i : integer;
begin
  i := 1;
  while (OutputColumnCount > 0) and (i <= MaxOutputColumns) do
    begin
      if OutputColumns[i] <> nil then
        RefreshGridRow(i);
      inc(i);
    end;
  sgOutputColumns.Row := 1;
  CurrentC := 1;
end;

procedure TfmPrepareOutput.RefreshGridRow(const n: integer);
var
  i: integer;
  s: string;
begin
  sgOutputColumns.Cells[1,n] := OutputColumns[n].GetHeading;
  sgOutputColumns.Cells[3,n] := OutputColumns[n].GetStyle;
  sgOutputColumns.Cells[4,n] := OutputColumns[n].GetFormatString;
  if CaptionStyle = 0
    then
      begin
        sgOutputColumns.Cells[2,n] := OutputColumns[n].GetTableName;
        s := OutputColumns[n].GetFieldName(0);
        if OutputColumns[n].FieldListCount > 1 then
          for i := 1 to OutputColumns[n].FieldListCount - 1 do
            s := s + ', ' + OutputColumns[n].GetFieldName(i);
      end
    else
      begin
        sgOutputColumns.Cells[2,n] := OutputColumns[n].GetTableNameLong;
        s := OutputColumns[n].GetFieldNameLong(0);
        if OutputColumns[n].FieldListCount > 1 then
          for i := 1 to OutputColumns[n].FieldListCount - 1 do
            s := s + ', ' + OutputColumns[n].GetFieldNameLong(i);
      end;
  sgOutputColumns.Cells[5,n] := s;
  TemplateSaved := False;
end;

procedure TfmPrepareOutput.FormCreate(Sender: TObject);
var
  i : integer;
begin
  Self.Caption := prepout_fmPrepareOutput;
  laTablesColumns.Caption := prepout_laTablesColumns;
  cbTablesColumns.Items.CommaText := prepout_cbTablesColumns;
  buLoadTemplate.Caption := prepout_buLoadTemplate;
  buSaveTemplate.Caption := prepout_buSaveTemplate;
  buMainScreen.Caption := prepout_buMainScreen;
  buBuildColumn.Caption := prepout_buBuildColumn;
  sbUpColumn.Hint := prepout_sbUpColumnHint;
  sbDownColumn.Hint := prepout_sbDownColumnHint;
  buClearColumn.Caption := prepout_buClearColumn;
  buClearAllColumns.Caption := prepout_buClearAllColumns;
  fmSelectTemplate := TfmSelectTemplate.Create(Self);
  cbTablesColumns.ItemIndex := 1; // defaults to full English caption
  CaptionStyle := cbTablesColumns.ItemIndex;
  TemplateSaved := False;
  OutputColumnCount := 0;
  with sgOutputColumns do
    begin
      RowCount := 1 + MaxOutputColumns;
      FixedRows := 1;
      for i := 1 to MaxOutputColumns do
        Cells[0,i] := Format('C%2.2d',[i]);
      Cells[1,0] := prepout_ColumnHeading;
      Cells[2,0] := prepout_SourceTable;
      Cells[3,0] := prepout_Style;
      Cells[4,0] := prepout_FormatString;
      Cells[5,0] := prepout_Fields;
      ColWidths[0] := 30;
      ColWidths[1] := 150;
      ColWidths[2] := 150;
      ColWidths[3] := 100;
      ColWidths[4] := 100;
      ColWidths[5] := 300;
    end;
  CurrentC := 1;
  sgOutputColumns.Row := 1;
end;

procedure TfmPrepareOutput.FormDestroy(Sender: TObject);
begin
  if not TemplateSaved and (OutputColumnCount > 0) then
    if MessageDlg(prepout_CurrentTemplateNotSaved,mtWarning,[mbYes,mbNo],0) = mrYes then
      buSaveTemplateClick(Self);
  ClearAllOutputColumns;
  if fmBuildOutputColumn <> nil then
    FreeAndNil(fmBuildOutputColumn);
  FreeAndNil(fmSelectTemplate);
end;

procedure TfmPrepareOutput.buBuildColumnClick(Sender: TObject);
begin
  if OutputColumns[CurrentC] <> nil then
    if MessageDlg(prepout_OverwriteCurrentOutputColumn,mtConfirmation,[mbOK,mbCancel],0) = mrCancel
      then Exit;
//  ProcessOutputColumn('Build');
  ProcessOutputColumn;
end;

procedure TfmPrepareOutput.buClearColumnClick(Sender: TObject);
begin
  ClearOutputColumn(CurrentC);
end;

procedure TfmPrepareOutput.buClearAllColumnsClick(Sender: TObject);
begin
  ClearAllOutputColumns;
  stbMaster.Panels.Items[0].Text := '';
  stbMaster.Panels.Items[1].Text := '';
end;

{
procedure TfmPrepareOutput.buEditColumnClick(Sender: TObject);
begin
  if OutputColumns[CurrentC] <> nil
    then ProcessOutputColumn('Edit')
    else MessageDlg('There is nothing to edit!',mtInformation,[mbOK],0);
end;
}

procedure TfmPrepareOutput.sgOutputColumnsClick(Sender: TObject);
begin
  CurrentC := sgOutputColumns.Row;
end;

procedure TfmPrepareOutput.cbTablesColumnsChange(Sender: TObject);
begin
  CaptionStyle := cbTablesColumns.ItemIndex;
  RefreshGrid;
end;

procedure TfmPrepareOutput.sbUpColumnClick(Sender: TObject);
var
  tmpColumn: TOutputColumn;
begin
  if (CurrentC = 1) or (OutputColumns[CurrentC] = nil) then
    begin
      ShowMessage(prepout_NothingToMoveFirst);
      Exit;
    end;
  if OutputColumns[CurrentC - 1] = nil
    then
      begin
        OutputColumns[CurrentC - 1] := OutputColumns[CurrentC];
        OutputColumns[CurrentC] := nil;
        RefreshGridRow(CurrentC - 1);
        ClearGridRow(CurrentC);
      end
    else
      begin
        tmpColumn := OutputColumns[CurrentC - 1];
        OutputColumns[CurrentC - 1] := OutputColumns[CurrentC];
        OutputColumns[CurrentC] := tmpColumn;
        RefreshGridRow(CurrentC - 1);
        RefreshGridRow(CurrentC);
      end;
  dec(CurrentC);
  sgOutputColumns.Row := CurrentC;
end;

procedure TfmPrepareOutput.sbDownColumnClick(Sender: TObject);
var
  tmpColumn: TOutputColumn;
begin
  if (CurrentC = MaxOutputColumns) or (OutputColumns[CurrentC] = nil) then
    begin
      ShowMessage(prepout_NothingToMoveLast);
      Exit;
    end;
  if OutputColumns[CurrentC + 1] = nil
    then
      begin
        OutputColumns[CurrentC + 1] := OutputColumns[CurrentC];
        OutputColumns[CurrentC] := nil;
        RefreshGridRow(CurrentC + 1);
        ClearGridRow(CurrentC);
      end
    else
      begin
        tmpColumn := OutputColumns[CurrentC + 1];
        OutputColumns[CurrentC + 1] := OutputColumns[CurrentC];
        OutputColumns[CurrentC] := tmpColumn;
        RefreshGridRow(CurrentC + 1);
        RefreshGridRow(CurrentC);
      end;
  inc(CurrentC);
  sgOutputColumns.Row := CurrentC;
end;

procedure TfmPrepareOutput.buSaveTemplateClick(Sender: TObject);
var
  i,j: integer;
  t_description: string;
  tl: TStringList; // template list
begin
  if not sdSaveTemplate.Execute then
    Exit;
  t_description := stbMaster.Panels.Items[1].Text;
  if not InputQuery(prepout_TemplateDescriptionInfo,prepout_TemplateDescriptionPrompt,t_description) then
    Exit;
  tl := TStringList.Create;
  try
    tl.Add(entity);
    tl.Add(AnsiUpperCase(UserInitials) + '-' + FormatDateTime('yyyymmddhhnn',Now));
    tl.Add(t_description);
    i := 1;
    while (i <= OutputColumnCount) and (i <= MaxOutputColumns) do
      begin
        if OutputColumns[i] <> nil then
          begin
            if OutputColumns[i].GetHeading = ''
              then tl.Add('[none]')
              else tl.Add(OutputColumns[i].GetHeading);
            tl.Add(OutputColumns[i].GetTableName);
            tl.Add(OutputColumns[i].GetStyle);
            if OutputColumns[i].GetStyle = bldcolumn_Concatenated then
              tl.Add(OutputColumns[i].GetConcatenationSeparator);
            tl.Add(IntToStr(OutputColumns[i].FieldListCount));
            for j := 0 to OutputColumns[i].FieldListCount - 1 do
              begin
                tl.Add(OutputColumns[i].GetFieldList.Strings[j]);
                tl.Add(OutputColumns[i].GetEncloserList.Strings[j]);
              end;
            if OutputColumns[i].GetPrefix = ''
              then tl.Add('[none]')
              else tl.Add(StringReplace(OutputColumns[i].GetPrefix,' ','_',[rfReplaceAll]));
            for j := 0 to OutputColumns[i].FieldListCount - 2 do
              tl.Add(StringReplace(OutputColumns[i].GetSeparatorList.Strings[j],' ','_',[rfReplaceAll]));
            if OutputColumns[i].GetSuffix = ''
              then tl.Add('[none]')
              else tl.Add(StringReplace(OutputColumns[i].GetSuffix,' ','_',[rfReplaceAll]));
          end;
        inc(i);
      end;
    tl.SaveToFile(sdSaveTemplate.FileName);
    TemplateSaved := True;
    stbMaster.Panels.Items[0].Text := tl.Strings[1];
    stbMaster.Panels.Items[1].Text := tl.Strings[2];
  finally
    tl.Free;
  end;
end;

procedure TfmPrepareOutput.buLoadTemplateMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not TemplateSaved and (OutputColumnCount > 0) then
    if MessageDlg(prepout_CurrentTemplateNotSaved,mtWarning,[mbYes,mbNo],0) = mrYes then
      buSaveTemplateClick(Self);
  if Button = mbLeft then
    if Shift = [ssLeft,ssAlt]
      then
        begin
          if not odLoadTemplate.Execute then
            Exit;
          LoadTemplate(odLoadTemplate.FileName);
          RefreshGrid;
          TemplateSaved := True;
        end
      else
        if Shift = [ssLeft] then
          begin
            fmSelectTemplate.FormPosition(fmPrepareOutput.Top,fmPrepareOutput.Left,
                                          fmPrepareOutput.Height,fmPrepareOutput.Width);
            if fmSelectTemplate.ShowModal = mrCancel then
              Exit;
            LoadTemplate(fmSelectTemplate.TemplateFileName);
            RefreshGrid;
            TemplateSaved := True;
          end;
end;

procedure TfmPrepareOutput.buLoadTemplateClick(Sender: TObject);
begin
  if not TemplateSaved and (OutputColumnCount > 0) then
    if MessageDlg(prepout_CurrentTemplateNotSaved,mtWarning,[mbYes,mbNo],0) = mrYes then
      buSaveTemplateClick(Self);
  fmSelectTemplate.FormPosition(fmPrepareOutput.Top,fmPrepareOutput.Left,
                                fmPrepareOutput.Height,fmPrepareOutput.Width);
  if fmSelectTemplate.ShowModal = mrCancel then
    Exit;
  LoadTemplate(fmSelectTemplate.TemplateFileName);
  RefreshGrid;
  TemplateSaved := True;
end;

end.
