unit selbysql;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Grids, DBGrids, StdCtrls, ExtCtrls, DB;

type
  TfmSelectBySQL = class(TForm)
    paTop: TPanel;
    paSelect: TPanel;
    buSelect: TButton;
    paCancel: TPanel;
    buCancel: TButton;
    paFiller: TPanel;
    paGrid: TPanel;
    Rolodex: TTabControl;
    DBGrid1: TDBGrid;
    laValue: TLabel;
    edValue: TEdit;
    procedure DBGrid1DblClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure buSelectClick(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RolodexMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure edValueChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    RolodexOffset: integer;
    function QuoteQuote(v: variant): string;
    procedure SetSelectedValues;
  public
    RolodexFieldIndex: integer;
    SearchFieldName: string;
    SearchFieldType: string;
    SelectedBoolean: boolean;
    SelectedCount: integer;
    SelectedNumber: integer;
    SelectedString: string;
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmSelectBySQL  : TfmSelectBySQL;

implementation

uses dmglob1, mastform, resources;

{$R *.DFM}

procedure TfmSelectBySQL.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

function TfmSelectBySQL.QuoteQuote(v: variant): string;
var
  i: integer;
  s: string;
begin
  Result := '';
  try
    s := v;
  except
    on EVariantError do
      Exit;
  end;
  for i := Length(s) downto 1 do
    begin
      Result := s[i] + Result;
      if s[i] = Chr(39) then
        Result := Chr(39) + Result;
    end;
end;

procedure TfmSelectBySQL.SetSelectedValues;
var
  i: integer;
  s: string;
begin
  if (edValue.Text <> '') and (DBGrid1.SelectedRows.Count = 0) then
    DBGrid1.SelectedRows.CurrentRowSelected := True;
  if dgMultiSelect in DBGrid1.Options
    then
      begin
        SelectedString := '';
        SelectedCount := DBGrid1.SelectedRows.Count;
        if DBGrid1.SelectedRows.Count > 0 then
          with DBGrid1.DataSource.DataSet do
            for i := 0 to DBGrid1.SelectedRows.Count-1 do
              begin
                GotoBookmark(pointer(DBGrid1.SelectedRows.Items[i]));
                if i > 0 then
                  SelectedString := SelectedString + ',';
                s := QuoteQuote(Fields[0].AsString);
                SelectedString := SelectedString + '''' + s + '''';
              end;
      end
    else
      begin
        SelectedCount := 1;
        SelectedString := QuoteQuote(dmGlobalC.FindBySQL.Fields[0].Value);
        if (Copy(SelectedString,1,1) = '0') or
         (Copy(SelectedString,1,1) = 'T')
          then SelectedBoolean := True
          else SelectedBoolean := False;
        try
          SelectedNumber := dmGlobalC.FindBySQL.Fields[0].Value;
        except
          on EVariantError do
            SelectedNumber := 0;
        end;
      end;
end;

procedure TfmSelectBySQL.FormActivate(Sender: TObject);
begin
  SelectedBoolean := False;
  SelectedNumber := 0;
  SelectedString := '';
  with dmGlobalC.FindBySQL do
    begin
      Open;
      if not Eof then
        begin
          buSelect.Enabled := RecordCount > 0;
          if Rolodex.Visible then
            begin
              while Fields[RolodexFieldIndex].IsNull do
                Next;
              if Fields[RolodexFieldIndex].Value > 'Z'
                then RolodexOffset := 97
                else RolodexOffset := 65;
            end;
          First;
        end;
    end;
  Rolodex.TabIndex := 0;
  edValue.Clear;
  buCancel.SetFocus;
end;

procedure TfmSelectBySQL.DBGrid1DblClick(Sender: TObject);
begin
  SetSelectedValues;
  Close;
end;

procedure TfmSelectBySQL.buSelectClick(Sender: TObject);
begin
  SetSelectedValues;
  Close;
end;

procedure TfmSelectBySQL.buCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfmSelectBySQL.FormClose(Sender: TObject;var Action: TCloseAction);
begin
  dmGlobalC.FindBySQL.Close;
end;

procedure TfmSelectBySQL.RolodexMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  FirstLetter : Char;
  FirstRec,LastRec,MidRec : longint;
begin
  with dmGlobalC.FindBySQL do
    begin
      FirstLetter := chr(RolodexOffset + Rolodex.TabIndex);
      FirstRec := 1;
      LastRec := RecordCount;
      DisableControls;
      while LastRec > FirstRec do
        begin
          First;
          MidRec := (FirstRec + LastRec) div 2;
          MoveBy(MidRec-1);
          if Fields[RolodexFieldIndex].Value > FirstLetter
            then LastRec := Midrec
            else FirstRec := Midrec + 1;
        end;
      if Fields[RolodexFieldIndex].Value < FirstLetter then Next;
      EnableControls;
    end;
end;

procedure TfmSelectBySQL.edValueChange(Sender: TObject);
begin
  if edValue.Text = '' then
    Exit;
  with dmGlobalC.FindBySQL do
    if SearchFieldType = 'T'
      then Locate(SearchFieldName,edValue.Text,[loCaseInsensitive,loPartialKey])
      else Locate(SearchFieldName,StrToInt(edValue.Text),[loCaseInsensitive,loPartialKey]);
end;

procedure TfmSelectBySQL.FormCreate(Sender: TObject);
begin
  Self.Caption := selbysql_fmSelectBySQL;
  buSelect.Caption := selbysql_buSelect;
  laValue.Caption := selbysql_laValue;
  buCancel.Caption := selbysql_buCancel;
end;

end.
