unit selrec;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, DBCtrls, ComCtrls, StdCtrls, ExtCtrls, mastform, gqclasses,
  Buttons, DB, DBClient;

type
  TfmSelectRecords = class(TForm)
    paTop: TPanel;
    stbSelectRecords: TStatusBar;
    sgSelectRecords: TStringGrid;
    paControls: TPanel;
    buCombineConditions: TButton;
    buClearAllConditions: TButton;
    paTablesColumns: TPanel;
    paCurrentSelection: TPanel;
    paExit: TPanel;
    laTablesColumns: TLabel;
    cbTablesColumns: TComboBox;
    laCurrentSelection: TLabel;
    cbCurrentSelection: TComboBox;
    buMainScreen: TButton;
    buClearCondition: TButton;
    buCancel: TButton;
    paFiller: TPanel;
    buDBTree: TButton;
    sbZoomIn: TSpeedButton;
    sbZoomOut: TSpeedButton;
    sbShowTable: TSpeedButton;
    procedure sgSelectRecordsClick(Sender: TObject);
    procedure buClearAllConditionsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure buClearConditionClick(Sender: TObject);
    procedure cbCurrentSelectionChange(Sender: TObject);
    procedure buCombineConditionsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buMainScreenClick(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure cbTablesColumnsChange(Sender: TObject);
    procedure buDBTreeClick(Sender: TObject);
    procedure sbZoomInClick(Sender: TObject);
    procedure sbZoomOutClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure sbShowTableClick(Sender: TObject);
  private
    CaptionStyle: integer;
    LongText: string;
    ShortText: string;
    SQLText: string;
    function AgeCondition(var ShortText,LongText,SQLText: string): boolean;
    function BooleanCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
    procedure BuildCondition;
    procedure CreateCondition(ConditionType: char;Keys: TStrings);
    procedure ClearAllConditions;
    procedure ClearCondition(i: integer);
    procedure CreateAgeDataTable;
    function KeyValuesCondition(var ShortText,LongText,SQLText: string;i: integer;st: string): boolean;
    function NullCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
    function NumericCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
    procedure PopulateAgetsmaTable;
    procedure RefreshGrid;
    procedure RefreshGridRow(const n: integer);
    procedure SetConditionStringLists(CurrentQ: integer);
    function StringCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
    function TextCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
    procedure UpdateDisplayGridCell;
  public
    cdsCTable: TClientDataSet;
    ConditionCount: integer;
    CurrentQ: integer;
    SelectAll: boolean;
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmSelectRecords: TfmSelectRecords;
  Conditions: array[1..MaxConditions] of TCondition;

implementation

uses selbysql, dmglob1, dmquery1, selnum,
  selstring, selage, startinf, ordtable, dbtree, dispgridcell, selnull,
  cmbcond1, dispcondtable, resources;

{$R *.DFM}

function TfmSelectRecords.AgeCondition(var ShortText,LongText,SQLText: string): boolean;
var
  tmpCursor: TCursor;
begin
  Result := False;
  with dmQuery.cdsAgetsma do
    begin
      Open;
      if Eof then
        begin
          tmpCursor := Screen.Cursor;
          Screen.Cursor := crSQLWait;
          PopulateAgetsmaTable;
          Screen.Cursor := tmpCursor;
        end;
      Close;
    end;
  if not dmQuery.AgeTableExists(SearchTable.GetName + '_agedata') then
    begin
      tmpCursor := Screen.Cursor;
      Screen.Cursor := crSQLWait;
      CreateAgeDataTable;
      Screen.Cursor := tmpCursor;
    end;
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select CTRYHOST from ' + SearchTable.GetName + ' ' +
                     'group by CTRYHOST';
      Open;
      fmSelectAge.cbCTRYHOST.Clear;
      while not EOF do
        begin
          if not FieldByName('CTRYHOST').IsNull then
            fmSelectAge.cbCTRYHOST.Items.Add(FieldByName('CTRYHOST').Value);
          Next;
        end;
      Close;
    end;
  fmSelectAge.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                           fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectAge.ShowModal;
  if fmSelectAge.AgeString = '' then
    Exit;
  ShortText := '[' + SearchTable.GetName + ']' + ' WHERE ' + fmSelectAge.AgeString;
  LongText := '[' + SearchTable.GetCaption + ']' + ' WHERE ' + fmSelectAge.AgeString;
  SQLText := fmSelectAge.AgeString;
  Result := True;
end;

function TfmSelectRecords.BooleanCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
var
  tf: string;
begin
  if (fmMaster.GQVersion = 'C_S') or (fmMaster.AliasDriver = 'PGSQL') or (fmMaster.DBVersion = '3.22')
    then tf := '''F'''
    else tf := '0';
  Result := False;
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select ' + SearchTable.GetName + '.' + SearchFields[i].GetName +
                     ',count(*) as HITS from ' + SearchTable.GetName + ' ';
      if SearchTable.GetJunction <> '' then
        CommandText := CommandText + 'where ' + SearchTable.GetName + '.' + SearchTable.GetKey(1) + ' in ' +
                       '(select ' + SearchTable.GetKey(1) + ' from ' + SearchTable.GetJunction + ') ';
      CommandText := CommandText + 'GROUP by ' + SearchTable.GetName + '.' + SearchFields[i].GetName;
    end;
  fmSelectBySQL.Rolodex.Visible := False;
  fmSelectBySQL.RolodexFieldIndex := 2;
  fmSelectBySQL.edValue.Enabled := False;
  fmSelectBySQL.laValue.Enabled := False;
  fmSelectBySQL.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                             fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectBySQL.ShowModal;
  if fmSelectBySQL.SelectedString = '' then
    Exit;
  ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + ']';
  LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + ']';
  if fmSelectBySQL.SelectedBoolean
    then
      begin
        ShortText := ShortText + ' ' + selrec_IsTrue;
        LongText := LongText + ' ' + selrec_IsTrue;
        SQLText := SearchFields[i].GetName + ' <> ' + tf;
      end
    else
      begin
        ShortText := ShortText + ' ' + selrec_IsFalse;
        LongText := LongText + ' ' + selrec_IsFalse;
        SQLText := SearchFields[i].GetName + ' = ' + tf;
      end;
  Result := True;
end;

procedure TfmSelectRecords.BuildCondition;
var
  ShortTextC: string;
  LongTextC: string;
  SQLTextC: string;
  RankSQL: string;
  i: integer;
  LogicalAND: string;
  ks: string;
  AgeDataTable: TClientDataSet;
begin
  ShortText := '';
  LongText := '';
  SQLText := '';
  for i := 1 to SearchFieldCount do
    begin
      ShortTextC := '';
      LongTextC := '';
      SQLTextC := '';
      case SearchFields[i].GetSearchType of
        'A' : if not AgeCondition(ShortTextC,LongTextC,SQLTextC) then
                Exit;
        'B' : if not BooleanCondition(ShortTextC,LongTextC,SQLTextC,i) then
                Exit;
        'K' : if not KeyValuesCondition(ShortTextC,LongTextC,SQLTextC,i,'K') then
                Exit;
        'N' : if not NumericCondition(ShortTextC,LongTextC,SQLTextC,i) then
                Exit;
        'R' : if not KeyValuesCondition(ShortTextC,LongTextC,SQLTextC,i,'R') then
                Exit;
        'S' : if not StringCondition(ShortTextC,LongTextC,SQLTextC,i) then
                Exit;
        'T' : if not TextCondition(ShortTextC,LongTextC,SQLTextC,i) then
                Exit;
        'Z' : if not NullCondition(ShortTextC,LongTextC,SQLTextC,i) then
                Exit;
        else Exit;
      end;
      if i = 1
        then LogicalAND := ''
        else LogicalAND := ' AND ';
      ShortText := ShortText + LogicalAND + '(' + ShortTextC + ')';
      LongText := LongText + LogicalAND + '(' + LongTextC + ')';
      SQLText := SQLText + LogicalAND + '(' + SQLTextC + ')';
    end;
  if SearchTable.GetRanked then
    begin
      fmOrdered.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                             fmSelectRecords.Height,fmSelectRecords.Width);
      fmOrdered.ShowModal;
      RankSQL := ' AND (RANK <= ' + IntToStr(fmOrdered.MaxRankValue) + ')';
      ShortText := ShortText + RankSQL;
      LongText := LongText + RankSQL;
      SQLText := SQLText + RankSQL;
    end;
  CreateCondition('S',fmDBTree.GetSearchTable(fmDBTree.GetTablenameIndex(SearchTable.GetName)).GetKeys);
  SetConditionStringLists(CurrentQ);
  Conditions[CurrentQ].SetShortText(ShortText);
  Conditions[CurrentQ].SetLongText(LongText);
  if SearchFields[1].GetSearchType = 'A'
    then
      begin
        AgeDataTable := dmQuery.cdsAgeTables.Items[dmQuery.AgeTableIndex(SearchTable.GetName + '_agedata')] as TClientDataSet;
        AgeDataTable.Filter := SQLText;
        AgeDataTable.Filtered := True;
        AgeDataTable.Open;
        Conditions[CurrentQ].Table.Open;
        while not AgeDataTable.Eof do
          begin
            with Conditions[CurrentQ].Table do
              begin
                Append;
                for i := 0 to SearchTable.GetKeyCount - 1 do
                  begin
                    ks := SearchTable.GetKey(i);
                    FieldByName(ks).Value := AgeDataTable.FieldByName(ks).Value;
                  end;
                Post;
              end;
            AgeDataTable.Next;
          end;
        Conditions[CurrentQ].SetHits(Format('%5d',[Conditions[CurrentQ].Table.RecordCount]));
        Conditions[CurrentQ].Table.Close;
        AgeDataTable.Close;
        AgeDataTable.Filtered := False;
        AgeDataTable.Filter := '';
      end
    else with dmGlobalC.FindBySQL do
      begin
        Close;
        CommandText := 'select ' + SearchTable.GetSQLKeyString + ' ';
        if SearchTable.GetJunction <> ''
          then CommandText := CommandText + 'from ' + SearchTable.GetJunction + ',' +
                              SearchTable.GetName + ' where '
          else CommandText := CommandText + 'from ' + SearchTable.GetName + ' where ';
        CommandText := CommandText + SQLText;
        if SearchTable.GetJunction <> '' then
          CommandText := CommandText + ' AND (' + SearchTable.GetName + '.' + SearchTable.GetKey(1) +
                         ' = ' + SearchTable.GetJunction + '.' + SearchTable.GetKey(1) + ')';
        Open;
        Conditions[CurrentQ].Table.Open;
        while not EOF do
          begin
            with Conditions[CurrentQ].Table do
              begin
                Append;
                for i := 0 to SearchTable.GetKeyCount - 1 do
                  begin
                    ks := SearchTable.GetKey(i);
                    FieldByName(ks).Value := dmGlobalC.FindBySQL.FieldByName(ks).Value;
                  end;
                Post;
              end;
            Next;
          end;
        Conditions[CurrentQ].SetHits(Format('%5d',[Conditions[CurrentQ].Table.RecordCount]));
        Conditions[CurrentQ].Table.Close;
        Close;
      end;
  TempTables := True;
  RefreshGridRow(CurrentQ);
  cbCurrentSelection.ItemIndex := CurrentQ;
  sbShowTable.Enabled := True;
end;

procedure TfmSelectRecords.ClearAllConditions;
var
  i: integer;
begin
  i := 1;
  while (ConditionCount > 0) and (i <= MaxConditions) do
    begin
      ClearCondition(i);
      inc(i);
    end;
  CurrentQ := 1;
end;

procedure TfmSelectRecords.ClearCondition(i: integer);
begin
  if Conditions[i] <> nil then
    begin
      FreeAndNil(Conditions[i]);
      sgSelectRecords.Cells[1,i] := '';
      sgSelectRecords.Cells[2,i] := '';
      dec(ConditionCount);
    end;
  UpdateDisplayGridCell;
end;

procedure TfmSelectRecords.CreateAgeDataTable;
var
  AgeDataTable: TClientDataSet;
  AgeDataTableIndex: integer;
  BadData: TStringList;
  CurrentDir: string;
  Good: boolean;
  StartMA: double;
  EndMA: double;
  SearchTableName: string;
  i: integer;
  ks: string;
  s:string;
  n: integer;
begin
  SearchTableName := SearchTable.GetName;
  // if these tables exist (leftover), delete them
  dmQuery.DeleteAgeTable(SearchTableName + '_agedata');
  // create agedata table
  dmQuery.AddAgeTable(SearchTableName + '_agedata');
  dmQuery.cdsAgeTables.Add(TClientDataSet.Create(dmQuery));
  AgeDataTableIndex := dmQuery.AgeTableIndex(SearchTableName + '_agedata');
  AgeDataTable := dmQuery.cdsAgeTables.Items[AgeDataTableIndex] as TClientDataSet;
  with AgeDataTable do
    begin
      for i := 0 to SearchTable.GetKeyCount - 1 do
        FieldDefs.Add(SearchTable.GetKey(i),ftInteger,0,True);
      FieldDefs.Add('CTRYHOST',ftString,30,False);
      FieldDefs.Add('STARTMA',ftFloat,0,False);
      FieldDefs.Add('ENDMA',ftFloat,0,False);
      IndexFieldNames := SearchTable.GetKeyString;
      IndexFieldNames := StringReplace(IndexFieldNames,',',';',[rfReplaceAll]);
      CreateDataSet;
    end;
  AgeDataTable.Open;
  BadData := TStringList.Create;
  try
    with dmGlobalC.FindBySQL do
      begin
        with dmQuery.cdsTabledoc do
          begin
            Filtered := False;
            Open;
            Locate('TABLENAME',SearchTableName,[]);
            n := FieldByName('RECORDCOUNT').AsInteger;
            Close;
          end;
        if SearchTable.GetJunction = ''
          then CommandText := 'select * from ' + SearchTableName
          else
            begin
              CommandText := 'select ' + SearchTable.GetJunction + '.' + SearchTable.GetKey(0) + ',';
              for i := 1 to SearchTable.GetKeyCount - 1 do
                CommandText := CommandText + SearchTableName + '.' + SearchTable.GetKey(i) + ',';
              CommandText := CommandText + 'STARTAGE,STARTQU,ENDAGE,ENDQU,CTRYHOST from ' +
                             SearchTable.GetJunction + ',' + SearchTableName + ' where ' +
                             '(' + SearchTableName + '.' + SearchTable.GetKey(1) +
                             ' = ' + SearchTable.GetJunction + '.' + SearchTable.GetKey(1) + ')';
            end;
        Open;
        fmStartInfo.Label1.Caption := selrec_PopulatingAgeTable;
        fmStartInfo.pbInfo.Position := 0;
        fmStartInfo.pbInfo.Min := 0;
        fmStartInfo.pbInfo.Step := 1;
        fmStartInfo.pbInfo.Max := n;
        fmStartInfo.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                                 fmSelectRecords.Height,fmSelectRecords.Width);
        fmStartInfo.Show;
        fmStartInfo.Update;
        while not EOF do
          begin
            StartMA := 10000.0;
            EndMA := 0.0;
            Good := True;
            dmQuery.cdsAgetsma.Open;
            if not FieldByName('STARTAGE').IsNull
              then
                begin
                  s := FieldByName('STARTAGE').Value;
                  if not FieldByName('STARTQU').IsNull then
                    s := s + ', ' + FieldByName('STARTQU').Value;
                  if dmQuery.cdsAgetsma.Locate('AGECOMBO',s,[])
                    then
                      begin
                        StartMA := dmQuery.cdsAgetsma.FieldByName('STARTMA').Value;
                        EndMA := dmQuery.cdsAgetsma.FieldByName('ENDMA').Value;
                      end
                    else Good := False;
                end
              else Good := False;
            if not FieldByName('ENDAGE').IsNull then
              begin
                s := FieldByName('ENDAGE').Value;
                if not FieldByName('ENDQU').IsNull then
                  s := s + ', ' + FieldByName('ENDQU').Value;
                if dmQuery.cdsAgetsma.Locate('AGECOMBO',s,[])
                  then
                    begin
                      if StartMA = 10000.0 then
                        StartMA := dmQuery.cdsAgetsma.FieldByName('STARTMA').Value;
                      EndMA := dmQuery.cdsAgetsma.FieldByName('ENDMA').Value;
                    end
                  else Good := False;
              end;
            dmQuery.cdsAgetsma.Close;
            if Good
              then
                begin
                  AgeDataTable.Append;
                  for i := 0 to SearchTable.GetKeyCount - 1 do
                    begin
                      ks := SearchTable.GetKey(i);
                      AgeDataTable.FieldByName(ks).Value := FieldByName(ks).Value;
                    end;
                  AgeDataTable.FieldByName('CTRYHOST').Value := FieldByName('CTRYHOST').Value;
                  AgeDataTable.FieldByName('STARTMA').Value := StartMA;
                  AgeDataTable.FieldByName('ENDMA').Value := EndMA;
                  AgeDataTable.Post;
                end
              else BadData.Add(IntToStr(FieldByName(SearchTable.GetKey(0)).Value));
            fmStartInfo.pbInfo.StepIt;
            Next;
          end;
        Close;
      end;
    if BadData.Count > 0 then
      begin
        BadData.SaveToFile(SearchTableName + '_baddata.txt');
        GetDir(0,CurrentDir);
        MessageDlg(Format(selrec_UnreferencedValues,[SearchTableName,CurrentDir,
                                                     SearchTableName,e_key]),mtWarning,[mbOK],0);
      end;
  finally
    BadData.Free;
  end;
  fmStartInfo.Hide;
  fmSelectRecords.Update;
  AgeDataTable.Close;
end;

procedure TfmSelectRecords.CreateCondition(ConditionType: char;Keys: TStrings);
var
  i: integer;
begin
  if Conditions[CurrentQ] <> nil then
    ClearCondition(CurrentQ);
  Conditions[CurrentQ] := TCondition.Create(CurrentQ,ConditionType,fmSelectRecords);
  with Conditions[CurrentQ].Table do
    begin
      for i := 0 to Keys.Count - 1 do
        FieldDefs.Add(Keys[i],ftInteger,0,True);
      IndexFieldNames := Keys.CommaText;
      IndexFieldNames := StringReplace(IndexFieldNames,',',';',[rfReplaceAll]);
      CreateDataSet;
    end;
  inc(ConditionCount);
end;

procedure TfmSelectRecords.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

procedure TfmSelectRecords.PopulateAgetsmaTable;
var
  n: integer;
begin
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select count(*) as CNT from agetsma';
      Open;
      n := FieldByName('CNT').AsInteger;
      Close;
      fmStartInfo.Label1.Caption := selrec_PopulatingGeologicAgeLookupTable;
      fmStartInfo.pbInfo.Position := 0;
      fmStartInfo.pbInfo.Min := 0;
      fmStartInfo.pbInfo.Step := 1;
      fmStartInfo.pbInfo.Max := n;
      fmStartInfo.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                               fmSelectRecords.Height,fmSelectRecords.Width);
      fmStartInfo.Show;
      fmStartInfo.Update;
      CommandText := 'select AGECOMBO,STARTMA,ENDMA from agetsma order by AGECOMBO';
      Open;
      dmQuery.cdsAgetsma.Open;
      while not Eof do
        begin
          dmQuery.cdsAgetsma.Append;
          dmQuery.cdsAgetsma.FieldByName('AGECOMBO').Value := FieldByName('AGECOMBO').AsString;
          dmQuery.cdsAgetsma.FieldByName('STARTMA').Value := FieldByName('STARTMA').AsFloat;
          dmQuery.cdsAgetsma.FieldByName('ENDMA').Value := FieldByName('ENDMA').AsFloat;
          dmQuery.cdsAgetsma.Post;
          fmStartInfo.pbInfo.StepIt;
          Next;
        end;
      Close;
      dmQuery.cdsAgetsma.Close;
    end;
  fmStartInfo.Hide;
  fmSelectRecords.Update;
end;

procedure TfmSelectRecords.RefreshGrid;
var
  i : integer;
begin
  i := 1;
  while (ConditionCount > 0) and (i <= MaxConditions) do
    begin
      if Conditions[i] <> nil then
        RefreshGridRow(i);
      inc(i);
    end;
  UpdateDisplayGridCell;
end;

function TfmSelectRecords.KeyValuesCondition(var ShortText,LongText,
  SQLText: string;i: integer;st: string): boolean;
begin
  Result := False;
  with dmGlobalC.FindBySQL do
    begin
      Close;
      if entity = 'DG'
        then
          begin
            CommandText := 'select dggeol.DGNO,dgname.DGNAME,dggeol.REVDATE ' +
                           'from dggeol left outer join dgname on dgname.DGNO = dggeol.DGNO ' +
                           'where dgname.RANK = 1 ';
            if st = 'R'
              then
                begin
                  CommandText := CommandText + 'order by dggeol.REVDATE desc';
                  fmSelectBySQL.SearchFieldName := 'REVDATE';
                  fmSelectBySQL.SearchFieldType := 'T';
                end
              else
                begin
                  CommandText := CommandText + 'order by dggeol.DGNO';
                  fmSelectBySQL.SearchFieldName := 'DGNO';
                  fmSelectBySQL.SearchFieldType := 'K';
                end;
          end
        else
          begin
            CommandText := 'select dpgeol.DEPNO,dpname.DEPNAME,dpgeol.REVDATE ' +
                           'from dpgeol left outer join dpname on dpname.DEPNO = dpgeol.DEPNO ' +
                           'where dpname.RANK = 1 ';
            if st = 'R'
              then
                begin
                  CommandText := CommandText + 'order by dpgeol.REVDATE desc';
                  fmSelectBySQL.SearchFieldName := 'REVDATE';
                  fmSelectBySQL.SearchFieldType := 'T';
                end
              else
                begin
                  CommandText := CommandText + 'order by dpgeol.DEPNO';
                  fmSelectBySQL.SearchFieldName := 'DEPNO';
                  fmSelectBySQL.SearchFieldType := 'K';
                end;
          end;
    end;
  fmSelectBySQL.Rolodex.Visible := False;
  fmSelectBySQL.RolodexFieldIndex := 0;
  fmSelectBySQL.edValue.Enabled := True;
  fmSelectBySQL.laValue.Enabled := True;
  fmSelectBySQL.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                             fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectBySQL.DBGrid1.Options := fmSelectBySQL.DBGrid1.Options + [dgMultiSelect];
  fmSelectBySQL.ShowModal;
  fmSelectBySQL.Rolodex.Visible := False;
  fmSelectBySQL.DBGrid1.Options := fmSelectBySQL.DBGrid1.Options - [dgMultiSelect];
  if fmSelectBySQL.SelectedString = ''
    then Exit
    else while AnsiPos('''',fmSelectBySQL.SelectedString) > 0 do
           Delete(fmSelectBySQL.SelectedString,AnsiPos('''',fmSelectBySQL.SelectedString),1);
  if fmSelectBySQL.SelectedCount > 1
    then
      begin
        ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] in (' +
                     fmSelectBySQL.SelectedString + ')';
        LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] in (' +
                    fmSelectBySQL.SelectedString + ')';
        SQLText := SearchFields[i].GetName + ' in (' + fmSelectBySQL.SelectedString + ')'
      end
    else
      begin
        ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] = ' +
                     fmSelectBySQL.SelectedString;
        LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] = ' +
                    fmSelectBySQL.SelectedString;
        SQLText := SearchFields[i].GetName + ' = ' + fmSelectBySQL.SelectedString
      end;
  Result := True;
end;

function TfmSelectRecords.NullCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
begin
  Result := False;
  case cbTablesColumns.ItemIndex of
    0 : fmSelectNull.gbFindValues.Caption := '[' + SearchTable.GetName + '].[' +
                                               SearchFields[i].GetName + ']';
    1 : fmSelectNull.gbFindValues.Caption := '[' + SearchTable.GetCaption + '].[' +
                                               SearchFields[i].GetCaption + ']';
    else ;
  end;
  fmSelectNull.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                              fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectNull.ShowModal;
  if fmSelectNull.TextString = '' then
    exit;
  ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] ' +
               fmSelectNull.TextString;
  LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] ' +
              fmSelectNull.TextString;
  SQLText := SearchFields[i].GetName + ' ' + fmSelectNull.TextString;
  Result := True;
end;

function TfmSelectRecords.NumericCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
begin
  Result := False;
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select min(' + SearchTable.GetName + '.' + SearchFields[i].GetName + ') as LOVALUE,' +
                     'max(' + SearchTable.GetName + '.' + SearchFields[i].GetName + ') as HIVALUE,' +
                     'count(*) as HITS ' +
                     'from ' + SearchTable.GetName + ' ';
      if SearchTable.GetJunction <> '' then
        CommandText := CommandText + 'where ' + SearchTable.GetName + '.' + SearchTable.GetKey(1) + ' in ' +
                       '(select ' + SearchTable.GetKey(1) + ' from ' + SearchTable.GetJunction + ') ';
    end;
  case cbTablesColumns.ItemIndex of
    0 : fmSelectNumeric.gbFindValues.Caption := 'Find [' + SearchTable.GetName + '].[' +
                                                SearchFields[i].GetName + ']';
    1 : fmSelectNumeric.gbFindValues.Caption := 'Find [' + SearchTable.GetCaption + '].[' +
                                                SearchFields[i].GetCaption + ']';
    else ;
  end;
  fmSelectNumeric.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                               fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectNumeric.ShowModal;
  if fmSelectNumeric.NumericString = '' then
    exit;
  ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] ' +
               fmSelectNumeric.NumericString;
  LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] ' +
              fmSelectNumeric.NumericString;
  SQLText := SearchFields[i].GetName + ' ' + fmSelectNumeric.NumericString;
  Result := True;
end;

procedure TfmSelectRecords.RefreshGridRow(const n: integer);
begin
  if CaptionStyle = 0
    then sgSelectRecords.Cells[1,n] := Conditions[n].GetShortText
    else sgSelectRecords.Cells[1,n] := Conditions[n].GetLongText;
  sgSelectRecords.Cells[2,n] := Conditions[n].GetHits;
  UpdateDisplayGridCell;
end;

procedure TfmSelectRecords.SetConditionStringLists(CurrentQ: integer);
var
  i: integer;
begin
  i := fmDBTree.GetTablenameIndex(SearchTable.GetName);
  Conditions[CurrentQ].SetKeys(fmDBTree.GetSearchTable(i).GetKeyString);
  Conditions[CurrentQ].SetTable(0,fmDBTree.GetSearchTable(i).GetName);
  Conditions[CurrentQ].SetCaption(0,fmDBTree.GetSearchTable(i).GetCaption);
  while fmDBTree.GetSearchTable(i).GetParent <> '[none]' do
    begin
      i := fmDBTree.GetTableNameIndex(fmDBTree.GetSearchTable(i).GetParent);
      Conditions[CurrentQ].SetTable(0,fmDBTree.GetSearchTable(i).GetName);
      Conditions[CurrentQ].SetCaption(0,fmDBTree.GetSearchTable(i).GetCaption);
    end;
  with Conditions[CurrentQ].GetTables do
    if Count > 2 then
      if fmDBTree.GetSearchTable(fmDBTree.GetTablenameIndex(Strings[1])).GetIsJunction then
        Conditions[CurrentQ].DropJunction(1);
end;

function TfmSelectRecords.StringCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
begin
  Result := False;
  case cbTablesColumns.ItemIndex of
    0 : fmSelectString.gbFindValues.Caption := '[' + SearchTable.GetName + '].[' +
                                               SearchFields[i].GetName + ']';
    1 : fmSelectString.gbFindValues.Caption := '[' + SearchTable.GetCaption + '].[' +
                                               SearchFields[i].GetCaption + ']';
    else ;
  end;
  fmSelectString.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                              fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectString.ShowModal;
  if fmSelectString.TextString = '' then
    exit;
  ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] ' +
               fmSelectString.TextString;
  LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] ' +
              fmSelectString.TextString;
  SQLText := SearchFields[i].GetName + ' ' + fmSelectString.TextString;
  Result := True;
end;

function TfmSelectRecords.TextCondition(var ShortText,LongText,SQLText: string;i: integer): boolean;
begin
  Result := False;
  with dmGlobalC.FindBySQL do
    begin
      Close;
      CommandText := 'select ' + SearchTable.GetName + '.' + SearchFields[i].GetName +
                     ',count(*) as HITS from ' + SearchTable.GetName + ' ';
      CommandText := CommandText + 'where ' + SearchTable.GetName + '.' + SearchFields[i].GetName +
                     ' is not Null ';
      if SearchTable.GetJunction <> '' then
        CommandText := CommandText + 'AND ' + SearchTable.GetName + '.' + SearchTable.GetKey(1) + ' in ' +
                       '(select ' + SearchTable.GetKey(1) + ' from ' + SearchTable.GetJunction + ') ';
      CommandText := CommandText + 'GROUP by ' + SearchTable.GetName + '.' + SearchFields[i].GetName + ' ' +
                     'ORDER by ' + SearchTable.GetName + '.' + SearchFields[i].GetName;
    end;
  fmSelectBySQL.Rolodex.Visible := True;
  fmSelectBySQL.RolodexFieldIndex := 0;
  fmSelectBySQL.edValue.Enabled := True;
  fmSelectbySQL.laValue.Enabled := True;
  fmSelectBySQL.SearchFieldName := SearchFields[i].GetName;
  fmSelectBySQL.SearchFieldType := 'T';
  fmSelectBySQL.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                             fmSelectRecords.Height,fmSelectRecords.Width);
  fmSelectBySQL.DBGrid1.Options := fmSelectBySQL.DBGrid1.Options + [dgMultiSelect];
  fmSelectBySQL.ShowModal;
  fmSelectBySQL.Rolodex.Visible := False;
  fmSelectBySQL.DBGrid1.Options := fmSelectBySQL.DBGrid1.Options - [dgMultiSelect];
  if fmSelectBySQL.SelectedString = '' then
    exit;
  if fmSelectBySQL.SelectedCount > 1
    then
      begin
        ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] in (' +
                     fmSelectBySQL.SelectedString + ')';
        LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] in (' +
                    fmSelectBySQL.SelectedString + ')';
        SQLText := SearchFields[i].GetName + ' in (' + fmSelectBySQL.SelectedString + ')'
      end
    else
      begin
        ShortText := '[' + SearchTable.GetName + '].[' + SearchFields[i].GetName + '] = ' +
                     fmSelectBySQL.SelectedString;
        LongText := '[' + SearchTable.GetCaption + '].[' + SearchFields[i].GetCaption + '] = ' +
                    fmSelectBySQL.SelectedString;
        SQLText := SearchFields[i].GetName + ' = ' + fmSelectBySQL.SelectedString
      end;
  Result := True;
end;

procedure TfmSelectRecords.UpdateDisplayGridCell;
begin
  if fmDisplayGridCell = nil then
    Exit;
  if fmDisplayGridCell.Visible then
    with fmDisplayGridCell do
      begin
        Memo1.Clear;
        Memo1.Lines.Add(sgSelectRecords.Cells[1,CurrentQ]);
        Memo1.SelStart := 0;
        Caption := selrec_ConditionNo + ' ' + IntToStr(CurrentQ);
        if Conditions[CurrentQ] <> nil
          then
            begin
              Memo2.Lines := Conditions[CurrentQ].GetKeys;
              Memo3.Lines := Conditions[CurrentQ].GetTables;
              Memo4.Lines := Conditions[CurrentQ].GetCaptions;
            end
          else
            begin
              Memo2.Clear;
              Memo3.Clear;
              Memo4.Clear;
            end;
      end;
end;

procedure TfmSelectRecords.sgSelectRecordsClick(Sender: TObject);
begin
  CurrentQ := sgSelectRecords.Row;
  cbCurrentSelection.ItemIndex := CurrentQ;
  UpdateDisplayGridCell;
  sbShowTable.Enabled := Conditions[CurrentQ] <> nil;
end;

procedure TfmSelectRecords.buClearAllConditionsClick(Sender: TObject);
begin
  ClearAllConditions;
  sbShowTable.Enabled := False;
end;

procedure TfmSelectRecords.FormCreate(Sender: TObject);
var
  i : integer;
begin
  Self.Caption := selrec_fmSelectRecords;
  laTablesColumns.Caption := selrec_laTablesColumns;
  cbTablesColumns.Items.CommaText := selrec_cbTablesColumns;
  laCurrentSelection.Caption := selrec_laCurrentSelection;
  buMainScreen.Caption := selrec_buMainScreen;
  buCancel.Caption := selrec_buCancel;
  buDBTree.Caption := selrec_buDBTree;
  buCombineConditions.Caption := selrec_buCombineConditions;
  buClearCondition.Caption := selrec_buClearCondition;
  buClearAllConditions.Caption := selrec_buClearAllConditions;
  sbShowTable.Hint := selrec_sbShowTableHint;
  sbZoomIn.Hint := selrec_sbZoomInHint;
  sbZoomOut.Hint := selrec_sbZoomOutHint;
  cbTablesColumns.ItemIndex := 1; // defaults to full English caption
  CaptionStyle := cbTablesColumns.ItemIndex;
  cbCurrentSelection.Clear;
  cbCurrentSelection.Items.Add(selrec_SelectAllRecords);
  ConditionCount := 0;
  with sgSelectRecords do
    begin
      RowCount := 1 + MaxConditions;
      FixedRows := 1;
      for i := 1 to MaxConditions do
        begin
          Cells[0,i] := Format('Q%2.2d',[i]);
          cbCurrentSelection.Items.Add(Format(selrec_FilterWithCondition + ' ''Q%2.2d''',[i]));
        end;
      Cells[1,0] := selrec_ConditionalExpression;
      Cells[2,0] := selrec_Hits;
      ColWidths[0] := 30;
      ColWidths[1] := 700;
    end;
  CurrentQ := 1;
  sgSelectRecords.Row := 1;
  cbCurrentSelection.ItemIndex := 0;
  sbShowTable.Visible := ((LowerCase(UserInitials) = 'rml') or (LowerCase(UserInitials) = 'test'));
end;

procedure TfmSelectRecords.FormDestroy(Sender: TObject);
begin
  ClearAllConditions;
  if fmOrdered <> nil then
    FreeAndNil(fmOrdered);
  if fmSelectBySQL <> nil then
    FreeAndNil(fmSelectBySQL);
  if fmSelectAge <> nil then
    FreeAndNil(fmSelectAge);
  if fmSelectNull <> nil then
    FreeAndNil(fmSelectNull);
  if fmSelectNumeric <> nil then
    FreeAndNil(fmSelectNumeric);
  if fmSelectString <> nil then
    FreeAndNil(fmSelectString);
  if fmCombineConditions <> nil then
    FreeAndNil(fmCombineConditions);
  if fmDisplayGridCell <> nil then
    FreeAndNil(fmDisplayGridCell);
  if fmDisplayConditionTable <> nil then
    FreeAndNil(fmDisplayConditionTable);
end;

procedure TfmSelectRecords.buClearConditionClick(Sender: TObject);
begin
  ClearCondition(CurrentQ);
  sbShowTable.Enabled := False;
end;

procedure TfmSelectRecords.cbCurrentSelectionChange(Sender: TObject);
begin
  with cbCurrentSelection do
    if ItemIndex > 0 then
      begin
        CurrentQ := ItemIndex;
        sgSelectRecords.Row := ItemIndex;
        UpdateDisplayGridCell;
      end;
end;

procedure TfmSelectRecords.buCombineConditionsClick(Sender: TObject);
var
  i: integer;
  ks: string;
  ss: TStringList;
begin
  if Conditions[CurrentQ] <> nil then
    if MessageDlg(selrec_OverwriteCurrentCondition,
                  mtConfirmation,[mbOK,mbCancel],0) = mrCancel
      then Exit;
  if ConditionCount = 0 then
    begin
      ShowMessage(selrec_NoConditionsToCombine);
      Exit;
    end;
  if fmCombineConditions = nil then
    fmCombineConditions := TfmCombineConditions.Create(Self);
  with fmCombineConditions do
    begin
      FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,
                   fmSelectRecords.Height,fmSelectRecords.Width);
      ShowModal;
      if SQLString = '' then
        Exit;
      if LowerCase(UserInitials) = 'test' then
        ShowMessage(SQLString + chr(10) + chr(10) +
                    selrec_ShortText + ' ' + ConditionShortText + chr(10) + chr(10) +
                    selrec_LongText + ' ' + ConditionLongText + chr(10) + chr(10) +
                    selrec_Captions + ' ' + CCCaptions.CommaText + chr(10) + chr(10) +
                    selrec_Keys + ' ' + CCKeys.CommaText + chr(10) + chr(10) +
                    selrec_Tables + ' ' + CCTables.CommaText);
    end;
  ss := TStringList.Create;
  try
    ss.AddStrings(fmCombineConditions.CCKeys);
    CreateCondition('C',ss);
  finally
    ss.Free;
  end;
  with Conditions[CurrentQ] do
    begin
      SetShortText(fmCombineConditions.ConditionShortText);
      SetLongText(fmCombineConditions.ConditionLongText);
      SetCaptions(fmCombineConditions.CCCaptions);
      SetKeys(fmCombineConditions.CCKeys.CommaText);
      SetTables(fmCombineConditions.CCTables);
    end;
  cdsCTable.Open;
  Conditions[CurrentQ].Table.Open;
  while not cdsCTable.Eof do
    begin
      with Conditions[CurrentQ].Table do
        begin
          Append;
          for i := 0 to Conditions[CurrentQ].GetKeys.Count - 1 do
            begin
              ks := Conditions[CurrentQ].GetKey(i);
              FieldByName(ks).Value := cdsCTable.FieldByName(ks).Value;
            end;
          Post;
        end;
      cdsCTable.Next;
    end;
  Conditions[CurrentQ].SetHits(Format('%5d',[Conditions[CurrentQ].Table.RecordCount]));
  Conditions[CurrentQ].Table.Close;
  cdsCTable.Close;
  cdsCTable.Free;
  RefreshGridRow(CurrentQ);
  sbShowTable.Enabled := True;
end;

procedure TfmSelectRecords.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Conditions[CurrentQ] = nil then
    cbCurrentSelection.ItemIndex := 0;
  SelectAll := cbCurrentSelection.ItemIndex = 0;
  if (fmDisplayGridCell <> nil) and fmDisplayGridCell.Visible then
    fmDisplayGridCell.Hide;
end;

procedure TfmSelectRecords.buMainScreenClick(Sender: TObject);
begin
  CurrentQ := sgSelectRecords.Row;
  Close;
end;

procedure TfmSelectRecords.buCancelClick(Sender: TObject);
begin
  cbCurrentSelection.ItemIndex := 0;
  Close;
end;

procedure TfmSelectRecords.cbTablesColumnsChange(Sender: TObject);
begin
  CaptionStyle := cbTablesColumns.ItemIndex;
  RefreshGrid;
end;

procedure TfmSelectRecords.buDBTreeClick(Sender: TObject);
begin
  if fmDBTree <> nil then
    if fmDBTree.CallingForm <> 'fmSelectRecords' then
      FreeAndNil(fmDBTree);
  if fmDBTree = nil then
    begin
      fmDBTree := TfmDBTree.Create(nil);
      fmDBTree.CallingForm := 'fmSelectRecords';
    end;
  if fmOrdered = nil then
    fmOrdered := TfmOrdered.Create(Self);
  if fmSelectBySQL = nil then
    fmSelectBySQL := TfmSelectBySQL.Create(Self);
  if fmSelectAge = nil then
    fmSelectAge := TfmSelectAge.Create(Self);
  if fmSelectNull = nil then
    fmSelectNull := TfmSelectNull.Create(Self);
  if fmSelectNumeric = nil then
    fmSelectNumeric := TfmSelectNumeric.Create(Self);
  if fmSelectString = nil then
    fmSelectString := TfmSelectString.Create(Self);
  if Conditions[CurrentQ] <> nil then
    if MessageDlg(selrec_OverwriteCurrentCondition,mtConfirmation,[mbOK,mbCancel],0) = mrCancel
      then Exit
      else ClearCondition(CurrentQ);
  dmQuery.cdsColumndoc.Close;
  fmDBTree.FormPosition(fmSelectRecords.Top,fmSelectRecords.Left,fmSelectRecords.Height,fmSelectRecords.Width);
  fmDBTree.CaptionStyle := CaptionStyle;
  fmDBTree.ShowModal;
  fmSelectRecords.Update;
  if SearchFieldCount = 0
    then Exit
    else BuildCondition;
end;

procedure TfmSelectRecords.sbZoomInClick(Sender: TObject);
begin
  if fmDisplayGridCell = nil then
    fmDisplayGridCell := TfmDisplayGridCell.Create(Self);
  sbZoomOut.Enabled := True;
  fmDisplayGridCell.Show;
  UpdateDisplayGridCell;
end;

procedure TfmSelectRecords.sbZoomOutClick(Sender: TObject);
begin
  fmDisplayGridCell.Hide;
end;

procedure TfmSelectRecords.FormShow(Sender: TObject);
begin
  if sbZoomIn.Down
    then fmDisplayGridCell.Show;
end;

procedure TfmSelectRecords.sbShowTableClick(Sender: TObject);
var
  TableOpen: boolean;
begin
  if fmDisplayConditionTable = nil then
    fmDisplayConditionTable := TfmDisplayConditionTable.Create(Self);
  TableOpen := Conditions[CurrentQ].Table.Active;
  dmQuery.dsConditionTable.DataSet := Conditions[CurrentQ].Table;
  if not TableOpen then
    Conditions[CurrentQ].Table.Open;
  fmDisplayConditionTable.ShowModal;
  Conditions[CurrentQ].Table.Active := TableOpen;
end;

end.
