unit seltmpl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls{, Filectrl}, ExtCtrls, Grids, DirectoryEdit;

type
  TfmSelectTemplate = class(TForm)
    paBottom: TPanel;
    buNext: TButton;
    buCancel: TButton;
    sgTemplates: TStringGrid;
    deCurrentDirectory: TDirectoryEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure buNextClick(Sender: TObject);
    procedure deCurrentDirectoryChange(Sender: TObject);
  private
    { Private declarations }
    CurrentRow: integer;
    Description: string;
    Dir: string;
    SearchRec: TSearchRec;
    TemplateList: TStringList;
    WhoWhen: string;
    function GetTemplate(const fn: string): string;
    procedure KeepTemplate;
  public
    TemplateFileName: string;
    procedure FormPosition(const ParentTop, ParentLeft, ParentHeight,
      ParentWidth: integer);
  end;

var
  fmSelectTemplate: TfmSelectTemplate;

implementation

uses mastform, resources;

{$R *.DFM}

procedure TfmSelectTemplate.FormPosition(const ParentTop,ParentLeft,ParentHeight,ParentWidth: integer);
begin
  Top := ParentTop + Trunc((ParentHeight - Height)/2);
  Left := ParentLeft + Trunc((ParentWidth - Width)/2);
end;

function TfmSelectTemplate.GetTemplate(const fn: string): string;
var
  f: TextFile;
  e: string;
begin
  AssignFile(f,Dir + '\' + fn);
  Reset(f);
  Readln(f,e);
  Readln(f,WhoWhen);
  Readln(f,Description);
  CloseFile(f);
  Result := e;
end;

procedure TfmSelectTemplate.KeepTemplate;
begin
  TemplateList.Add(Dir + '\' + SearchRec.Name);
  sgTemplates.Cells[1,CurrentRow] := WhoWhen;
  sgTemplates.Cells[2,CurrentRow] := Description;
  inc(CurrentRow);
end;

procedure TfmSelectTemplate.FormCreate(Sender: TObject);
begin
  Self.Caption := seltmpl_fmSelectTemplate;
  buNext.Caption := seltmpl_buNext;
  buCancel.Caption := seltmpl_buCancel;
  with sgTemplates do
    begin
      sgTemplates.RowCount := MaxTemplates + 1;
      Cells[1,0] := seltmpl_CreationDate;
      Cells[2,0] := seltmpl_Description;
      ColWidths[0] := 10;
      ColWidths[1] := 120;
      ColWidths[2] := 400;
    end;
  sgTemplates.Row := 1;
  TemplateList := TStringList.Create;
  Dir := '';
end;

procedure TfmSelectTemplate.FormShow(Sender: TObject);
var
  i: integer;
begin
  if Dir = '' then
    GetDir(0,Dir);
  deCurrentDirectory.Text := Dir;
  if FindFirst(Dir + '\*.gqt',faAnyFile,SearchRec) = 0
    then
      begin
        for i := 1 to MaxTemplates do
          begin
            sgTemplates.Cells[1,i] := '';
            sgTemplates.Cells[2,i] := '';
          end;
        TemplateList.Clear;
        CurrentRow := 1;
        if GetTemplate(SearchRec.Name) = entity then
          KeepTemplate;
        while FindNext(SearchRec) = 0 do
          if GetTemplate(SearchRec.Name) = entity then
            KeepTemplate;
        FindClose(SearchRec);
        if TemplateList.Count = 0 then
          begin
            ShowMessage(seltmpl_NoTemplateForCurrentEntity);
            buNext.Enabled := False;;
            Exit;
          end;
        TemplateFileName := '';
        sgTemplates.Row := 1;
        buNext.Enabled := True;
      end
    else ShowMessage(seltmpl_NoTemplateFoundInThisDirectory);
end;

procedure TfmSelectTemplate.FormDestroy(Sender: TObject);
begin
  TemplateList.Free;
end;

procedure TfmSelectTemplate.buNextClick(Sender: TObject);
begin
  if sgTemplates.Row > TemplateList.Count then
    begin
      ShowMessage(seltmpl_SelectRowNotEmpty);
      ModalResult := mrCancel;
      Exit;
    end;
  TemplateFileName := TemplateList.Strings[sgTemplates.Row - 1];
end;

procedure TfmSelectTemplate.deCurrentDirectoryChange(Sender: TObject);
var
  SaveDir: string;
begin
  SaveDir := Dir;
  Dir := deCurrentDirectory.Text;
  FormShow(Self);
  Dir := SaveDir;
  ChDir(Dir);
end;

end.
