unit autoserverDG_SA;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  ComObj, ActiveX, GShellADO_SA_TLB, StdVcl, Variants;

type
  TDGServer = class(TAutoObject, IDGServer)
  protected
    function AddDGRecord(Latitude, Longitude: Double): Integer; safecall;
    function Get_CurrentMDBFileName: WideString; safecall;
    function Get_Datum: WideString; safecall;
    function Get_DGNO: Integer; safecall;
    function Get_Latitude: Double; safecall;
    function Get_Longitude: Double; safecall;
    procedure DeleteDGDigitizedRecord(DGNO: Integer); safecall;
    procedure DeleteDGRecord(DGNO: Integer); safecall;
    procedure GetDGCompilationProgress(DGNO: Integer; out Compstage,
      Proglevel: WideString); safecall;
    procedure GetDGDigitizedRecord(DGNO: Integer; out X, Y: Double;
      out EPSG: Integer); safecall;
    procedure GetDGLatLong(DGNO: Integer; out Latitude, Longitude: Double);
      safecall;
    procedure LocateDGRecord(DGNO: Integer); safecall;
    procedure UpdateDGDatum(DGNO: Integer; const Datum: WideString); safecall;
    procedure UpdateDGDigitizedRecord(DGNO: Integer; X, Y: Double;
      EPSG: Integer); safecall;
    procedure UpdateDGLatLong(DGNO: Integer; Latitude, Longitude: Double);
      safecall;

  end;

implementation

uses ComServ, dmglob1, dpgroups, DBCtrls, mastform;

function TDGServer.AddDGRecord(Latitude, Longitude: Double): Integer;
begin
  Result := 0;
  if dmGlobal.dggeol.Active then
    begin
      fmDepGroup.buInsertClick(nil);
      dmGlobal.dggeol.Edit;
      dmGlobal.dggeolLATDD.AsFloat := Latitude;
      dmGlobal.dggeolLONGDD.AsFloat := Longitude;
      fmDepGroup.dbeDggeolLONGDDExit(nil);
      fmDepGroup.dbeDggeolLATDDExit(nil);
      dmGlobal.dggeol.Post;
      Result := dmGlobal.dggeolDGNO.AsInteger;
    end;
end;

function TDGServer.Get_CurrentMDBFileName: WideString;
begin
  Result := dmGlobal.GetFullDBPath;
end;

function TDGServer.Get_Datum: WideString;
begin
  if dmGlobal.dggeol.Active
    then Result := dmGlobal.dggeolDATUM.AsString
    else Result := '';
end;

function TDGServer.Get_DGNO: Integer;
begin
  if dmGlobal.dggeol.Active
    then Result := dmGlobal.dggeolDGNO.AsInteger
    else Result := 0;
end;

function TDGServer.Get_Latitude: Double;
begin
  if dmGlobal.dggeol.Active
    then Result := dmGlobal.dggeolLATDD.AsFloat
    else Result := 0.0;
end;

function TDGServer.Get_Longitude: Double;
begin
  if dmGlobal.dpgeol.Active
    then Result := dmGlobal.dggeolLONGDD.AsFloat
    else Result := 0.0;
end;

procedure TDGServer.DeleteDGDigitizedRecord(DGNO: Integer);
begin
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        if not dmGlobal.dgdigitized.Active then
          dmGlobal.dgdigitized.Open;
        if dmGlobal.dgdigitized.Locate('DGNO;RANK',VarArrayOf([DGNO,1]),[]) then
          dmGlobal.dgdigitized.Delete;
      end;
end;

procedure TDGServer.DeleteDGRecord(DGNO: Integer);
begin
  if dmGlobal.dggeol.Active then
    begin
      dmGlobal.dggeol.Locate('DGNO',DGNO,[]);
      fmDepGroup.buDeleteClick(nil);
    end;
end;

procedure TDGServer.GetDGCompilationProgress(DGNO: Integer; out Compstage,
  Proglevel: WideString);
begin
  Compstage := '';
  Proglevel := '';
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        if not dmGlobal.dgprogress.Active then
          dmGlobal.dgprogress.Open;
        if dmGlobal.dgprogress.Locate('DGNO;RANK',VarArrayOf([DGNO,1]),[])
          then
            begin
              Compstage := dmGlobal.dgprogressCOMPSTAGE.AsString;
              Proglevel := dmGlobal.dgprogressPROGLEVEL.AsString;
            end;
      end;
end;

procedure TDGServer.GetDGDigitizedRecord(DGNO: Integer; out X, Y: Double;
  out EPSG: Integer);
begin
  X := 0.0;
  Y := 0.0;
  EPSG := 0;
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        if not dmGlobal.dgdigitized.Active then
          dmGlobal.dgdigitized.Open;
        if dmGlobal.dgdigitized.Locate('DGNO;RANK',VarArrayOf([DGNO,1]),[])
          then
            begin
              X := dmGlobal.dgdigitizedDIGITIZED_X.AsFloat;
              Y := dmGlobal.dgdigitizedDIGITIZED_Y.AsFloat;
              EPSG := dmGlobal.dgdigitizedDIGITIZED_EPSG.AsInteger;
            end;
      end;
end;

procedure TDGServer.GetDGLatLong(DGNO: Integer; out Latitude,
  Longitude: Double);
begin
  Latitude := 0.0;
  Longitude := 0.0;
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[])
      then
        begin
          Latitude := dmGlobal.dggeolLATDD.AsFloat;
          Longitude := dmGlobal.dggeolLONGDD.AsFloat;
        end;
end;

procedure TDGServer.LocateDGRecord(DGNO: Integer);
begin
  if dmGlobal.dggeol.Active then
    begin
      dmGlobal.dggeol.Locate('DGNO',DGNO,[]);
      fmDepGroup.dnDggeolClick(nil,nbRefresh);
    end;
end;

procedure TDGServer.UpdateDGDatum(DGNO: Integer; const Datum: WideString);
begin
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        dmGlobal.dggeol.Edit;
        dmGlobal.dggeolDATUM.AsString := Datum;
        dmGlobal.dggeol.Post;
      end;
end;

procedure TDGServer.UpdateDGDigitizedRecord(DGNO: Integer; X, Y: Double;
  EPSG: Integer);
var
  RANK: longint;
begin
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        if not dmGlobal.dgdigitized.Active then
          dmGlobal.dgdigitized.Open;
        fmDepGroup.CreateSubrecordXternal(dmGlobal.dgdigitized,'dpdigitized',dmGlobal.dggeol,'DGNO');
        dmGlobal.dgdigitized.Last;
        RANK := dmGlobal.dgdigitizedRANK.AsInteger;
        dmGlobal.dgdigitized.Edit;
        dmGlobal.dgdigitizedDIGITIZED_X.AsFloat := X;
        dmGlobal.dgdigitizedDIGITIZED_Y.AsFloat := Y;
        dmGlobal.dgdigitizedDIGITIZED_EPSG.AsInteger := EPSG;
        dmGlobal.dgdigitized.Post;
        fmDepGroup.FillDigitizedData(DGNO,RANK,EPSG);
      end;
end;

procedure TDGServer.UpdateDGLatLong(DGNO: Integer; Latitude,
  Longitude: Double);
begin
  if dmGlobal.dggeol.Active then
    if dmGlobal.dggeol.Locate('DGNO',DGNO,[]) then
      begin
        dmGlobal.dggeol.Edit;
        dmGlobal.dggeolLATDD.AsFloat := Latitude;
        dmGlobal.dggeolLONGDD.AsFloat := Longitude;
        fmDepGroup.dbeDggeolLONGDDExit(nil);
        fmDepGroup.dbeDggeolLATDDExit(nil);
        dmGlobal.dggeol.Post;
      end;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TDGServer, Class_DGServer,
    ciMultiInstance, tmApartment);
end.
