unit autoserverDP_SA;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  ComObj, ActiveX, GShellADO_SA_TLB, StdVcl, Variants;

type
  TDPServer = class(TAutoObject, IDPServer)
  protected
    function AddDPRecord(Latitude, Longitude: Double): Integer; safecall;
    function Get_CurrentMDBFileName: WideString; safecall;
    function Get_Datum: WideString; safecall;
    function Get_DEPNO: Integer; safecall;
    function Get_Latitude: Double; safecall;
    function Get_Longitude: Double; safecall;
    procedure DeleteDPDigitizedRecord(DEPNO: Integer); safecall;
    procedure DeleteDPRecord(DEPNO: Integer); safecall;
    procedure GetDPCompilationProgress(DEPNO: Integer; out Compstage,
      Proglevel: WideString); safecall;
    procedure GetDPDigitizedRecord(DEPNO: Integer; out X, Y: Double;
      out EPSG: Integer); safecall;
    procedure GetDPLatLong(DEPNO: Integer; out Latitude, Longitude: Double);
      safecall;
    procedure LocateDPRecord(DEPNO: Integer); safecall;
    procedure UpdateDPDatum(DEPNO: Integer; const Datum: WideString); safecall;
    procedure UpdateDPDigitizedRecord(DEPNO: Integer; X, Y: Double;
      EPSG: Integer); safecall;
    procedure UpdateDPLatlong(DEPNO: Integer; Latitude, Longitude: Double);
      safecall;

  end;

implementation

uses ComServ, dmglob1, deposits, DBCtrls, mastform;

function TDPServer.AddDPRecord(Latitude, Longitude: Double): Integer;
begin
  Result := 0;
  if dmGlobal.dpgeol.Active then
    begin
      fmDeposit.buInsertClick(nil);
      dmGlobal.dpgeol.Edit;
      dmGlobal.dpgeolLATDD.AsFloat := Latitude;
      dmGlobal.dpgeolLONGDD.AsFloat := Longitude;
      fmDeposit.dbeDpgeolLONGDDExit(nil);
      fmDeposit.dbeDpgeolLATDDExit(nil);
      dmGlobal.dpgeol.Post;
      Result := dmGlobal.dpgeolDEPNO.AsInteger;
    end;
end;

function TDPServer.Get_CurrentMDBFileName: WideString;
begin
  Result := dmGlobal.GetFullDBPath;
end;

function TDPServer.Get_Datum: WideString;
begin
  if dmGlobal.dpgeol.Active
    then Result := dmGlobal.dpgeolDATUM.AsString
    else Result := '';
end;

function TDPServer.Get_DEPNO: Integer;
begin
  if dmGlobal.dpgeol.Active
    then Result := dmGlobal.dpgeolDEPNO.AsInteger
    else Result := 0;
end;

function TDPServer.Get_Latitude: Double;
begin
  if dmGlobal.dpgeol.Active
    then Result := dmGlobal.dpgeolLATDD.AsFloat
    else Result := 0.0;
end;

function TDPServer.Get_Longitude: Double;
begin
  if dmGlobal.dpgeol.Active
    then Result := dmGlobal.dpgeolLONGDD.AsFloat
    else Result := 0.0;
end;

procedure TDPServer.DeleteDPDigitizedRecord(DEPNO: Integer);
begin
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        if not dmGlobal.dpdigitized.Active then
          dmGlobal.dpdigitized.Open;
        if dmGlobal.dpdigitized.Locate('DEPNO;RANK',VarArrayOf([DEPNO,1]),[]) then
          dmGlobal.dpdigitized.Delete;
      end;
end;

procedure TDPServer.DeleteDPRecord(DEPNO: Integer);
begin
  if dmGlobal.dpgeol.Active then
    begin
      dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]);
      fmDeposit.buDeleteClick(nil);
    end;
end;

procedure TDPServer.GetDPCompilationProgress(DEPNO: Integer; out Compstage,
  Proglevel: WideString);
begin
  Compstage := '';
  Proglevel := '';
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        if not dmGlobal.dpprogress.Active then
          dmGlobal.dpprogress.Open;
        if dmGlobal.dpprogress.Locate('DEPNO;RANK',VarArrayOf([DEPNO,1]),[])
          then
            begin
              Compstage := dmGlobal.dpprogressCOMPSTAGE.AsString;
              Proglevel := dmGlobal.dpprogressPROGLEVEL.AsString;
            end;
      end;
end;

procedure TDPServer.GetDPDigitizedRecord(DEPNO: Integer; out X, Y: Double;
  out EPSG: Integer);
begin
  X := 0.0;
  Y := 0.0;
  EPSG := 0;
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        if not dmGlobal.dpdigitized.Active then
          dmGlobal.dpdigitized.Open;
        if dmGlobal.dpdigitized.Locate('DEPNO;RANK',VarArrayOf([DEPNO,1]),[])
          then
            begin
              X := dmGlobal.dpdigitizedDIGITIZED_X.AsFloat;
              Y := dmGlobal.dpdigitizedDIGITIZED_Y.AsFloat;
              EPSG := dmGlobal.dpdigitizedDIGITIZED_EPSG.AsInteger;
            end;
      end;
end;

procedure TDPServer.GetDPLatLong(DEPNO: Integer; out Latitude,
  Longitude: Double);
begin
  Latitude := 0.0;
  Longitude := 0.0;
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[])
      then
        begin
          Latitude := dmGlobal.dpgeolLATDD.AsFloat;
          Longitude := dmGlobal.dpgeolLONGDD.AsFloat;
        end;
end;

procedure TDPServer.LocateDPRecord(DEPNO: Integer);
begin
  if dmGlobal.dpgeol.Active then
    begin
      dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]);
      fmDeposit.dnDpgeolClick(nil,nbRefresh);
    end;
end;

procedure TDPServer.UpdateDPDatum(DEPNO: Integer; const Datum: WideString);
begin
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        dmGlobal.dpgeol.Edit;
        dmGlobal.dpgeolDATUM.AsString := Datum;
        dmGlobal.dpgeol.Post;
      end;
end;

procedure TDPServer.UpdateDPDigitizedRecord(DEPNO: Integer; X, Y: Double;
  EPSG: Integer);
var
  RANK: longint;
begin
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        if not dmGlobal.dpdigitized.Active then
          dmGlobal.dpdigitized.Open;
        fmDeposit.CreateSubrecordXternal(dmGlobal.dpdigitized,'dpdigitized',dmGlobal.dpgeol,'DEPNO');
        dmGlobal.dpdigitized.Last;
        RANK := dmGlobal.dpdigitizedRANK.AsInteger;
        dmGlobal.dpdigitized.Edit;
        dmGlobal.dpdigitizedDIGITIZED_X.AsFloat := X;
        dmGlobal.dpdigitizedDIGITIZED_Y.AsFloat := Y;
        dmGlobal.dpdigitizedDIGITIZED_EPSG.AsInteger := EPSG;
        dmGlobal.dpdigitized.Post;
        fmDeposit.FillDigitizedData(DEPNO,RANK,EPSG);
      end;
end;

procedure TDPServer.UpdateDPLatlong(DEPNO: Integer; Latitude,
  Longitude: Double);
begin
  if dmGlobal.dpgeol.Active then
    if dmGlobal.dpgeol.Locate('DEPNO',DEPNO,[]) then
      begin
        dmGlobal.dpgeol.Edit;
        dmGlobal.dpgeolLATDD.AsFloat := Latitude;
        dmGlobal.dpgeolLONGDD.AsFloat := Longitude;
        fmDeposit.dbeDpgeolLONGDDExit(nil);
        fmDeposit.dbeDpgeolLATDDExit(nil);
        dmGlobal.dpgeol.Post;
      end;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TDPServer, Class_DPServer,
    ciMultiInstance, tmApartment);
end.
