unit dmrefresh;

interface

uses
  SysUtils, Classes, DB, ADODB, Controls, Dialogs, Forms, StrUtils;

type
  TdmRefreshSummaryTables = class(TDataModule)
    Connection1: TADOConnection;
    Command1: TADOCommand;
    DataSet1: TADODataSet;
    DataSet2: TADODataSet;
    SummaryTable: TADODataSet;
  private
    e_key: string;
    e_table: string;
    function GetCommodities(entity: string;keyno: longint): string;
    function GetCountry(entity: string;keyno: longint): string;
    function GetLocation: string;
    function GetNames(entity: string;keyno: longint): string;
    procedure PopulateNTSTable(entity: string);
    procedure PopulateSummaryTable(entity: string);
  public
    TableList: TStringList;
    procedure PopulateRecordCountTable;
    procedure PopulateSummaryTables;
  end;

var
  dmRefreshSummaryTables: TdmRefreshSummaryTables;

implementation

uses dmglob1;

{$R *.dfm}

function TdmRefreshSummaryTables.GetLocation: string;
begin
  result := '';
  with DataSet1 do
    begin
      if not FieldByName('LATDEG').IsNull then
        result := Format('%2.2d',[FieldByName('LATDEG').AsInteger]);
      if not FieldByName('LATMIN').IsNull then
        result := result + '.' + Format('%2.2d',[FieldByName('LATMIN').AsInteger]);
      if not FieldByName('LATSEC').IsNull then
        result := result + '.' + Format('%2.2d',[FieldByName('LATSEC').AsInteger]);
      if not FieldByName('NS').IsNull then
        result := result + ' ' + FieldByName('NS').Value;
      if not FieldByName('LONGDEG').IsNull then
        result := result + ' -- ' + Format('%3.3d',[FieldByName('LONGDEG').AsInteger]);
      if not FieldByName('LONGMIN').IsNull then
        result := result + '.' + Format('%2.2d',[FieldByName('LONGMIN').AsInteger]);
      if not FieldByName('LONGSEC').IsNull then
        result := result + '.' + Format('%2.2d',[FieldByName('LONGSEC').AsInteger]);
      if not FieldByName('EW').IsNull then
        result := result + ' ' + FieldByName('EW').Value;
    end;
end;

function TdmRefreshSummaryTables.GetCommodities(entity: string;keyno: longint): string;
var
  NameTable : string;
  NameField : string;
begin
  if entity = 'DP'
    then
      begin
        NameTable := 'dpcom';
        NameField := 'COMMODITY';
      end
    else
      begin
        NameTable := 'dgcom';
        NameField := 'COMMODITY';
      end;
  result := '';
  with Dataset2 do
    begin
      Close;
      CommandText := 'select RANK,' + NameField + ' from ' + NameTable + ' ' +
                     'where ' + e_key + ' = ' + IntToStr(keyno) + ' ' +
                     'order by RANK';
      Open;
      if EOF then
        begin
          Close;
          Exit;
        end;
      if not FieldByName(NameField).IsNull then
        result := FieldByName(NameField).Value;
      Next;
      while not EOF do
        begin
          if not FieldByName(NameField).IsNull then
            result := result + '; ' + FieldByName(NameField).Value;
          Next;
        end;
      Close;
    end;
end;

function TdmRefreshSummaryTables.GetCountry(entity: string;keyno: longint): string;
var
  ctry: string;
//  nts: string;
  dtable: string;
begin
  if entity = 'DP'
    then dtable := 'dpctry'
    else dtable := 'dgctry';
  result := '';
  with Dataset2 do
    begin
      Close;
      CommandText := 'select RANK,COUNTRY,PROVSTATE from ' + dtable +
                     ' where ' + e_key + ' = ' + IntToStr(keyno) +
                     ' order by RANK';
      Open;
      if not EOF then
        begin
          if not FieldByName('COUNTRY').IsNull then
            ctry := FieldByName('COUNTRY').Value;
          if not FieldByName('PROVSTATE').IsNull then
            if ctry <> ''
              then ctry := ctry + ' (' + FieldByName('PROVSTATE').AsString + ')'
              else ctry := FieldByName('PROVSTATE').AsString;
          Next;
          while not EOF do
            begin
              if not FieldByName('COUNTRY').IsNull then
                ctry := ctry + ' / ' + FieldByName('COUNTRY').Value;
              if not FieldByName('PROVSTATE').IsNull then
                if ctry <> ''
                  then ctry := ctry + ' (' + FieldByName('PROVSTATE').AsString + ')'
                  else ctry := FieldByName('PROVSTATE').AsString;
              Next;
            end;
        end;
      Close;
    end;
  result := ctry;
end;

function TdmRefreshSummaryTables.GetNames(entity: string;keyno: longint): string;
var
  NameTable : string;
  NameField : string;
begin
  if entity = 'DP'
    then
      begin
        NameTable := 'dpname';
        NameField := 'DEPNAME';
      end
    else
      begin
        NameTable := 'dgname';
        NameField := 'DGNAME';
      end;
  result := '';
  with Dataset2 do
    begin
      Close;
      CommandText := 'select RANK,' + NameField + ' from ' + NameTable + ' ' +
                     'where ' + e_key + ' = ' + IntToStr(keyno) + ' ' +
                     'order by RANK';
      Open;
      if EOF then
        begin
          Close;
          Exit;
        end;
      if not FieldByName(NameField).IsNull then
        result := FieldByName(NameField).Value;
      Next;
      while not EOF do
        begin
          if not FieldByName(NameField).IsNull then
            result := result + '; ' + FieldByName(NameField).Value;
          Next;
        end;
      Close;
    end;
end;

procedure TdmRefreshSummaryTables.PopulateNTSTable(entity: string);
var
  NTSlist: TStringList;
  qrylocation: TADOQuery;
  qrynts: TADOQuery;
  ntstable: TADOTable;
  s: string;
  i: integer;
  nts: string;
  latdd: double;
  longdd: double;
  band: char;
begin
  NTSlist := TStringList.Create;
  qrylocation := TADOQuery.Create(nil);
  qrynts := TADOQuery.Create(nil);
  try
    if entity = 'DG'
      then ntstable := dmGlobal.dgnts
      else ntstable := dmGlobal.dpnts;
    qrylocation.Connection := Connection1;
    qrynts.Connection := Connection1;
    qrylocation.SQL.Text := Format('select %s,latdd,longdd,country from %s_summary ' +
                                   'where (not ((latdd is null) or (longdd is null))) and ' +
                                   '(%s not in (select distinct %s from %snts))',
                                   [e_key,AnsiLowerCase(entity),e_key,e_key,AnsiLowerCase(entity)]);
    qrylocation.Open;
    while not qrylocation.Eof do
      begin
        if AnsiContainsStr(AnsiUpperCase(qrylocation.FieldByName('country').AsString),'CANADA') then
          begin
            latdd := qrylocation.Fields[1].AsFloat;
            if latdd < 68.0 then
              band := 'S'
            else if latdd < 80.0 then
              band := 'M'
            else band := 'N';
            longdd := Abs(qrylocation.Fields[2].AsFloat);
            nts := dmGlobal.CalculateNTS(latdd,longdd);
            if nts <> '' then
              s := nts;
            case band of
              'S' : if ((Frac(latdd*4.0) = 0.0) and
                        (Frac(longdd*2.0) = 0.0)) then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd) +
                               ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1) +
                               ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd-0.1)
                    else if Frac(latdd*4.0) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd)
                    else if Frac(longdd*2.0) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1);
              'M' : if ((Frac(latdd*4.0) = 0.0) and
                        (Frac(longdd) = 0.0)) then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd) +
                               ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1) +
                               ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd-0.1)
                    else if Frac(latdd*4.0) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd)
                    else if Frac(longdd) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1);
              'N' : if ((Frac(latdd*4.0) = 0.0) and
                        (Frac(longdd*0.5) = 0.0)) then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd) +
                               ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1) +
                               ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd-0.1)
                    else if Frac(latdd*4.0) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd-0.1,longdd)
                    else if Frac(longdd*0.5) = 0.0 then
                      s := s + ',' + dmGlobal.CalculateNTS(latdd,longdd-0.1);
              else ;
            end;
            if s <> '' then
              begin
                NTSlist.CommaText := s;
                ntstable.Open;
                for i := 1 to NTSlist.Count do
                  begin
                    qrynts.SQL.Text := 'select MAPID,NTSP,NTSS,NTST,QUADRANT,NTSNAME from ntsmaps ' +
                                       'where MAPID = ''' + NTSlist.Strings[i-1] + '''';
                    qrynts.Open;
                    if not qrynts.Eof then
                      with ntstable do
                        begin
                          Append;
                          FieldByName(e_key).AsInteger := qrylocation.FieldByName(e_key).AsInteger;
                          FieldByName('RANK').AsInteger := i;
                          FieldByName('MAPID').AsString := qrynts.FieldByName('MAPID').AsString;
                          FieldByName('NTSP').AsString := qrynts.FieldByName('NTSP').AsString;
                          FieldByName('NTSS').AsString := qrynts.FieldByName('NTSS').AsString;
                          FieldByName('NTST').AsString := qrynts.FieldByName('NTST').AsString;
                          FieldByName('QUADRANT').AsString := qrynts.FieldByName('QUADRANT').AsString;
                          FieldByName('NTSNAME').AsString := qrynts.FieldByName('NTSNAME').AsString;
                          Post;
                        end;
                    qrynts.Close;
                  end;
                ntstable.Close;
              end;
          end;
        qrylocation.Next;
      end;
    qrylocation.Close;
  finally
    NTSlist.Free;
    qrylocation.Close;
    qrylocation.Free;
    qrynts.Close;
    qrynts.Free;
  end;
end;

procedure TdmRefreshSummaryTables.PopulateRecordCountTable;
var
  k: integer;
  tmpCursor: TCursor;
  tn: string;
begin
//  ShowMessage('creating and populating gq_recordcount ...');
  with Connection1 do
    begin
      Open;
      tmpCursor := Screen.Cursor;
      Screen.Cursor := crSQLWait;
      if TableList.IndexOf('gq_recordcount') > -1 then
        begin
          Command1.CommandText := 'drop table gq_recordcount;';
          Command1.Execute;
        end;
      Command1.CommandText := 'create table gq_recordcount (' +
                              '  TABLENAME varchar(16) not null,' +
                              '  RECORDCOUNT integer,' +
                              '  constraint PK_GQ_RECORDCOUNT primary key (TABLENAME)' +
                              ');';
      Command1.Execute;
      Dataset1.CommandType := cmdTable;
      Dataset2.CommandType := cmdText;
      Dataset1.CommandText := 'tabledoc';
      Dataset1.Open;
      while not Dataset1.Eof do
        begin
          tn := Dataset1.FieldByName('TABLENAME').AsString;
          Dataset2.CommandText := 'select count(*) as cnt from ' + tn;
          Dataset2.Open;
          k := Dataset2.FieldByName('cnt').AsInteger;
          Dataset2.Close;
          Command1.CommandText := 'insert into gq_recordcount (tablename,recordcount) ' +
                                  'values (''' + tn + ''',' + IntToStr(k) + ');';
          Command1.Execute;
          Dataset1.Next;
        end;
      Dataset1.Close;
      Screen.Cursor := tmpCursor;
      Close;
    end;
end;

procedure TdmRefreshSummaryTables.PopulateSummaryTable(entity: string);
// populate summary table
var
  tmpCursor: TCursor;
  e_country: string;
  dbname: string;
  compby: string;
  reldate: string;
begin
  tmpCursor := Screen.Cursor;
  Screen.Cursor := crSQLWait;

  // get keyno and location from main table
  e_country := AnsiLowerCase(entity) + 'ctry';
  Dataset1.CommandType := cmdText;
  with Dataset1 do
    begin
      Close;
      CommandText := 'select dbname,compby,reldate from dbversion';
      Open;
      if FieldByName('DBNAME').IsNull
        then dbname := ''
        else dbname := FieldByName('DBNAME').AsString;
      if FieldByName('COMPBY').IsNull
        then compby := ''
        else compby := FieldByName('COMPBY').AsString;
      if FieldByName('RELDATE').IsNull
        then reldate := ''
        else
          begin
            reldate := FieldByName('RELDATE').AsString;
            reldate := Format('%s-%s-%s',[Copy(reldate,1,4),Copy(reldate,5,2),Copy(reldate,7,2)]);
          end;
      Close;

      CommandText := 'select ' + e_table + '.' + e_key +
                     ',LATDEG,LATMIN,LATSEC,NS,LONGDEG,LONGMIN,LONGSEC,' +
                     'EW,LONGDD,LATDD,DEPCLAN,COUNTRY ' +
                     'from ' + e_table + ' left outer join ' + e_country + ' ' +
                     'on ' + e_country + '.' + e_key + ' = ' + e_table + '.' + e_key + ' ' +
                     'where (' + e_country + '.RANK is null) or (' + e_country + '.RANK = 1)';
      Open;
    end;
  // get COUNTRY, reformat LOCATION and get NAMES and COMMODITIES
  SummaryTable.CommandText := AnsiLowerCase(entity) + '_summary';
  SummaryTable.CommandType := cmdTable;
  SummaryTable.Open;
  while not Dataset1.EOF do
    begin
      with SummaryTable do
        begin
          Append;
          FieldByName(e_key).Value := Dataset1.FieldByName(e_key).Value;
          FieldByName('COUNTRY').Value := GetCountry(entity,Dataset1.FieldByName(e_key).Value);
          FieldByName('LOCATION').Value := GetLocation;
          FieldByName('ALLNAMES').Value := GetNames(entity,Dataset1.FieldByName(e_key).Value);
          FieldByName('COMMODITIES').Value := GetCommodities(entity,Dataset1.FieldByName(e_key).Value);
          FieldByName('DEPCLAN').Value := Dataset1.FieldByName('DEPCLAN').Value;
          FieldByName('LONGDD').Value := Dataset1.FieldByName('LONGDD').Value;
          FieldByName('LATDD').Value := Dataset1.FieldByName('LATDD').Value;
          FieldByName('DBNAME').Value := dbname;
          FieldByName('COMPBY').Value := compby;
          FieldByName('RELDATE').Value := reldate;
          Post;
        end;
      Dataset1.Next;
    end;
  SummaryTable.Close;
  Dataset1.Close;
  Screen.Cursor := tmpCursor;
end;

procedure TdmRefreshSummaryTables.PopulateSummaryTables;
begin
//  ShowMessage('creating and populating dg_summary ...');
  if TableList.IndexOf('dg_summary') > -1 then
    begin
      Command1.CommandText := 'drop table dg_summary;';
      Command1.Execute;
    end;
  Command1.CommandText := 'create table dg_summary(' +
                          '  DGNO integer not null,' +
                          '  COUNTRY varchar(35),' +
                          '  LOCATION varchar(30),' +
                          '  ALLNAMES varchar(200),' +
                          '  COMMODITIES varchar(100),' +
                          '  DEPCLAN varchar(50),' +
                          '  LONGDD float,' +
                          '  LATDD float,' +
                          '  DBNAME varchar(30),' +
                          '  COMPBY varchar(200),' +
                          '  RELDATE varchar(10),' +
                          '  constraint PK_DG_SUMMARY primary key (DGNO))';
  Command1.Execute;
  e_key := 'DGNO';
  e_table := 'dggeol';
  PopulateSummaryTable('DG');
  PopulateNTSTable('DG');

//  ShowMessage('creating and populating dp_summary ...');
  if TableList.IndexOf('dp_summary') > -1 then
    begin
      Command1.CommandText := 'drop table dp_summary;';
      Command1.Execute;
    end;
  Command1.CommandText := 'create table dp_summary(' +
                          '  DEPNO integer not null,' +
                          '  COUNTRY varchar(35),' +
                          '  LOCATION varchar(30),' +
                          '  ALLNAMES varchar(200),' +
                          '  COMMODITIES varchar(100),' +
                          '  DEPCLAN varchar(50),' +
                          '  LONGDD float,' +
                          '  LATDD float,' +
                          '  DBNAME varchar(30),' +
                          '  COMPBY varchar(200),' +
                          '  RELDATE varchar(10),' +
                          '  constraint PK_DP_SUMMARY primary key (DEPNO))';
  Command1.Execute;
  e_key := 'DEPNO';
  e_table := 'dpgeol';
  PopulateSummaryTable('DP');
  PopulateNTSTable('DP');
end;

end.
