unit mastform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, Buttons, ADOdb;

type
  TfmMaster = class(TForm)
    MainMenu1: TMainMenu;
    miFile: TMenuItem;
    miExit: TMenuItem;
    miEdit: TMenuItem;
    miHelp: TMenuItem;
    N1: TMenuItem;
    miAbout: TMenuItem;
    miDepositGroup: TMenuItem;
    miDeposit: TMenuItem;
    miProduction: TMenuItem;
    miResource: TMenuItem;
    miReferences: TMenuItem;
    N2: TMenuItem;
    miLookupTables: TMenuItem;
    miMine: TMenuItem;
    paTop: TPanel;
    paLeft: TPanel;
    paRight: TPanel;
    paCenter: TPanel;
    buExit: TButton;
    sbDepositGroup: TSpeedButton;
    sbDeposit: TSpeedButton;
    sbMine: TSpeedButton;
    sbProduction: TSpeedButton;
    sbResource: TSpeedButton;
    sbReferences: TSpeedButton;
    sbLookupTables: TSpeedButton;
    miSelectDatabase: TMenuItem;
    laApplicationName: TLabel;
    N3: TMenuItem;
    miCustomizeTabPages: TMenuItem;
    miHints: TMenuItem;
    laDbversionDBNAME: TLabel;
    meDbversionDBTEXT: TMemo;
    miTools: TMenuItem;
    miRefreshSummaryTables: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miLookupTablesClick(Sender: TObject);
    procedure miReferencesClick(Sender: TObject);
    procedure buExitClick(Sender: TObject);
    procedure miDepositClick(Sender: TObject);
    procedure miSelectDatabaseClick(Sender: TObject);
    procedure miCustomizeTabPagesClick(Sender: TObject);
    procedure miDepositGroupClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miMineClick(Sender: TObject);
    procedure miProductionClick(Sender: TObject);
    procedure miResourceClick(Sender: TObject);
    procedure miHintsClick(Sender: TObject);
    procedure miRefreshSummaryTablesClick(Sender: TObject);
  private
    { Private declarations }
    procedure ActivateMenuAndButtons(TrueFalse : boolean);
    procedure DisableEmptyControls;
    procedure SaveHintsToFile;
    procedure SetReadOnly;
    procedure VerifyCustomizationFile;
  public
    GShellVersion: string;
  end;

var
  fmMaster     : TfmMaster;
  UserInitials : string;
  BrowseEdit   : boolean;
  DatabaseUDL  : string;
  DpStd        : TStringList;
  DpTSNames    : TStringList;
  DgTSNames    : TStringList;
  DgStd        : TStringList;
  DpSpecs      : TStringList;
  DgSpecs      : TStringList;

implementation

uses referenc, deposits, seldb, lookups, dmglob1, customiz, dpgroups, mines,
  production, resource, startinf, seljourn, selnews, selsvol, selbysql,
  selref, selnts, dmrefresh, dpqueries, resources;

{$R *.DFM}

procedure TfmMaster.ActivateMenuAndButtons(TrueFalse : boolean);
begin
  with fmMaster do
    begin
      miDepositGroup.Enabled := TrueFalse;
      miDeposit.Enabled := TrueFalse;
      miMine.Enabled := TrueFalse;
      miProduction.Enabled := TrueFalse;
      miResource.Enabled := TrueFalse;
      miReferences.Enabled := TrueFalse;
      miLookupTables.Enabled := TrueFalse;
      sbDepositGroup.Enabled := TrueFalse;
      sbDeposit.Enabled := TrueFalse;
      sbMine.Enabled := TrueFalse;
      sbProduction.Enabled := TrueFalse;
      sbResource.Enabled := TrueFalse;
      sbReferences.Enabled := TrueFalse;
      sbLookupTables.Enabled := TrueFalse;
      miRefreshSummaryTables.Enabled := TrueFalse;
    end;
end;

procedure TfmMaster.VerifyCustomizationFile;
var
  i: integer;
  ok: boolean;
begin
  ok := True;
  i := 0;
  while i < DpSpecs.Count do
    begin
      if DpStd.IndexOf(DpSpecs.Strings[i]) < 0
        then ok := False;
      inc(i);
    end;
  i := 0;
  while i < DgSpecs.Count do
    begin
      if DgStd.IndexOf(DgSpecs.Strings[i]) < 0
        then ok := False;
      inc(i);
    end;
  if not ok then
    MessageDlg(mastform_RunCustomizeTabs,mtWarning,[mbOK],0)
end;

procedure TfmMaster.DisableEmptyControls;
begin
  with dmGlobal do
    begin
      dggeol.Open;
      if dggeol.Eof then
        begin
          miDepositGroup.Enabled := False;
          sbDepositGroup.Enabled := False;
        end;
      dggeol.Close;
      dpgeol.Open;
      if dpgeol.Eof then
        begin
          miDeposit.Enabled := False;
          sbDeposit.Enabled := False;
        end;
      dpgeol.Close;
      mine.Open;
      if mine.Eof then
        begin
          miMine.Enabled := False;
          sbMine.Enabled := False;
        end;
      mine.Close;
      prodmain.Open;
      if prodmain.Eof then
        begin
          miProduction.Enabled := False;
          sbProduction.Enabled := False;
        end;
      prodmain.Close;
      resmain.Open;
      if resmain.Eof then
        begin
          miResource.Enabled := False;
          sbResource.Enabled := False;
        end;
      resmain.Close;
      refmain.Open;
      if refmain.Eof then
        begin
          miReferences.Enabled := False;
          sbReferences.Enabled := False;
        end;
      refmain.Close;
    end;
end;

procedure TfmMaster.SaveHintsToFile;
var
  SaveCursor : TCursor;

  procedure ProcessForm(fm : TForm; fc : string);
  var
    i : integer;
    c : TControl;
    h : TStringList;
  begin
    h := TStringList.Create;
    try
      h.Sorted := True;
      with fm do
        for i := 0 to ComponentCount-1 do
          if (Components[i] is TControl) and not (Components[i] is TLabel) then
            begin
              c := TControl(Components[i]);
              h.Add(fc + chr(9) + c.Name + chr(9) + c.Hint);
            end;
      h.SaveToFile('c:\temp\' + fc + 'GShellHints.txt');
    finally
      h.Free
    end;
  end;

begin
  SaveCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;

  fmDepGroup := TfmDepGroup.Create(nil);
  try
    ProcessForm(fmDepGroup,'dg');
  finally
    fmDepGroup.Release;
  end;

  fmDeposit := TfmDeposit.Create(nil);
  try
    ProcessForm(fmDeposit,'dp');
  finally
    fmDeposit.Release;
  end;

  fmMine := TfmMine.Create(nil);
  try
    ProcessForm(fmMine,'mn');
  finally
    fmMine.Release;
  end;

  fmProduction := TfmProduction.Create(nil);
  try
    ProcessForm(fmProduction,'pr');
  finally
    fmProduction.Release;
  end;

  fmResource := TfmResource.Create(nil);
  try
    ProcessForm(fmResource,'rs');
  finally
    fmResource.Release;
  end;

  fmReferences := TfmReferences.Create(nil);
  try
    ProcessForm(fmReferences,'rf');
  finally
    fmReferences.Release;
  end;

  fmLookupTables := TfmLookupTables.Create(nil);
  try
    ProcessForm(fmLookupTables,'lk');
  finally
    fmLookupTables.Release;
  end;

  ShowMessage('Hints files saved');
  Screen.Cursor := SaveCursor;
end;

procedure TfmMaster.SetReadOnly;
var
  i : word;
  t : TADOTable;
begin
  for i := 0 to dmGlobal.ComponentCount-1 do
    if dmGlobal.Components[i] is TADOTable then
      begin
        t := TADOTable(dmGlobal.Components[i]);
        t.ReadOnly := True;
      end;
end;

procedure TfmMaster.FormCreate(Sender: TObject);
var
  d     : string;
  sts   : TStringList; //TStrings;
  i     : integer;
begin
  if UserInitials <> 'ZZZ' then
    begin
      fmStartInfo.Label1.Caption := mastform_BuildingDM;
      fmStartInfo.pbInfo.Position := 0;
      fmStartInfo.Show;
      fmStartInfo.Update;
      dmGlobal := TdmGlobal.Create(nil);
      if not BrowseEdit then
        SetReadOnly;
      fmStartInfo.Label1.Caption := mastform_PreparingMainForm;
      fmStartInfo.Show;
      fmStartInfo.Update;
      UserInitials := AnsiUpperCase(UserInitials);
    // extra code for customization of tabsheet visibility
      DpStd := TStringList.Create;
      DpStd.CommaText := mastform_DpStd;
{
      DpStd.Add('Names');
      DpStd.Add('Country/Province');
      DpStd.Add('Commodities');
      DpStd.Add('Mineralogy');
      DpStd.Add('Country Rocks');
      DpStd.Add('Host Rocks');
      DpStd.Add('Geological Age');
      DpStd.Add('References');
      DpStd.Add('Metallic Signature');
      DpStd.Add('Mineralization Style');
      DpStd.Add('Coincident Feature');
      DpStd.Add('Alteration Signature');
      DpStd.Add('Related Igneous Rocks');
      DpStd.Add('Tectonic Structure');
      DpStd.Add('Radiometric Age');
      DpStd.Add('Deposit Type');
      DpStd.Add('Deposit Status');
      DpStd.Add('Tectonic Setting');
      DpStd.Add('General Shape');
      DpStd.Add('Comments');
      DpStd.Add('Links to Other Databases');
      DpStd.Add('NTS Map Data');
      DpStd.Add('Digitization Data');
      DpStd.Add('Geophysical-Chemical Signature');
      DpStd.Add('Compilation Stage and Progress');
      DpStd.Add('Sample Data');
}
      DpTSNames := TStringList.Create;
      DpTSNames.Add('tsDpname');
      DpTSNames.Add('tsDpctry');
      DpTSNames.Add('tsDpcom');
      DpTSNames.Add('tsDpmnrlgy');
      DpTSNames.Add('tsDpctryrx');
      DpTSNames.Add('tsDphostrx');
      DpTSNames.Add('tsDpagets');
      DpTSNames.Add('tsDpref');
      DpTSNames.Add('tsDpmetsig');
      DpTSNames.Add('tsDpminstl');
      DpTSNames.Add('tsDpcfeat');
      DpTSNames.Add('tsDpaltsig');
      DpTSNames.Add('tsDpignrx');
      DpTSNames.Add('tsDptstruc');
      DpTSNames.Add('tsDpradage');
      DpTSNames.Add('tsDpdeptyp');
      DpTSNames.Add('tsDpstatus');
      DpTSNames.Add('tsDptecset');
      DpTSNames.Add('tsDpshape');
      DpTSNames.Add('tsDpcomment');
      DpTSNames.Add('tsDpxlink');
      DpTSNames.Add('tsDpnts');
      DpTSNames.Add('tsDpdigitized');
      DpTSNames.Add('tsDpgeosig');
      DpTSNames.Add('tsDpprogress');
      DpTSNames.Add('tsDpsample');
      DgStd := TStringList.Create;
      DgStd.CommaText := mastform_DgStd;
{
      DgStd.Add('Names');
      DgStd.Add('Country/Province');
      DgStd.Add('Commodities');
      DgStd.Add('Mineralogy');
      DgStd.Add('Country Rocks');
      DgStd.Add('Host Rocks');
      DgStd.Add('Geological Age');
      DgStd.Add('References');
      DgStd.Add('Metallic Signature');
      DgStd.Add('Mineralization Style');
      DgStd.Add('Coincident Feature');
      DgStd.Add('Alteration Signature');
      DgStd.Add('Related Igneous Rocks');
      DgStd.Add('Tectonic Structure');
      DgStd.Add('Host Names');
      DgStd.Add('Radiometric Age');
      DgStd.Add('Deposit Type');
      DgStd.Add('Deposit Group Status');
      DgStd.Add('Tectonic Setting');
      DgStd.Add('General Shape');
      DgStd.Add('Comments');
      DgStd.Add('Links to Other Databases');
      DgStd.Add('NTS Map Data');
      DgStd.Add('Digitization Data');
      DgStd.Add('Geophysical-Chemical Signature');
      DgStd.Add('Compilation Stage and Progress');
      DgStd.Add('Sample Data');
 }
      DgTSNames := TStringList.Create;
      DgTSNames.Add('tsDgname');
      DgTSNames.Add('tsDgctry');
      DgTSNames.Add('tsDgcom');
      DgTSNames.Add('tsDgmnrlgy');
      DgTSNames.Add('tsDgctryrx');
      DgTSNames.Add('tsDghostrx');
      DgTSNames.Add('tsDgagets');
      DgTSNames.Add('tsDgref');
      DgTSNames.Add('tsDgmetsig');
      DgTSNames.Add('tsDgminstl');
      DgTSNames.Add('tsDgcfeat');
      DgTSNames.Add('tsDgaltsig');
      DgTSNames.Add('tsDgignrx');
      DgTSNames.Add('tsDgtstruc');
      DgTSNames.Add('tsDghostnm');
      DgTSNames.Add('tsDgradage');
      DgTSNames.Add('tsDgdeptyp');
      DgTSNames.Add('tsDgstatus');
      DgTSNames.Add('tsDgtecset');
      DgTSNames.Add('tsDgshape');
      DgTSNames.Add('tsDgcomment');
      DgTSNames.Add('tsDgxlink');
      DgTSNames.Add('tsDgnts');
      DgTSNames.Add('tsDgdigitized');
      DgTSNames.Add('tsDggeosig');
      DgTSNames.Add('tsDgprogress');
      DgTSNames.Add('tsDgsample');
      GetDir(0,d);
    // check for existence of customization file, create if missing.
      if not FileExists(d + '\GShell.' + UserInitials) then
        begin
          sts := TStringList.Create;
          try
            sts.Add('[Deposits Module]');
            for i := 0 to DpStd.Count-1 do
              sts.Add(DpStd.Strings[i]);
            sts.Add('[Deposit Groups Module]');
            for i := 0 to DgStd.Count-1 do
              sts.Add(DgStd.Strings[i]);
            sts.SaveToFile(d + '\GShell.' + AnsiLowerCase(UserInitials));
          finally
            sts.Free;
          end;
        end;
      DpSpecs := TStringList.Create;
      DpSpecs.LoadFromFile(d + '\GShell.' + AnsiLowerCase(UserInitials));
      i := 0;
      repeat
        inc(i)
      until DpSpecs.Strings[i] = '[Deposit Groups Module]';
      repeat
        DpSpecs.Delete(i)
      until DpSpecs.Count = i;
      DpSpecs.Delete(0);
      DgSpecs := TStringList.Create;
      DgSpecs.LoadFromFile(d + '\GShell.' + AnsiLowerCase(UserInitials));
      while not (DgSpecs.Strings[0] = '[Deposit Groups Module]') do
        DgSpecs.Delete(0);
      DgSpecs.Delete(0);
      VerifyCustomizationFile;
    // end of extra code for customization
    end;
// Center form on screen, fill screen if 800x600
{
  fmMaster.Top := Trunc ((Screen.Height - 28 - 572) / 2);
  fmMaster.Left := Trunc ((Screen.Width - 800) / 2);
}
  Self.Caption := mastform_fmMaster;
  miFile.Caption := mastform_miFile;
  miSelectDatabase.Caption := mastform_miSelectDatabase;
  miExit.Caption := mastform_miExit;
  miEdit.Caption := mastform_miEditEdit;
  miDepositGroup.Caption :=  mastform_miDepositGroup;
  miDeposit.Caption := mastform_miDeposit;
  miMine.Caption := mastform_miMine;
  miProduction.Caption := mastform_miProduction;
  miResource.Caption := mastform_miResource;
  miReferences.Caption := mastform_miReferences;
  miLookupTables.Caption := mastform_miLookupTables;
  miCustomizeTabPages.Caption := mastform_miCustomizeTabPages;
  miTools.Caption := mastform_miTools;
  miRefreshSummarytables.Caption := mastform_miRefreshSummaryTables;
  miHelp.Caption := mastform_miHelp;
  miHints.Caption := mastform_miHints;
  miAbout.Caption := mastform_miAbout;
  sbDepositGroup.Caption := mastform_sbDepositGroup;
  sbDeposit.Caption := mastform_sbDeposit;
  sbMine.Caption := mastform_sbMine;
  sbProduction.Caption := mastform_sbProduction;
  sbResource.Caption := mastform_sbResource;
  sbReferences.Caption := mastform_sbReferences;
  sbLookupTables.Caption := mastform_sbLookupTables;
  buExit.Caption := mastform_buExit;

  fmMaster.Top := 0;
  fmMaster.Left := 0;
  fmMaster.Height := 572;
  fmMaster.Width := 800;
  ActivateMenuAndButtons(False);
  if UserInitials = 'ZZZ' then
    begin
      miFile.Enabled := False;
      miEdit.Enabled := False;
      miHelp.Enabled := False;
      sbDeposit.Enabled := False;
      sbDepositGroup.Enabled := False;
      sbReferences.Enabled := False;
      sbLookupTables.Enabled := False;
      sbProduction.Enabled := False;
      sbResource.Enabled := False;
      ShowMessage(mastform_GShellCANCEL);
      Application.Terminate;
    end;
end;

procedure TfmMaster.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMaster.miLookupTablesClick(Sender: TObject);
begin
  miLookupTables.Enabled := False;
  sbLookupTables.Enabled := False;
  fmLookupTables := TfmLookupTables.Create(nil);
  try
    fmLookupTables.Top := fmMaster.Top;
    fmLookupTables.Left := fmMaster.Left;
    fmLookupTables.Height := fmMaster.Height;
    fmLookupTables.Width := fmMaster.Width;
    fmLookupTables.ShowModal;
  finally
    fmLookupTables.Release;
  end;
end;

procedure TfmMaster.miReferencesClick(Sender: TObject);
begin
  miReferences.Enabled := False;
  sbReferences.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmReferences := TfmReferences.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  fmSelectJournal := TfmSelectJournal.Create(nil);
  fmSelectNewsname := TfmSelectNewsname.Create(nil);
  fmSelectSvol := TfmSelectSvol.Create(nil);
  try
    fmReferences.Top := fmMaster.Top;
    fmReferences.Left := fmMaster.Left;
    fmReferences.Height := fmMaster.Height;
    fmReferences.Width := fmMaster.Width;
    fmReferences.ShowModal;
  finally
    Application.OnHint := nil;
    fmReferences.Release;
    fmSelectRef.Release;
    fmSelectJournal.Release;
    fmSelectNewsname.Release;
    fmSelectSvol.Release;
  end;
end;

procedure TfmMaster.buExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMaster.miDepositClick(Sender: TObject);
begin
  miDeposit.Enabled := False;
  sbDeposit.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmDeposit := TfmDeposit.Create(nil);
  fmSelectBySQL := TfmSelectBySQL.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  fmSelNTS := TfmSelNTS.Create(nil);
  fmDPQueries := TfmDPQueries.Create(nil);
  fmDeposit.buQuery.Enabled := fmDPQueries.ComboBox1.Items.Count > 0;
  try
    fmDeposit.Top := fmMaster.Top;
    fmDeposit.Left := fmMaster.Left;
    fmDeposit.Height := fmMaster.Height;
    fmDeposit.Width := fmMaster.Width;
    fmDeposit.ShowModal;
  finally
    Application.OnHint := nil;
    fmDeposit.Release;
    fmSelectBySQL.Release;
    fmSelectRef.Release;
    fmSelNTS.Release;
    if fmDPQueries.Active then
      fmDPQueries.Close;
    fmDPQueries.Release;
  end;
end;

procedure TfmMaster.miSelectDatabaseClick(Sender: TObject);
var
  sd : TfmSelectDB;
begin
  sd := TfmSelectDB.Create(nil);
  try
    sd.ShowModal;
  finally
    sd.Release;
  end;
  if dmGlobal.DataBase1.ConnectionString <> '' then
    begin
      ActivateMenuAndButtons(True);
      with dmGlobal.dbversion do
        begin
          Open;
          if not EOF
            then
              begin
                laDbversionDBNAME.Caption := FieldByName('DBNAME').Value;
                meDbversionDBTEXT.Lines.Clear;
                if not FieldByName('DBREL').IsNull then
                  meDbversionDBTEXT.Lines.Add(FieldByName('DBREL').Value);
                meDbversionDBTEXT.Lines.Add('');
                if not FieldByName('COMPBY').IsNull then
                  meDbversionDBTEXT.Lines.Add(FieldByName('COMPBY').Value);
                meDbversionDBTEXT.Lines.Add('');
                if not FieldByName('DBAGENCY').IsNull then
                  meDbversionDBTEXT.Lines.Add(FieldByName('DBAGENCY').Value);
              end
            else
              begin
                laDbversionDBNAME.Caption := '';
                meDbversionDBTEXT.Lines.Clear;
              end;
          Close;
        end;
      fmMaster.Update;
      if not BrowseEdit  // Browse mode only
        then
          begin
            DisableEmptyControls;
            miRefreshSummaryTables.Enabled := False;
          end
        else miRefreshSummaryTables.Enabled := True;
    end;
end;

procedure TfmMaster.miCustomizeTabPagesClick(Sender: TObject);
var
  cs : TfmCustomize;
begin
  cs := TfmCustomize.Create(nil);
  try
    cs.ShowModal;
  finally
    cs.Release;
  end;
end;

procedure TfmMaster.miDepositGroupClick(Sender: TObject);
begin
  miDepositGroup.Enabled := False;
  sbDepositGroup.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmDepGroup := TfmDepGroup.Create(nil);
  fmSelectBySQL := TfmSelectBySQL.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  fmSelNTS := TfmSelNTS.Create(nil);
  try
    fmDepGroup.Top := fmMaster.Top;
    fmDepGroup.Left := fmMaster.Left;
    fmDepGroup.Height := fmMaster.Height;
    fmDepGroup.Width := fmMaster.Width;
    fmDepGroup.ShowModal;
  finally
    Application.OnHint := nil;
    fmDepGroup.Release;
    fmSelectBySQL.Release;
    fmSelectRef.Release;
    fmSelNTS.Release;
  end;
end;

procedure TfmMaster.FormShow(Sender: TObject);
begin
  if GShellVersion = 'GShellADO'
    then if MessageDlg(mastform_BrowseModeOnly,mtConfirmation,[mbYes,mbNo],0) = mrYes
           then BrowseEdit := False
           else BrowseEdit := True
    else BrowseEdit := False;

  if BrowseEdit
    then miEdit.Caption := mastform_miEditEdit
    else
      begin
        miEdit.Caption := mastform_miEditBrowse;
        laApplicationName.Caption := laApplicationName.Caption + mastform_BrowseOnly;
      end;
  fmStartInfo.Hide;
end;

procedure TfmMaster.miAboutClick(Sender: TObject);
begin
  MessageDlg('Designed and built by Robert M. Larame' + chr(10) +
             'Geological Survey of Canada, 2001-2005',mtInformation,[mbOK],0);
end;

procedure TfmMaster.FormDestroy(Sender: TObject);
begin
  dmGlobal.Free;
  DpStd.Free;
  DpTSNames.Free;
  DgStd.Free;
  DgTSNames.Free;
  DpSpecs.Free;
  DgSpecs.Free;
end;

procedure TfmMaster.miMineClick(Sender: TObject);
begin
  miMine.Enabled := False;
  sbMine.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmMine:= TfmMine.Create(nil);
  fmSelectBySQL := TfmSelectBySQL.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  try
    fmMine.Top := fmMaster.Top;
    fmMine.Left := fmMaster.Left;
    fmMine.Height := fmMaster.Height;
    fmMine.Width := fmMaster.Width;
    fmMine.ShowModal;
  finally
    Application.OnHint := nil;
    fmMine.Release;
    fmSelectBySQL.Release;
    fmSelectRef.Release;
  end;
end;

procedure TfmMaster.miProductionClick(Sender: TObject);
begin
  miProduction.Enabled := False;
  sbProduction.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmProduction := TfmProduction.Create(nil);
  fmSelectBySQL := TfmSelectBySQL.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  try
    fmProduction.Top := fmMaster.Top;
    fmProduction.Left := fmMaster.Left;
    fmProduction.Height := fmMaster.Height;
    fmProduction.Width := fmMaster.Width;
    fmProduction.ShowModal;
  finally
    Application.OnHint := nil;
    fmProduction.Release;
    fmSelectBySQL.Release;
    fmSelectRef.Release;
  end;
end;

procedure TfmMaster.miResourceClick(Sender: TObject);
begin
  miResource.Enabled := False;
  sbResource.Enabled := False;
  fmStartInfo.Label1.Caption := mastform_BuildingForm;
  fmStartInfo.pbInfo.Position := 0;
  fmStartInfo.Show;
  fmMaster.Update;
  fmStartInfo.Update;
  fmResource := TfmResource.Create(nil);
  fmSelectBySQL := TfmSelectBySQL.Create(nil);
  fmSelectRef := TfmSelectRef.Create(nil);
  try
    fmResource.Top := fmMaster.Top;
    fmResource.Left := fmMaster.Left;
    fmResource.Height := fmMaster.Height;
    fmResource.Width := fmMaster.Width;
    fmResource.ShowModal;
  finally
    Application.OnHint := nil;
    fmResource.Release;
    fmSelectBySQL.Release;
    fmSelectRef.Release;
  end;
end;

procedure TfmMaster.miHintsClick(Sender: TObject);
begin
  SaveHintsToFile;
end;

procedure TfmMaster.miRefreshSummaryTablesClick(Sender: TObject);
var
  tmpCursor: TCursor;
begin
  if MessageDlg(mastform_BuildRebuildSummary,mtConfirmation,[mbOK,mbCancel],0) <> mrOK
    then Exit;
  if dmRefreshSummaryTables = nil
    then dmRefreshSummaryTables := TdmRefreshSummaryTables.Create(nil);
  try
    with dmRefreshSummaryTables do
      begin
        Connection1.ConnectionString := 'File Name=' + DatabaseUDL;
        TableList := TStringList.Create;
        Connection1.Open;
        tmpCursor := Screen.Cursor;
        Screen.Cursor := crSQLWait;
        Command1.CommandType := cmdText;
        try
          Connection1.GetTableNames(TableList);
          PopulateRecordCountTable;
          PopulateSummaryTables;
        finally
          Screen.Cursor := tmpCursor;
          TableList.Free;
          Connection1.Close;
        end;
      end;
  finally
    FreeAndNil(dmRefreshSummaryTables);
  end;
  ShowMessage(mastform_SummaryRebuilt);
end;

initialization
  UserInitials := '';
  repeat
    if not InputQuery(mastform_Initialization1,mastform_Initialization2,UserInitials)
      then UserInitials := 'ZZZ';
    if UserInitials = ''
      then ShowMessage(mastform_NoInitials);
  until UserInitials <> '';
  if UserInitials = 'ZZZ'
    then BrowseEdit := False
    else BrowseEdit := True;
{
      if MessageDlg(mastform_BrowseModeOnly,mtConfirmation,[mbYes,mbNo],0) = mrYes
        then BrowseEdit := False
        else BrowseEdit := True;
}
  fmStartInfo := TfmStartInfo.Create(nil);

finalization
  fmStartInfo.Release;

end.
