unit selnts;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, DBGrids, ComCtrls, Buttons, StdCtrls, ExtCtrls, DB;

type
  TfmSelNTS = class(TForm)
    paTop: TPanel;
    paSelect: TPanel;
    buSelect: TButton;
    paCancel: TPanel;
    buCancel: TButton;
    paSearch: TPanel;
    Label1: TLabel;
    sbNTSNAME: TSpeedButton;
    sbNTS: TSpeedButton;
    paGrid: TPanel;
    Rolodex: TTabControl;
    DBGrid1: TDBGrid;
    sbMAPID: TSpeedButton;
    Panel1: TPanel;
    Label2: TLabel;
    edMapId: TEdit;
    procedure FormActivate(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buSelectClick(Sender: TObject);
    procedure RolodexMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure sbNTSClick(Sender: TObject);
    procedure sbMAPIDClick(Sender: TObject);
    procedure sbNTSNAMEClick(Sender: TObject);
    procedure edMapIdChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure SetSelectedValues;
  public
    { Public declarations }
  end;

var
  fmSelNTS: TfmSelNTS;
  LatDD: double;
  LongDD: double;
  MAPID: string;
  NTSP: string;
  NTSS: string;
  NTST: string;
  QUADRANT: string;
  NTSNAME: string;
  SortField: string;

implementation

uses dmglob1, resources;

{$R *.dfm}

procedure TfmSelNTS.SetSelectedValues;
begin
  with dmGlobal.FindNTS do
    begin
      MAPID := FieldbyName('MAPID').AsString;
      NTSP := FieldbyName('NTSP').AsString;
      NTSS := FieldbyName('NTSS').AsString;
      NTST := FieldbyName('NTST').AsString;
      NTSNAME := FieldbyName('NTSNAME').AsString;
    end;
end;

procedure TfmSelNTS.FormActivate(Sender: TObject);
var
  s: string;
begin
  MAPID := '';
  edMapId.Clear;
  buCancel.SetFocus;
  sbMAPID.Down := True;
  with dmGlobal.FindNTS do
    begin
      SortField := 'MAPID';
      Active := False;
      SQL[2] := 'order by MAPID';
      Active := True;
      First;
      Rolodex.Enabled := False;
    end;
  LongDD := -LongDD; // West longitudes are negative
  if (LatDD <> 0.0) and (LongDD <> 0.0) then
    s := dmGlobal.CalculateNTS(LatDD,LongDD);
  if (LatDD = 0.0) or (LongDD = 0.0) or (s = '') then
    MessageDlg(selnts_NoNTSCalculated,mtInformation,[mbOK],0);
  edMapId.Text := s;
end;

procedure TfmSelNTS.buCancelClick(Sender: TObject);
begin
  MAPID := '';
  Close;
end;

procedure TfmSelNTS.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  dmGlobal.FindNTS.Active := False;
  sbMAPID.Down := False;
  sbNTS.Down := False;
  sbNTSNAME.Down := False;
end;

procedure TfmSelNTS.buSelectClick(Sender: TObject);
begin
  SetSelectedValues;
  Close;
end;

procedure TfmSelNTS.RolodexMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  FirstLetter : Char;
  FirstRec,LastRec,MidRec : longint;
begin
  with dmGlobal.FindNTS do
    begin
      FirstLetter := chr(65 + Rolodex.TabIndex);
      DisableControls;
      FirstRec := 1;
      LastRec := RecordCount;
      while LastRec > FirstRec do
        begin
          First;
          MidRec := (FirstRec + LastRec) div 2;
          MoveBy(MidRec-1);
          if AnsiUpperCase(FieldByName(SortField).AsString) > FirstLetter
            then LastRec := Midrec
            else FirstRec := Midrec + 1;
        end;
      if AnsiUpperCase(FieldByName(SortField).AsString) < FirstLetter then Next;
      EnableControls;
    end;
end;

procedure TfmSelNTS.sbNTSClick(Sender: TObject);
begin
  with dmGlobal.FindNTS do
    if sbNTS.Down
      then
        begin
          SortField := 'NTSP,NTSS,NTST';
          Active := False;
          SQL[2] := 'order by ' + SortField;
          Active := True;
          First;
          Rolodex.Enabled := False;
          Rolodex.TabIndex := 0;
        end
      else Active := False;
end;

procedure TfmSelNTS.sbMAPIDClick(Sender: TObject);
begin
  with dmGlobal.FindNTS do
    if sbMAPID.Down
      then
        begin
          SortField := 'MAPID';
          Active := False;
          SQL[2] := 'order by ' + SortField;
          Active := True;
          First;
          Rolodex.Enabled := False;
          Rolodex.TabIndex := 0;
        end
      else Active := False;
end;

procedure TfmSelNTS.sbNTSNAMEClick(Sender: TObject);
begin
  with dmGlobal.FindNTS do
    if sbNTSNAME.Down
      then
        begin
          SortField := 'NTSNAME';
          Active := False;
          SQL[2] := 'order by ' + SortField;
          Active := True;
          First;
          Rolodex.Enabled := True;
          Rolodex.TabIndex := 0;
        end
      else
        begin
          Active := False;
          Rolodex.Enabled := False;
        end;
end;

procedure TfmSelNTS.edMapIdChange(Sender: TObject);
begin
  with dmGlobal.FindNTS do
    if Active then
      if not Locate('MAPID',edMapId.Text,[loCaseInsensitive,loPartialKey]) then
        MessageDlg(selnts_NTSNotFound,mtInformation,[mbOK],0);
end;

procedure TfmSelNTS.FormCreate(Sender: TObject);
begin
  Self.Caption := selnts_fmSelNTS;
  buSelect.Caption := selnts_buSelect;
  Label1.Caption := selnts_Label1;
  sbMAPID.Caption := selnts_sbMAPID;
  sbNTS.Caption := selnts_sbNTS;
  sbNTSNAME.Caption := selnts_sbNTSNAME;
  Label2.Caption := selnts_Label2;
  buCancel.Caption := selnts_buCancel;
end;

end.
