unit selref;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Grids, DBGrids, StdCtrls, ExtCtrls, Buttons, DB;

type
  TfmSelectRef = class(TForm)
    paTop: TPanel;
    paSelect: TPanel;
    buSelect: TButton;
    paCancel: TPanel;
    buCancel: TButton;
    paSearch: TPanel;
    Label1: TLabel;
    sbByTitle: TSpeedButton;
    sbByAuthor: TSpeedButton;
    paFiller: TPanel;
    paGrid: TPanel;
    Rolodex: TTabControl;
    DBGrid1: TDBGrid;
    edStringSearch: TEdit;
    buStringSearch: TButton;
    buStringSearchNext: TButton;
    procedure DBGrid1DblClick(Sender: TObject);
    procedure buSelectClick(Sender: TObject);
    procedure buCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure sbByTitleClick(Sender: TObject);
    procedure sbByAuthorClick(Sender: TObject);
    procedure RolodexMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormActivate(Sender: TObject);
    procedure buStringSearchClick(Sender: TObject);
    procedure buStringSearchNextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure SetSelectedValues;
    procedure StringSearch(ff : boolean);
  public
    { Public declarations }
  end;

var
  fmSelectRef: TfmSelectRef;
  SelectedRefmain : longint;

implementation

uses dmglob1, resources;

var
  SortField : string;

{$R *.DFM}

procedure TfmSelectRef.SetSelectedValues;
begin
  SelectedRefmain := dmGlobal.FindRef.FieldByName('REFNO').Value;
end;

procedure TfmSelectRef.StringSearch(ff : boolean);
var
  book_m  : TBookmarkStr;
  got_it  : boolean;
  sstring : string;
begin
  got_it := False;
  sstring := AnsiUpperCase(edStringSearch.Text);
  with dmGlobal.FindRef do
    begin
      book_m := Bookmark;
      if ff
        then First
        else Next;
      DisableControls;
      while not (EOF or got_it) do
        begin
          got_it := False;
          if not FieldByName(SortField).IsNull then
            got_it := AnsiPos(sstring,AnsiUpperCase(FieldByName(SortField).Value)) > 0;
          if not got_it then Next;
        end;
      if not got_it then
        begin
          Bookmark := book_m;
          ShowMessage(selref_StringNotFound);
        end;
      EnableControls;
    end;
end;

procedure TfmSelectRef.FormActivate(Sender: TObject);
begin
  SelectedRefmain := 0;
  buCancel.SetFocus;
  sbByTitle.Down := True;
  edStringSearch.Text := '';
  with dmGlobal.FindRef do
    begin
      SortField := 'TITLE';
      Active := False;
      SQL[3] := 'and not (refmain.TITLE is NULL) order by refmain.TITLE';
      Active := True;
      First;
      Rolodex.Enabled := True;
      Rolodex.TabIndex := 0;
    end;
end;

procedure TfmSelectRef.DBGrid1DblClick(Sender: TObject);
begin
  SetSelectedValues;
  dmGlobal.FindRef.Active := False;
  Close;
end;

procedure TfmSelectRef.buSelectClick(Sender: TObject);
begin
  SetSelectedValues;
  dmGlobal.FindRef.Active := False;
  Close;
end;

procedure TfmSelectRef.buCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfmSelectRef.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  dmGlobal.FindRef.Active := False;
  sbByAuthor.Down := False;
  sbByTitle.Down := False;
end;

procedure TfmSelectRef.sbByTitleClick(Sender: TObject);
begin
  with dmGlobal.FindRef do
    if sbByTitle.Down
      then
        begin
          SortField := 'TITLE';
          Active := False;
          SQL[3] := 'and not (refmain.TITLE is NULL) order by refmain.TITLE';
          Active := True;
          First;
          Rolodex.Enabled := True;
          Rolodex.TabIndex := 0;
          edStringSearch.Visible := True;
          buStringSearch.Visible := True;
          buStringSearchNext.Visible := True;
        end
      else
        begin
          Active := False;
          Rolodex.Enabled := False;
          edStringSearch.Visible := False;
          buStringSearch.Visible := False;
          buStringSearchNext.Visible := False;
        end;
end;

procedure TfmSelectRef.sbByAuthorClick(Sender: TObject);
begin
  with dmGlobal.FindRef do
    if sbByAuthor.Down
      then
        begin
          SortField := 'LASTNAME';
          Active := False;
          SQL[3] := 'order by authors.LASTNAME,authors.FIRSTNAME';
          Active := True;
          First;
          Rolodex.Enabled := True;
          Rolodex.TabIndex := 0;
        end
      else
        begin
          Active := False;
          Rolodex.Enabled := False;
        end;
  edStringSearch.Visible := False;
  buStringSearch.Visible := False;
  buStringSearchNext.Visible := False;
end;

procedure TfmSelectRef.RolodexMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  FirstLetter : Char;
  FirstRec,LastRec,MidRec : longint;
begin
  with dmGlobal.FindRef do
    begin
      FirstLetter := chr(65 + Rolodex.TabIndex);
      DisableControls;
      FirstRec := 1;
      LastRec := RecordCount;
      while LastRec > FirstRec do
        begin
          First;
          MidRec := (FirstRec + LastRec) div 2;
          MoveBy(MidRec-1);
          if AnsiUpperCase(FieldByName(SortField).Value) > FirstLetter
            then LastRec := Midrec
            else FirstRec := Midrec + 1;
        end;
      if AnsiUpperCase(FieldByName(SortField).Value) < FirstLetter then Next;
      EnableControls;
    end;
end;

procedure TfmSelectRef.buStringSearchClick(Sender: TObject);
begin
  StringSearch(True);
end;

procedure TfmSelectRef.buStringSearchNextClick(Sender: TObject);
begin
  StringSearch(False);
end;

procedure TfmSelectRef.FormCreate(Sender: TObject);
begin
  Self.Caption := selref_fmSelectRef;
  buSelect.Caption := selref_buSelect;
  buStringSearch.Caption := selref_buStringSearch;
  buStringSearchNext.Caption := selref_buStringSearchNext;
  buCancel.Caption := selref_buCancel;
  sbByAuthor.Caption := selref_sbByAuthor;
  sbByTitle.Caption := selref_sbByTitle;
  Label1.Caption := selref_Label1;
end;

end.
